<?php

namespace Drupal\miniorange_saml_idp\Controller;

use DOMDocument;
use Drupal\miniorange_saml_idp\MiniorangeSAMLSP;
use Drupal\miniorange_saml_idp\Utilities;
use Drupal\Core\Controller\ControllerBase;
use http\Env\Request;
use \Symfony\Component\HttpFoundation\Cookie;
use Drupal\miniorange_saml_idp\LogoutRequest;
use Drupal\miniorange_saml_idp\XMLSecurityKey;
use Drupal\miniorange_saml_idp\XMLSecurityDSig;
use \Symfony\Component\HttpFoundation\Response;
use Drupal\miniorange_saml_idp\GenerateResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\miniorange_saml_idp\MiniorangeSAMLCustomer;
use Drupal\miniorange_saml_idp\MiniOrangeAuthnRequest;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpSupport;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpConstants;
use Drupal\Component\Utility\Xss;
use \Drupal\user\Entity\User;
Use \Drupal\node\Entity\Node;

class miniorange_saml_idpController extends ControllerBase {

function miniorange_saml_idp_metadata(){
    $download_req = \Drupal::request()->query->get('download');
    $download = isset($download_req) ? $download_req : '';
    self::_generate_metadata($download);
    return new Response();
}

function miniorange_saml_idp_certificate(){
  self::_generate_metadata('certificate');
  return new Response();
}

public static function _generate_metadata($download)
    {
        global $base_url;

        $site_url = $base_url . '/';

        $entity_id = $site_url . '?q=admin/config/people/miniorange_saml_idp/';
        $login_url = $site_url . 'initiatelogon';
         $logout_url = $site_url . 'user/logout';

        define('DRUPAL_BASE_ROOT', dirname(__FILE__));
        $module_path = drupal_get_path('module', 'miniorange_saml_idp');
      $custom_public_certificate = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_publ_certificate');
      if(!empty($custom_public_certificate)){
            $certificate_raw = $custom_public_certificate;
      }else {
            $certificate_raw = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_certificate;

  }
        $certificate = preg_replace("/[\r\n]+/", "", $certificate_raw);
        $certificate = str_replace("-----BEGIN CERTIFICATE-----", "", $certificate);
        $certificate = str_replace("-----END CERTIFICATE-----", "", $certificate);
        $certificate = str_replace(" ", "", $certificate);

        if ($download == 'certificate'){
          header('Content-Disposition: attachment; filename="Certificate.crt"');
          echo xss::filter($certificate_raw);
          exit;
        }

        if ($download) {
            header('Content-Disposition: attachment; filename="Metadata.xml"');
        } else {
            header('Content-Type: text/xml');
        }

        $sign_authentication_request = \Drupal::config('miniorange_saml_idp.settings')->get('mo_saml_idp_sign_authentication_request') == 'signed' ? 'true' : 'false';

        echo '<?xml version="1.0" encoding="UTF-8"?>
<md:EntityDescriptor xmlns:md="urn:oasis:names:tc:SAML:2.0:metadata" entityID="' . xss::filter($entity_id) . '">
    <md:IDPSSODescriptor WantAuthnRequestsSigned="'. $sign_authentication_request .'" protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
        <md:KeyDescriptor use="signing">
            <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
                <ds:X509Data>
                    <ds:X509Certificate>' . xss::filter($certificate) . '</ds:X509Certificate>
                </ds:X509Data>
            </ds:KeyInfo>
        </md:KeyDescriptor>
        <md:SingleLogoutService
      Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
      Location="' . xss::filter($logout_url) . '"/>
    <md:SingleLogoutService
      Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
      Location="' . xss::filter($logout_url) . '"/>
        <md:NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress</md:NameIDFormat>
        <md:NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified</md:NameIDFormat>        <md:SingleSignOnService
      Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
      Location="'. xss::filter($login_url) .'"/>
    <md:SingleSignOnService
      Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
      Location="'. xss::filter($login_url) .'"/>
          </md:IDPSSODescriptor>
</md:EntityDescriptor>';
        exit;
    }

function saml_logout() {
    global $base_url;

    if ( empty($_REQUEST['SAMLRequest'])){
        if ( isset($_COOKIE['slo_initiator']) ) {
            $slo_initiator =  json_decode( $_COOKIE['slo_initiator'],true);
            $saml_response_params = isset( $_COOKIE['response_params'] ) ? json_decode($_COOKIE['response_params'], true) : array();

            if (sizeof($saml_response_params) > 0) {
                foreach ($saml_response_params as $key => $value) {
                    $id = $key;
                    $saml_sp = new MiniorangeSAMLSP($id);

                    $nameid = $saml_sp->getMoIdpNameidFormat();
                    $issuer = $saml_sp->getMoIdpSpIssuer();
                    $logout_url = $saml_sp->getMoIdpSloUrl();
                    if ($logout_url == '') {
                        session_destroy();
                        $response = new RedirectResponse($base_url);
                        $response->send();
                        return new Response();
                    }

                    unset($saml_response_params[$key]);
                    setcookie('response_params',json_encode( $saml_response_params ),time() + 7200, '/');

                    $this->mo_idp_send_logout_request($id, $nameid, $issuer, $logout_url);
                }
            } else {

                //Remove cookie and send logout response to the SLO initiator
                unset($_COOKIE['response_params']);
                setcookie('response_params', '', time() - 3600, '/');
                unset($_COOKIE['slo_initiator']);
                setcookie('slo_initiator', '', time() - 3600, '/');

              $base64EncodedXML = $_SESSION['base64EncodedXML'];
              session_destroy();
              $this->_send_logout_response($base64EncodedXML,$slo_initiator['relayState'],$slo_initiator['destination']);
            }
        }
        else{
            $saml_response_params = isset( $_COOKIE['response_params'] ) ? json_decode( $_COOKIE['response_params'],true) : array();
            foreach ($saml_response_params as $id => $value) {
                $saml_sp = new MiniorangeSAMLSP($id);

                $nameid = $saml_sp->getMoIdpNameidFormat();
                $issuer = $saml_sp->getMoIdpSpIssuer();
                $logout_url = $saml_sp->getMoIdpSloUrl();

                if($logout_url=='') {
                    setcookie('response_params',json_encode( $saml_response_params ),time() - 7200, '/');
                    session_destroy();
                    $response = new RedirectResponse($base_url);
                    $response->send();
                    return new Response();
                }

                unset($saml_response_params[$id]);
                setcookie('response_params',json_encode( $saml_response_params ),time() + 7200, '/');

                $this->mo_idp_send_logout_request($id, $nameid, $issuer, $logout_url);
            }

            //Unset cookie if the cookie is set
            if ( isset($_COOKIE['response_params']) ) {
              unset($_COOKIE['response_params']);
              setcookie('response_params', '', time() - 3600, '/');
            }

            $redirect = $base_url;
            session_destroy();
            $response = new RedirectResponse($redirect);
            $response->send();
            return new Response();
        }
    }

    if(array_key_exists('SAMLRequest', $_REQUEST) && !empty($_REQUEST['SAMLRequest'])){
        $samlRequest = gzinflate(base64_decode($_REQUEST['SAMLRequest']));
        $document = new DOMDocument();
        $document->loadXML($samlRequest);
        $samlRequestXML = $document->firstChild;

        $relay_State = '/';
        if(array_key_exists('RelayState', $_REQUEST)) {
          $relay_state = $_REQUEST['RelayState'];
        }

      if( $samlRequestXML->localName == 'LogoutRequest' ){
        $logoutRequest = new LogoutRequest( $samlRequestXML );
        $logout_request_id = $logoutRequest->getId();
        $issuer = $logoutRequest->getIssuer();

        $saml_sp = new MiniorangeSAMLSP();
        $result = $saml_sp->sqlQueryWithWhereClause('*','mo_idp_sp_issuer',$issuer);

        $destination = $result[0]->mo_idp_slo_url;
        $binding_type = $result[0]->mo_idp_logout_binding_type;
        $signed_slo_response = $result[0]->mo_idp_response_signed;

        if($binding_type == 1){
          $binding_type = "HTTP-POST";
        }else{
          $binding_type = 'HTTP-Redirect';
        }

        $idp_issuer = $base_url . '/?q=admin/config/people/miniorange_saml_idp/';
        $logoutResponse = Utilities::createLogoutResponse($logout_request_id, $idp_issuer, $destination, $binding_type);

        $custom_private_key = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_private_certificate');
        $custom_public_certificate = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_publ_certificate');

        if(!empty($custom_private_key) && !empty($custom_public_certificate)){
          $privateKey = $custom_private_key;
          $publicCert = $custom_public_certificate;
        }else {
          $privateKey = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_key;
          $publicCert = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_certificate;
        }
        $base64EncodedXML = Utilities::signXML($binding_type, $logoutResponse, $publicCert, $privateKey,$signed_slo_response);
      }

        $logoutRequest = new LogoutRequest( $samlRequestXML );
        $issuer = $logoutRequest->getIssuer();

        $saml_sp = new MiniorangeSAMLSP();
        $result = $saml_sp->sqlQueryWithWhereClause('*','mo_idp_sp_issuer',$issuer);
        $id = $result[0]->id;
        $saml_response_params =  json_decode( $_COOKIE['response_params'],true);

        //Setting cookie for SLO initiator
        $slo_initiator = $saml_response_params[$id];
        $slo_initiator['id'] = $id;
        $_SESSION['base64EncodedXML'] = $base64EncodedXML;
        $slo_initiator['destination'] = $destination;

        setcookie('slo_initiator',json_encode( $slo_initiator ),time() + 7200, '/');

        //Unsetting the SLO initiator form the response_params cookie
        unset($saml_response_params[$id]);
        setcookie('response_params',json_encode( $saml_response_params ),time() + 7200, '/');

        foreach ($saml_response_params as $id => $value) {
            $saml_sp = new MiniorangeSAMLSP($id);

            $nameid = $saml_sp->getMoIdpNameidFormat();
            $issuer = $saml_sp->getMoIdpSpIssuer();
            $logout_url = $saml_sp->getMoIdpSloUrl();

            //Unsetting the SLO for the SP from the cookie and sending logout request
            unset($saml_response_params[$id]);
            setcookie('response_params',json_encode( $saml_response_params ),time() + 7200, '/');

            $this->mo_idp_send_logout_request($id, $nameid, $issuer, $logout_url);
        }
        $redirect = $base_url . '/user/logout/';
        $response = new RedirectResponse($redirect);
        $response->send();
        return new Response();
    }
    elseif (array_key_exists('SAMLResponse', $_REQUEST) && !empty($_REQUEST['SAMLResponse'])){
        $this->_read_saml_response($_REQUEST,\Drupal::request()->query->getIterator()->getArrayCopy());
    }
    return new Response();
}



function mo_idp_send_logout_request($id,$nameid, $issuer, $logout_url){
    global $base_url;
    $relayState = $base_url;
    $saml_sp = new MiniorangeSAMLSP($id);
    $logout_binding_type = $saml_sp->getMoIdpLogoutBindingType();
    if($logout_binding_type == 1){
      $logout_binding_type = "HTTP-POST";
    }else{
      $logout_binding_type = 'HTTP-Redirect';
    }

    if($logout_binding_type == 'HTTP-POST') {
        $samlRequest = Utilities::createLogoutRequest($nameid, '', $issuer, $logout_url, 'HTTP-POST');

        $custom_private_key = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_private_certificate');
        $custom_public_certificate = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_publ_certificate');
        if(!empty($custom_private_key) && !empty($custom_public_certificate)){
              $privateKey = $custom_private_key;
              $publicCert = $custom_public_certificate;
        }else {
              $privateKey = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_key;
              $publicCert = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_certificate;
        }

        $base64EncodedXML = Utilities::signXML($logout_binding_type, $samlRequest, $publicCert, $privateKey);
        $this->_send_logout_request($base64EncodedXML, $relayState, $logout_url);
        return;
    }else {

        $samlRequest = Utilities::createLogoutRequest($nameid, '', $issuer, $logout_url, 'HTTP-Redirect');
        $samlRequest = 'SAMLRequest=' . $samlRequest . '&RelayState=' . urlencode($relayState) . '&SigAlg='. urlencode(XMLSecurityKey::RSA_SHA256);
        $param =array( 'type' => 'private');
        $key = new XMLSecurityKey(XMLSecurityKey::RSA_SHA256, $param);

        $custom_private_key = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_private_certificate');
        if(!empty($custom_private_key)){
          $private_key = $custom_private_key;
        }else {
          $private_key = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_key;
        }

        $key->loadKey($private_key);
        $objXmlSecDSig = new XMLSecurityDSig();
        $signature = $key->signData($samlRequest);
        $signature = base64_encode($signature);
        $redirect = $logout_url;
        if (strpos($logout_url,'?') !== false) {
            $redirect .= '&';
        } else {
            $redirect .= '?';
        }
        $redirect .= $samlRequest . '&Signature=' . urlencode($signature);

        $response = new RedirectResponse($redirect);
        $response->send();
        return new Response();
    }
}

function _read_samllogout_request($REQUEST,$GET){
    global $base_url;
    $samlRequest = $REQUEST['SAMLRequest'];
    $relay_State = '/';
    if(array_key_exists('RelayState', $REQUEST)) {
        $relay_state = $REQUEST['RelayState'];
    }

    $samlRequest = base64_decode($samlRequest);
    if(array_key_exists('SAMLRequest', $GET) && !empty($GET['SAMLRequest'])) {
        $samlRequest = gzinflate($samlRequest);
    }

    $sigAlg = array_key_exists('SigAlg',$REQUEST) ? $REQUEST['SigAlg'] : null;
    $signature = array_key_exists('Signature', $REQUEST) ? $REQUEST['Signature'] : null;

    $document = new DOMDocument();
    $document->loadXML($samlRequest);
    $samlRequestXML = $document->firstChild;

    if( $samlRequestXML->localName == 'LogoutRequest' ){
            $logoutRequest = new LogoutRequest( $samlRequestXML );
            $logout_request_id = $logoutRequest->getId();
            $issuer = $logoutRequest->getIssuer();

            $saml_sp = new MiniorangeSAMLSP();
            $result = $saml_sp->sqlQueryWithWhereClause('*','mo_idp_sp_issuer',$issuer);

            $destination = $result[0]->mo_idp_slo_url;
            $binding_type = $result[0]->mo_idp_logout_binding_type;

            if($binding_type == 1){
                $binding_type = "HTTP-POST";
            }else{
                $binding_type = 'HTTP-Redirect';
            }

            $logoutResponse = Utilities::createLogoutResponse($logout_request_id, $issuer, $destination, $binding_type);

        $custom_private_key = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_private_certificate');
        $custom_public_certificate = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_publ_certificate');

        if(!empty($custom_private_key) && !empty($custom_public_certificate)){
          $privateKey = $custom_private_key;
          $publicCert = $custom_public_certificate;
        }else {
          $privateKey = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_key;
          $publicCert = MiniorangeSAMLIdpConstants::Miniorange_saml_idp_certificate;
        }

      $base64EncodedXML = Utilities::signXML($binding_type, $logoutResponse, $publicCert, $privateKey);
      $this->_send_logout_response($base64EncodedXML, $relay_state, $destination);
    }
}

public function _read_saml_response($REQUEST, $GET) {

    $samlResponse = $REQUEST['SAMLResponse'];
    $relayState = '/';
    if(array_key_exists('RelayState', $REQUEST)) {
      $relayState = $REQUEST['RelayState'];
    }

    $samlResponse = base64_decode($samlResponse);
    if(array_key_exists('SAMLResponse', $GET) && !empty($GET['SAMLResponse'])) {
      $samlResponse = gzinflate($samlResponse);
    }

    $document = new DOMDocument();
    $document->loadXML($samlResponse);
    $samlResponseXML = $document->firstChild;
    if( $samlResponseXML->localName == 'LogoutResponse' ) {
      session_destroy();
      $response = new RedirectResponse($relayState);
      $response->send();
      return new Response();
    }
}

function _send_logout_response($base64EncodedXML, $relayState, $logout_url){
    $response = htmlspecialchars($base64EncodedXML);
    echo '<form id="responseform" action="'. xss::filter($logout_url) .'" method="post">
      <input type="hidden" name="SAMLResponse" value="'. xss::filter($response) .'" />
      <input type="hidden" name="RelayState" value="'. xss::filter($relayState) .'" />
    </form>
    <script>
      document.getElementById("responseform").submit();
    </script>';
        session_destroy();
    exit;
}

function test_configuration() {
    $id = \Drupal::request()->query->get('id');
    $saml_sp = new MiniorangeSAMLSP($id);
    $relayState = '/';

    $acs = $saml_sp->getMoIdpAcsUrl();
    $sp_issuer = $saml_sp->getMoIdpSpIssuer();

    if($acs == '' || is_null($acs) || $sp_issuer == '' || is_null($sp_issuer)){
        \Drupal::messenger()->addMessage(t('Please configure your SP configurations first and then click on Test Configuration.'));
    }

    $this->mo_idp_authorize_user($id,$acs, $sp_issuer, $relayState );
}

function _send_logout_request($base64EncodedXML, $relayState, $logout_url){
    $request = htmlspecialchars($base64EncodedXML);
    echo '<form id="requestform" action="'. xss::filter($logout_url) .'" method="post">
      <input type="hidden" name="SAMLRequest" value="'. xss::filter($request) .'" />
      <input type="hidden" name="RelayState" value="'. xss::filter($relayState) .'" />
    </form>
    <script>
      document.getElementById("requestform").submit();
    </script>';
    session_destroy();
    exit;
}

function miniorange_saml_idp_initiated_login(){
    global $base_url;
    $spName = $_REQUEST['sp'];
    $saml_sp = new MiniorangeSAMLSP();
    $result = $saml_sp->sqlQueryWithWhereClause('*','mo_idp_sp_name',$spName);
    $id = $result[0]->id;

    $configured_sp =$result[0]->mo_idp_sp_name;
    if($spName != $configured_sp){
        $redirect_url = $base_url;
        $response = new RedirectResponse($redirect_url);
        $response->send();
        return new Response();
    }

    $acs_url = $result[0]->mo_idp_acs_url;
    $sp_issuer = $result[0]->mo_idp_sp_issuer;
    $relay_state = isset($_REQUEST['RelayState'])? $_REQUEST['RelayState']: $result[0]->mo_idp_default_relayState;
    $this->mo_idp_authorize_user($id,$acs_url, $sp_issuer, $relay_state);
    return new Response();
}


function miniorange_saml_idp_login_request() {
   if(array_key_exists('SAMLRequest', $_REQUEST) && !empty($_REQUEST['SAMLRequest'])) {
     Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'SAML request received.');
   self::_read_saml_request($_REQUEST,\Drupal::request()->query->getIterator()->getArrayCopy());
    return new Response();
  }
}

function _read_saml_request($REQUEST,$GET) {
  $samlRequest = $REQUEST['SAMLRequest'];
  $relayState = '/';
  if(array_key_exists('RelayState', $REQUEST)) {
    $relayState = $REQUEST['RelayState'];
  }

  $samlRequest = base64_decode($samlRequest);
  if(array_key_exists('SAMLRequest', $GET) && !empty($GET['SAMLRequest'])) {
    $samlRequest = gzinflate($samlRequest);
  }

  $document = new DOMDocument();
  $document->loadXML($samlRequest);
  $samlRequestXML = $document->firstChild;

  $authnRequest = new MiniOrangeAuthnRequest($samlRequestXML);

  $errors = '';
  if(strtotime($authnRequest->getIssueInstant()) > (time() + 60))
    $errors.= '<strong>INVALID_REQUEST: </strong>Request time is greater than the current time.<br/>';
  if($authnRequest->getVersion()!=='2.0')
    $errors.='We only support SAML 2.0! Please send a SAML 2.0 request.<br/>';

  $acs_url_from_request = $authnRequest->getAssertionConsumerServiceURL();
  $sp_issuer_from_request = $authnRequest->getIssuer();

    $saml_sp = new MiniorangeSAMLSP();
    $result = $saml_sp->sqlQueryWithWhereClause('id','mo_idp_sp_issuer',$sp_issuer_from_request);
  $id = $result[0]->id;
  if($id == NULL){
      Utilities::miniornage_show_SAML_msg('Configuration error.','Please configure your SP with the IDP','SP not configured at the IDP');
  }
  $saml_sp = new MiniorangeSAMLSP($id);
  $acs_url = $saml_sp->getMoIdpAcsUrl();
  $sp_issuer = $saml_sp->getMoIdpSpIssuer();

  if(empty($acs_url) || empty($sp_issuer)){
    $errors.= '<strong>INVALID_SP: </strong>Service Provider is not configured. Please configure your Service Provider.<br/>';
  }else{
    if(strcmp($acs_url,$acs_url_from_request) !== 0 ){
      $errors.= '<strong>INVALID_ACS: </strong>Invalid ACS URL!. Please check your Service Provider Configurations.<br/>';
  }
  if(strcmp($sp_issuer,$sp_issuer_from_request) !== 0){
      $errors.='<strong>INVALID_ISSUER: </strong>Invalid Issuer! Please check your configuration.<br/>';
  }
  }

  $inResponseTo = $authnRequest->getRequestID();

    if(empty($errors)){
        $signatureData = Utilities::validateElement($samlRequestXML);
        $spCertificate = $saml_sp->getMoIdpCert();

        $spCertificate = XMLSecurityKey::getRawThumbprint($spCertificate);

        /* remove whitespaces */
        $spCertificate = preg_replace('/\s+/', '', $spCertificate);

        /* Validate signature */
        if($signatureData !== FALSE) {
            $validSignature = Utilities::processRequest($spCertificate, $signatureData);
            if($validSignature === FALSE) {
                $errors.='<strong>INVALID_SIGNATURE: </strong>Invalid Signature!.<br/>';
                exit;
            }
        }
    }

  if(empty($errors)){
  $module_path = drupal_get_path('module', 'miniorange_saml_idp');
  ?>
  <div style="vertical-align:center;text-align:center;width:100%;font-size:25px;background-color:white;">
    <img src="<?php echo xss::filter($module_path);?>/includes/images/loader_gif.gif"></img>
    <h3>PROCESSING...PLEASE WAIT!</h3>
  </div>
  <?php
    Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'Complted reading SAML request.');
  $this->mo_idp_authorize_user($id,$acs_url_from_request, $sp_issuer_from_request, $relayState, $inResponseTo);
  } else{

  echo sprintf(xss::filter($errors));
  exit;
  }
}
  function mo_idp_authorize_user($id,$acs_url,$audience,$relayState,$inResponseTo=null){
    $config = \Drupal::config('miniorange_saml_idp.settings');
    $upgrade_url = $config->get('miniorange_saml_upgrade_url');
    $code = trim( $config->get('miniorange_saml_license_key'));

    if(is_null($upgrade_url) || empty($upgrade_url) || strpos($upgrade_url,'/moas/api/plugin/drupalJoomlaUpdate/') === FALSE){
      $editConfig       = \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings');
      $hostName         = MiniorangeSAMLIdpConstants::BASE_URL;
      $licensePlanName  = Utilities::getSamlRequestPlanName();
      $licenseType      = Utilities::getSamlRequestPlanType();
      $customerId       = $config->get('miniorange_saml_customer_id');
      $apiKey           = $config->get('miniorange_saml_customer_api_key');
      $upgradeUrl       = Utilities::createUpdateUrl($code,$licensePlanName,$licenseType,$apiKey,$customerId,$hostName);
      $editConfig->set('miniorange_oauth_client_upgrade_url',$upgradeUrl)->save();
    }

        $saml_sp = new MiniorangeSAMLSP($id);
        $logout_url = $saml_sp->getMoIdpSloUrl();

        $saml_response_params = array();
        if (isset($_COOKIE['response_params'])) {
            $saml_response_params = json_decode($_COOKIE['response_params'], true);
        }
      Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'Setting cookie for response_parameters.');
        //Setting cookie to handle SSO response and SLO
        $isSLOConfigured = isset($logout_url) && !empty($logout_url);
        $saml_response_params[$id] = array('slo_configured' => $isSLOConfigured,'request_received' => TRUE, 'response_sent' => FALSE, 'moIdpsendResponse' => "true", "acs_url" => $acs_url, "audience" => $audience, "relayState" => $relayState, "inResponseTo" => $inResponseTo);

        setcookie('response_params',json_encode( $saml_response_params ),time() + 7200, '/');

  $user =  \Drupal::currentUser();

  if ( $user->isAuthenticated())
  {
        Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'User already authenticated.');
        $this->mo_idp_send_response($id,$acs_url,$audience,$relayState,$inResponseTo);
  }else {
        Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'User not authenticated, redirecting for authentication.');
        global $base_url;
        $redirect_url = $base_url . '/user/login';
        $response = new RedirectResponse($redirect_url);
        $response->send();
        return new Response();
  }
}


 public static function lime($datediff){

     $exp_li   = \Drupal::config('miniorange_saml_idp.settings')->get('tmp_exp');
     $expire   = \Drupal::config('miniorange_saml_idp.settings')->get('miniOrange_saml_idp_l_exp');
     $admin_el = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
     $datediff = intval($datediff);
     $gprd     = $exp_li+2592000;
     $gprd     = date("Y-m-d", $gprd);
     $exp_li   = date("Y-m-d", $exp_li);

     if (time() > $expire + 1296000)                                 //1296000: 15 days in seconds
     {
          if (!Utilities::licensevalidity($expire)) {

              Utilities::limit($exp_li);                          //deactivated mail
              $connection = \Drupal::database();
              $update  = $connection->update('miniorange_saml_idp_user')
                ->fields(['UserIn' => 0,])
                ->execute();
              Utilities::dellkey();                                                                      //DEACTIVATE PLUGIN
          }
     }

     $exp_date = date("Y-M-d (l) h:i:sa", $expire);

     if($datediff>=16 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_exli_thirtyabc'))){
          $info = Utilities::lexpireinfo($datediff);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exli_thirtyabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
               MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     }else if($datediff<16 && $datediff>5 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_exli_fifteenabc'))){
          $info = Utilities::lexpireinfo($datediff);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exli_fifteenabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
              MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     }else if($datediff<6 && $datediff >0 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_exli_fiveabc'))){
          $info = Utilities::lexpireinfo($datediff);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exli_fiveabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
              MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     }else if($datediff==0 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_exliabc'))){
          $info = Utilities::lexpireinfo($datediff);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exliabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
              MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     }else if($datediff>-6 && $datediff<0 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_grc_fiveabc'))){
          $info = Utilities::gperiod($exp_date);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_grc_fiveabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
              MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     }else if($datediff>-15 && $datediff<-5 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_grc_tenabc'))){
          $info = Utilities::gperiod($exp_date);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_grc_tenabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
              MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     } else if($datediff<-30 && !(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_grc_fifteenabc'))){
          $info = Utilities::grcperiodend($exp_date);
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_grc_fifteenabc', 1)->save();
          if(!Utilities::licensevalidity($expire)){
              MiniorangeSAMLIdpSupport::liexpi($admin_el,$info);
          }
     }
     return;
 }

  public function mo_idp_send_response($id,$acs_url,$audience,$relayState, $inResponseTo=null){
      $exp_li = \Drupal::config('miniorange_saml_idp.settings')->get('tmp_exp');
      $datediff = $exp_li-time();
      $datediff = $datediff / 86400;

      if( $datediff <= 30 ){
          self::lime( $datediff );
      }

      $user = \Drupal::currentUser();

      $email = $user->getEmail();
      $username = $user->getAccountName();
      $current_user_roles = $user->getRoles();
      global $base_url;
      $issuer = $base_url . '/?q=admin/config/people/miniorange_saml_idp/';

      $saml_sp = new MiniorangeSAMLSP($id);

      $name_id_attr = ($saml_sp->getMoIdpNameidAttrMap() == '' )? 'emailAddress' : $saml_sp->getMoIdpNameidAttrMap();
      $idp_response_signed = $saml_sp->getMoIdpResponseSigned();
      $idp_assertion_signed = $saml_sp->getMoIdpAssertionSigned();
      $idp_encrypted_assertion = $saml_sp->getMoIdpEncryptedAssertion();
      $idp_cert_for_encryption = $saml_sp->getMoIdpCertEncrypt();
      $name_id_attr_format = $saml_sp->getMoIdpNameidFormat();
      $attributes = $this->mo_get_idp_attributes( $id,$user );

      /**
       * License Duplicate Check
       */
      $code = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_original_license_key');
      $time_to_check = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_idp_time');
      $mo_current_time = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_idp_current_time');
      $mo_number_of_days =  \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_idp_number_of_days');

      $mo_drupal_configFactory = \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings');
      $customer = new MiniorangeSAMLCustomer(NULL, NULL, NULL, NULL);

      if(isset( $time_to_check )  && !is_null($mo_number_of_days) && $mo_current_time + 86400 < \Drupal::time()->getCurrentTime()){
        $mo_drupal_configFactory->set('miniorange_saml_idp_current_time', $mo_current_time + 86400 )->save();
        $mo_drupal_configFactory->set('miniorange_saml_idp_number_of_days', $mo_number_of_days - 1 )->save();
      }
      if ( isset( $time_to_check ) && $mo_current_time + 86400 < \Drupal::time()->getCurrentTime() && $mo_number_of_days === 0 ) {
        $response = json_decode( $customer->mo_saml_vl( $code, TRUE, $mo_number_of_days ),TRUE );
      }else {
        $response = json_decode( $customer->mo_saml_vl( $code, FALSE, 0 ),TRUE );
      }

      if ( strcasecmp( $response['status'], 'SUCCESS') == 0 ) {
         if ( isset($response['moConnection']) && strcasecmp( $response['moConnection'], 'FALSE') != 0 ) {
            $mo_drupal_configFactory->clear('miniorange_saml_idp_time')->save();
            $mo_drupal_configFactory->clear('miniorange_saml_idp_current_time')->save();
            $mo_drupal_configFactory->clear('miniorange_saml_idp_number_of_days')->save();
         }
      } else {
          //Duplicate license found
          $dup_li_encountered_time = \Drupal::time()->getCurrentTime() +  ( 5 * 24 * 60 * 60 );
          if ( !isset( $time_to_check ) ) {
            $mo_drupal_configFactory->set('miniorange_saml_idp_time', $dup_li_encountered_time)->save();
            $mo_drupal_configFactory->set('miniorange_saml_idp_current_time', \Drupal::time()->getCurrentTime())->save();
            $mo_drupal_configFactory->set('miniorange_saml_idp_number_of_days', 3 )->save();
          }
          elseif( \Drupal::time()->getCurrentTime() >= $time_to_check ) {
              //Remove the license from the site
            $mo_drupal_configFactory->clear('miniorange_saml_license_key')->save();
              if( in_array('administrator', $current_user_roles ) ) {
                 $msg = 'Please enter a key which has not been used before on any other instance or if you have exausted all your keys then buy more license from Licensing.';
                 Utilities::miniornage_show_SAML_msg( 'License key you have entered has already been used.', $msg);
              }else {
                 Utilities::miniornage_show_SAML_msg( 'We could not sign you in. Please contact your Administrator.', '' );
              }
          }
      }

      $admin_username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');

      $extraDataAttached = \Drupal::moduleHandler()->invokeAll( 'mo_idp_attach_attributes', [$user] );
      $attributes = array_merge($attributes,$extraDataAttached);

      if( $admin_username != null && $admin_username != '' ) {
          $saml_response_obj = new GenerateResponse($email, $username, $acs_url, $issuer, $audience, $inResponseTo, $name_id_attr, $idp_response_signed, $attributes, $name_id_attr_format, $idp_assertion_signed, $idp_cert_for_encryption, $idp_encrypted_assertion);
      }else {
          Utilities::miniornage_show_SAML_msg( 'You are not logged in.', 'Please login first to activate single sign on.', 'Make sure you have logged in/ Register in to module.' );
      }

      $saml_response = $saml_response_obj->createSamlResponse();
      Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'SAML response generated.');
      if (ob_get_length() > 0) {
        ob_clean();
      }

      Utilities::addLogger(__FILE__,__FUNCTION__,__LINE__,'Sending SAML response.');
     self::_send_response($saml_response, $relayState, $acs_url);
      return new Response();
  }

   public function mo_get_idp_attributes( $id,$user)
    {
        $email = $user->getEmail();
        $username = $user->getAccountName();
        $roles = $user->getRoles();
        $rolelist = $roles;
        $attribute = array();

        $saml_sp = new MiniorangeSAMLSP($id);

       $attr1Name = $saml_sp->getMoIdpAttr1Name();
       $attr1Value = $saml_sp->getMoIdpAttr1Value();

        if(!empty($attr1Name) && !empty($attr1Value)) {
            $value = self::mo_get_attribute_value($email, $username, $rolelist, $attr1Value);
            $attribute[$attr1Name] = $value;
        }

       $attr2Name = $saml_sp->getMoIdpAttr2Name();
       $attr2Value = $saml_sp->getMoIdpAttr2Value();

        if(!empty($attr2Name) && !empty($attr2Value)) {
            $value = self::mo_get_attribute_value($email, $username, $rolelist, $attr2Value);
            $attribute[$attr2Name] = $value;
        }

       $attr3Name = $saml_sp->getMoIdpAttr3Name();
       $attr3Value = $saml_sp->getMoIdpAttr3Value();

        if(!empty($attr3Name) && !empty($attr3Value)) {
           $value = self::mo_get_attribute_value($email, $username,  $rolelist, $attr3Value);
           $attribute[$attr3Name] = $value;
        }

       $userProfileAttributes = $saml_sp->getMoIdpUserAttributes();
        if(isset($userProfileAttributes) && !empty($userProfileAttributes))
        {
            $userProfileAttributes = unserialize($userProfileAttributes);
            if (! is_array($userProfileAttributes)){
              $userProfileAttributes = array();
            }
            $usr = User::load($this->currentUser()->id());
            $usrVal =  $usr->toArray();

            foreach($userProfileAttributes as $profileAttribute)
            {
                $user_attr_name = $profileAttribute["attr_name"];
                $user_attr_value = $profileAttribute["attr_value"];

                //Fetching fields with value as key
                if(isset($usrVal[$user_attr_value][0]['value']))
                {
                    $value = $usrVal[$user_attr_value][0]['value'];
                    if(!empty($value))
                    {
                        $attribute[$user_attr_name] = $value;
                    }
                }

                //Fetching fields with target_id as key
                elseif (isset($usrVal[$user_attr_value][0]['target_id']))
                {
                  $target_id = $usrVal[$user_attr_value][0]['target_id'];
                  if (!empty($target_id)) {

                    //current user roles
                    if ($user_attr_value == 'roles') {
                      $attribute[$user_attr_name] = $user->getRoles();
                    }

                    //current user profile picture url
                    elseif ( $user_attr_value == 'user_picture' ){
                      $attribute[$user_attr_name] = $usr->get('user_picture')->entity->url();
                    }

                    //body of the contents
                    else {
                      $content_body = array();
                      foreach ($usrVal[$user_attr_value] as $index => $item) {
                        $node = Node::load($item['target_id']);
                        if (isset($node->toArray()['body'][0]['value']) && !is_null($node->toArray()['body'][0]['value'])) {
                          array_push($content_body, htmlspecialchars( $node->toArray()['body'][0]['value']));
                        }
                      }
                      $attribute[$user_attr_name] = $content_body;
                    }
                  }
                }
            }
        }

       $constAttributes = $saml_sp->getMoIdpConstAttributes();
        if(isset($constAttributes)){
            $constAttributes = unserialize($constAttributes);

            if ( !is_array($constAttributes)){
              $constAttributes = array();
            }
            foreach($constAttributes as $profileAttribute)
            {
              $user_attr_name = $profileAttribute["attr_name"];
              $user_attr_value = $profileAttribute["attr_value"];
              $attribute[$user_attr_name]=$user_attr_value;
            }
        }
        return $attribute;
    }

    Public static function mo_get_attribute_value($email, $username, $roles, $attrValue)
    {
        switch($attrValue) {
            case 'name':
                return $username;
            case 'mail':
                return $email;
            case 'roles':
                return $roles;
            default:
                return '';
        }
    }


    Public static function _send_response($saml_response, $ssoUrl, $acs_url)
    {

          $saml_response = base64_encode($saml_response);
            ?>
        <form id="responseform" action="<?php echo xss::filter($acs_url); ?>" method="post">
            <input type="hidden" name="SAMLResponse" value="<?php echo htmlspecialchars(xss::filter($saml_response));  ?>" />
            <input type="hidden" name="RelayState" value="<?php echo xss::filter($ssoUrl); ?>" />
        </form>
        <script>
            setTimeout(function(){
            document.getElementById('responseform').submit();
            }, 100);
        </script>
        <?php
        exit;
    }

    public static function remove_license()
    {
      $post = \Drupal::request()->request->getIterator()->getArrayCopy();
      if (isset($post['value_check']) && $post['value_check'] == 'True') {
        if (\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_license_key') != NULL) {
          $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
          $phone = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_phone');
          $customer = new MiniorangeSAMLCustomer($username, $phone, NULL, NULL);
          $response = json_decode($customer->update_status());

          if ($response->status == 'SUCCESS') {
            $config = \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings');
            $config->clear('miniorange_saml_customer_admin_email');
            $config->clear('miniorange_saml_customer_admin_phone');
            $config->clear('miniorange_saml_customer_id');
            $config->clear('miniorange_saml_customer_api_key');
            $config->clear('miniorange_saml_license_key');
            $config->set('miniorange_saml_status', 'CUSTOMER_SETUP')->save();
            \Drupal::messenger()->addMessage(t('Your account has been removed successfuly!'), 'status');
            $post['value_check'] = 'False';
          }
        }
      }

      global $base_url;
      $redirect_url = $base_url . '/admin/config/people/miniorange_saml_idp/customer_setup';
      $response = new RedirectResponse($redirect_url);
      $response->send();
      return new Response();
    }
}
