<?php


namespace Drupal\miniorange_saml_idp\Form;

use Drupal\Core\Form\FormBase;
use Drupal\miniorange_saml_idp\MiniorangeSAMLSP;
use Drupal\miniorange_saml_idp\Utilities;
use Symfony\Component\HttpFoundation\RedirectResponse;

class AdvanceSettings extends FormBase
{
  public function getFormId()
  {
    return 'miniorange_saml_idp_advance_settings';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
  {
    $form['markup_start'] = array(
      '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                    <h2>Advance Settings</h2><hr/>',
        '#attached' => array(
          'library' => array(
              "miniorange_saml_idp/miniorange_saml_idp.admin",
          )
        ),
    );

    $disabled = Utilities::isCustomerRegistered($form, $form_state);

    $form['mo_saml_idp_import_export_configurations'] = array(
      '#type' => 'details',
      '#title' => t('Import/Export configurations' ),
      '#open' => TRUE,
      '#attributes' => array( 'style' => 'padding:0% 2%; margin-bottom:2%' )
    );

    $form['mo_saml_idp_import_export_configurations']['markup_1'] = array(
      '#markup' => '<br><div class="mo_saml_highlight_background_note"><p><b>NOTE: </b>This tab will help you to transfer your module configurations when you change your Drupal instance.
                        <br>Example: When you switch from test environment to production.<br>Follow these 3 simple steps to do that:<br>
                        <br><strong>1.</strong> Download module configuration file by clicking on the Download Configuration button given below.
                        <br><strong>2.</strong> Install the module on new Drupal instance.<br><strong>3.</strong> Upload the configuration file in Import module Configurations section.<br>
                        <br><b>And just like that, all your module configurations will be transferred!</b></p></div>',
    );

    $form['mo_saml_idp_import_export_configurations']['markup_top'] = array(
      '#markup' => '<br><br><div class="mo_saml_sp_font_for_sub_heading">Export Configuration</div><hr/><p>
                        Click on the button below to download module configuration.</p>'
    );

    global $base_url;
    $url = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_base_url');
    $b_url = isset($url) && !empty($url)? $url:$base_url;
    $configu_disabled = FALSE;
    if(self::miniorange_saml_is_sp_configured()) {
      $form['mo_saml_idp_import_export_configurations']['miniorange_saml_imo_option_export'] = array(
        '#markup' => '<div class="mo_saml_configure_message">Please <a href="' . $b_url . '/admin/config/people/miniorange_saml_idp/sp_setup">configure the plugin</a> first to export the configurations</div><br>',
      );
      $configu_disabled = TRUE;
    }

    $form['mo_saml_idp_import_export_configurations']['miniorange_saml_imo_option_exists_export'] = array(
      '#type' => 'submit',
      '#value' => t('Download Configuration'),
      '#button_type' => 'primary',
      '#prefix'=>'<td>',
      '#suffix'=>'</td>',
      '#submit' => array('::miniorange_export'),
      '#limit_validation_errors' => array(),
      '#disabled' => $configu_disabled || $disabled,
    );

    $form['mo_saml_idp_import_export_configurations']['markup_import'] = array(
      '#markup' => '<br/><br/><br/><br/><div class="mo_saml_sp_font_for_sub_heading">Import Configuration</div><hr/><p>This tab will help you to<span style="font-weight: bold"> Import your module configurations</span> when you change your Drupal instance.</p>
                <p>choose <b>"json"</b> Extened module configuration file and upload by clicking on the button given below. </p><br/>'
    );

    $form['mo_saml_idp_import_export_configurations']['import_Config_file'] = array(
      '#type' => 'file',
      '#disabled' => $disabled,
    );

    $form['mo_saml_idp_import_export_configurations']['miniorange_saml_idp_import'] = array(
      '#type' => 'submit',
      '#value' => t('Upload'),
      '#submit' => array('::miniorange_import_Config'),
      '#limit_validation_errors' => array(),
      '#disabled' => $disabled,
      '#suffix' => '<br><br>',
    );

    /**
     * Create container to hold @moSAMLCustomCertificate form elements.
     */
      $config   = \Drupal::config('miniorange_saml_idp.settings');
      $disabled = Utilities::isCustomerRegistered($form, $form_state);
      $form['mo_saml_idp_custom_certificate'] = array(
        '#type' => 'details',
        '#title' => t('Custom Certificate ( Add or Generate Custom Certificate ) '),
        '#attributes' => array( 'style' => 'padding:0% 2%; margin-bottom:6%' ),
      );

      $form['mo_saml_idp_custom_certificate']['markup_top'] = array(
        '#markup' =>t('<br><div class="mo_saml_idp_font_SP_setup_for_heading">You can add or generate custom certificate from here.</div><br><div class="mo_saml_sp_font_for_sub_heading">Add Custom Certificate</div><hr><br>'),
      );

      $form['mo_saml_idp_custom_certificate']['miniorange_saml_private_certificate'] = array(
        '#type' => 'textarea',
        '#title' => t('X.509 Private Certificate'),
        '#disabled' => $disabled,
        '#attributes' => array(
          'style' => 'width:80%;',
        ),
        '#default_value'  => $config->get('miniorange_saml_private_certificate'),
        '#description'=> t('<strong>Note :</strong> Format of the Private key:<br /><strong>-----BEGIN PRIVATE KEY-----<br />
                    XXXXXXXXXXXXXXXXXXXXXXXXXXX<br />-----END PRIVATE KEY-----</strong><br /><br />'),
      );

      $form['mo_saml_idp_custom_certificate']['miniorange_saml_publ_certificate'] = array(
        '#type' => 'textarea',
        '#title' => t('X.509 Public Certificate '),
        '#disabled' => $disabled,
        '#attributes' => array(
          'style' => 'width:80%;',
        ),
        '#default_value'  => $config->get('miniorange_saml_publ_certificate'),
        '#description'=> t('<strong>Note :</strong> Format of the certificate:<br><strong>-----BEGIN CERTIFICATE-----<br />
                    XXXXXXXXXXXXXXXXXXXXXXXXXXX<br />-----END CERTIFICATE-----</strong><br><br>'),
      );

      $form['mo_saml_idp_custom_certificate']['save_config_elements'] = array(
        '#type' => 'submit',
        '#name'=>'submit',
        '#button_type' => 'primary',
        '#value' => t('Upload'),
        '#submit' => array('::submitForm'),
        '#limit_validation_errors' => array(array('miniorange_saml_publ_certificate'),array('miniorange_saml_private_certificate')),
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['save_config_elements1'] = array(
        '#type' => 'submit',
        '#value' => t('Reset'),
        '#submit' => array('::miniorange_saml_reset_custom_certificate'),
        '#suffix' => '<br><br><br>',
        '#limit_validation_errors' => array(),
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['generate_certificate_markup'] = array(
        '#markup' =>t('<br><br><div class="mo_saml_sp_font_for_sub_heading">Generate Custom Certificate</div><hr><br>'),
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_country_code_text'] = array(
        '#type' => 'textfield',
        '#title' => t('Country Code:'),
        '#description' => t('<b>Note: </b>Check your country code <a target="_blank" href="https://www.digicert.com/kb/ssl-certificate-country-codes.htm">here</a>.'),
        '#attributes' => array(
          'style' => 'width:80%;',
          'placeholder' => t('Enter Country code ( eg. US, CA, IN etc )')
        ),
        '#default_value' => $config->get('mo_saml_country_code_text'),
        '#required' => TRUE,
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_certificate_state_name'] = array(
        '#type' => 'textfield',
        '#title' => t('State:'),
        '#attributes' => array(
          'style' => 'width:80%; margin-bottom:1.5%;',
          'placeholder' => t('State Name')
        ),
        '#default_value' =>$config->get('mo_saml_certificate_state_name'),
          '#required' => TRUE,
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_certificate_company_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Company:'),
        '#attributes' => array(
          'style' => 'width:80%; margin-bottom:1.5%;',
          'placeholder' => t('Company Name')
        ),
        '#default_value' => $config->get('mo_saml_certificate_company_name'),
          '#required' => TRUE,
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['miniorange_saml_unit_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Unit:'),
        '#attributes' => array(
          'style' => 'width:80%; margin-bottom:1.5%;',
          'placeholder' => t('Unit name')
        ),
        '#default_value' => $config->get('miniorange_saml_unit_name'),
          '#required' => TRUE,
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_certificate_common_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Common:'),
        '#attributes' => array(
          'style' => 'width:80%; margin-bottom:1.5%;',
          'placeholder' => t('Common Name')
        ),
        '#default_value' => $config->get('mo_saml_certificate_common_name'),
          '#required' => TRUE,
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_select_digest_algo'] = array(
        '#type' => 'select',
        '#title' => t('Digest Algorithm:'),
        '#options' => array(
          'sha512' => t('SHA512'),
          'sha384' => t('SHA384'),
          'sha256' => t('SHA256'),
          'sha1' => t('SHA1'),
        ),
        '#default_value' =>$config->get('mo_saml_select_digest_algo'),
        '#attributes' => array('style' => 'width:80%;height:29px;margin-bottom:1.5%;'),
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_select_private_key_bit'] = array(
        '#type' => 'select',
        '#title' => t('Bits to generate the private key:'),
        '#options' => array(
          '2048' => t('2048 bits'),
          '1024' => t('1024 bits'),
        ),
        '#default_value' => $config->get('mo_saml_select_private_key_bit'),
        '#attributes' => array('style' => 'width:80%;height:29px;margin-bottom:1.5%;'),
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['mo_saml_select_valid_days'] = array(
        '#type' => 'select',
        '#title' => t('Valid Days:'),
        '#options' => array(
          '365' => t('365 days'),
          '180' => t('180 days'),
          '90' => t('90 days'),
          '45' => t('45 days'),
          '30' => t('30 days'),
          '15' => t('15 days'),
          '7' => t('7 days'),
        ),
        '#default_value' => $config->get('mo_saml_select_valid_days'),
        '#attributes' => array('style' => 'width:80%;height:29px;margin-bottom:1.5%;'),
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['generate_config_elements'] = array(
        '#type' => 'submit',
        '#button_type' => 'primary',
        '#value' => t('Generate Certificate'),
        '#prefix' => '<br><br>',
        '#submit' => array('::miniorange_saml_generate_custom_certificate'),
        '#disabled' => $disabled,
      );

      $form['mo_saml_idp_custom_certificate']['clear_genrate_certificate_data'] = array(
        '#type' => 'submit',
        '#value' => t('Clear Data'),
        '#suffix' => '<br><br></div>',
        '#submit' => array('::miniorange_saml_clear_genrate_certificate_data'),
        '#disabled' => $disabled,
      );


    Utilities::spConfigGuide( $form, $form_state);

    return $form;
  }


  /*
   * Functions for Import/Export
   * */
    function miniorange_export() {
        $tab_class_name = Utilities::getClassNameForImport_Export();
        $configuration_array = array();
        foreach($tab_class_name as $key => $value) {
            if ($key == 'Service_Provider' || $key == 'Attribute_Mapping'){
                $configuration_array[$key] = self::mo_get_configuration_array_sp($value);
            }
            else {
                $configuration_array[$key] = self::mo_get_configuration_array($value);
            }
        }

        $configuration_array["Version_dependencies"] = self::mo_get_version_informations();
        header("Content-Disposition: attachment; filename = miniorange_saml_config.json");
        echo(json_encode($configuration_array, JSON_PRETTY_PRINT|JSON_UNESCAPED_SLASHES));
        exit;
    }

    function mo_get_configuration_array_sp($class_name) {
        $saml_sp = new MiniorangeSAMLSP();
        $res = $saml_sp->sqlQuery('id');
        $main_array = array();
        foreach ($res as $index => $object) {
            $class_object = Utilities::getVariableArrayForImport_Export($class_name);
            $mo_array = array();
            foreach ($class_object as $key => $value)
            {
                $saml_sp = new MiniorangeSAMLSP();
                $res = $saml_sp->sqlQueryWithWhereClause('*','id', $object->id);
                $mo_option_exists = $res[0]->$value;
                if (!is_null($mo_option_exists))
                {
                    if ($value == 'miniorange_saml_idp_user_attributes' || $value == 'miniorange_saml_idp_const_attributes')
                        $mo_array = self::add_dynamic_user_and_constant_mapping_attributes($mo_array, $mo_option_exists, $value);
                    else
                        $mo_array[$key] = $mo_option_exists;
                }
            }
            $main_array[$res[0]->mo_idp_sp_name] = $mo_array;
        }
        return $main_array;
    }


  function mo_get_configuration_array($class_name) {
    $class_object = Utilities::getVariableArrayForImport_Export($class_name);
    $mo_array = array();
    foreach($class_object as $key => $value) {
      $mo_option_exists = \Drupal::config('miniorange_saml_idp.settings')->get($value);
      if( $mo_option_exists ) {
        if ($value == 'miniorange_saml_idp_user_attributes' || $value == 'miniorange_saml_idp_const_attributes')
          $mo_array = self::add_dynamic_user_and_constant_mapping_attributes($mo_array, $mo_option_exists, $value);
        else
          $mo_array[$key] = $mo_option_exists;
      }
    }
    return $mo_array;
  }

  function add_dynamic_user_and_constant_mapping_attributes($mo_array, $attributes, $value){
    $attributes = unserialize($attributes);
    $prefix = ($value == 'miniorange_saml_idp_user_attributes') ? 'User_Profile_Attibute_' : 'Constant_Attribute_';
    $i = 1;
    foreach ($attributes as $attribute) {
      $mo_array[$prefix . 'Name_' . $i] = $attribute['attr_name'];
      $mo_array[$prefix . 'Value_' . $i] = $attribute['attr_value'];
      $i++;
    }
    return $mo_array;
  }


  function miniorange_import_Config() {
    if(! empty( $_FILES['files']['tmp_name']['import_Config_file'] )) {
      $file = $_FILES['files']['name']['import_Config_file'];
      list($name_without_extention, $extention) = explode('.', $file);
      if( $extention == 'json' ) {
        $file = @file_get_contents($_FILES['files']['tmp_name']['import_Config_file']);  //Array -> Sub-Array -> Field
        $configuration_array = json_decode($file, true);
        self::mo_update_configuration_array($configuration_array);
      }else {
        $this->messenger()->addError($this->t('<b style="color: red">File type is not compatible</b> <br> Please Select <b style="color: red">".json"</b> extended file to upload Configuration!'));
      }
    }else {
      $this->messenger()->addError($this->t('<b style="color: red">Please select file first to upload Configuration!</b>'));
    }
  }

  function mo_update_configuration_array( $configuration_array )
  {
    $tab_class_name = Utilities::getClassNameForImport_Export();
    $id_store = array();
    $sp_count = 0;

    $SP_added = 0;
    $show_limit_reached_error = 0;
    foreach ($tab_class_name as $tab_name => $class_name) {
      $class_object = Utilities::getVariableArrayForImport_Export($class_name);
      if ($class_name == 'mo_options_enum_attribute_mapping') {
        $user_profile_attributes = array();
        $constant_attributes = array();
      }

      foreach ($configuration_array[$tab_name] as $key => $value) {
        $noOfSP = Utilities::getNumberOfSP();

        $saml_sp = new MiniorangeSAMLSP();
        $spCount = $saml_sp->spCount();

        if ($tab_name == 'Service_Provider') {
          //Check for number of SP configuration allowed
          if ($spCount >= $noOfSP) {
            $show_limit_reached_error = 1;
            break;
          }

          $config = array();
          foreach ($class_object as $key1 => $value1) {
            array_push($config, isset($value[$key1]) ? $value[$key1] : '');
          }
          $saml_sp = new MiniorangeSAMLSP();

          //$id is set to false if SP with same Name or Entity-id/Issuer already exists.
          $id = call_user_func_array(array($saml_sp, 'insert'), $config);

          array_push($id_store, $id);

          $SP_added += 1;
        } elseif ($tab_name == 'Attribute_Mapping') {
          if ($SP_added <= 0) {
            break;
          }

          //Checking if SP configuration with same SP Name or SP Entity-id/Issuer already exists
          if ($id_store[$sp_count] !== false) {
            $config = array();
            foreach ($class_object as $key1 => $value1) {
              array_push($config, isset($value[$key1]) ? $value[$key1] : '');
            }
            $saml_sp = new MiniorangeSAMLSP($id_store[$sp_count]);
            call_user_func_array(array($saml_sp, 'updateMapping'), $config);
          }
          $sp_count += 1;
          $SP_added -= 1;
        }
        else {
            $database_table_key = $class_object[$key];
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set($database_table_key, $value)->save();
        }
      }
    }

    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_config_status', 'CONFIGURATION_SAVED')->save();

    global $base_url;
    if ($show_limit_reached_error == 1) {
      $idp_setup_tab_path = $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup';
      $this->messenger()->addError($this->t('You have exceeded the limit of Service Providers you can configure. Go to <a href="' . $idp_setup_tab_path . '">Identity Provider Setup Tab</a> to see updated list of IDP\'s'));
    } else {
      $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/idp_setup');
      $response->send();
    }
  }
  function miniorange_saml_is_sp_configured()
  {
      $saml_sp = new MiniorangeSAMLSP();
      $sp_count = $saml_sp->spCount();

      if ($sp_count < 1){
          return TRUE;
        } else {
          return FALSE;
        }
  }
  function mo_get_version_informations()
  {
    $array_version = array();
    $array_version["PHP_version"] = phpversion();
    $array_version["Drupal_version"] = \DRUPAL::VERSION;
    $array_version["OPEN_SSL"] = self::mo_saml_is_openssl_installed();
    $array_version["CURL"] = self::mo_saml_is_curl_installed();
    $array_version["ICONV"] = self::mo_saml_is_iconv_installed();
    $array_version["DOM"] = self::mo_saml_is_dom_installed();
    return $array_version;
  }

  function mo_saml_is_openssl_installed() {
    if ( in_array('openssl', get_loaded_extensions())) {
      return 1;
    }else {
      return 0;
    }
  }

  function mo_saml_is_curl_installed() {
    if ( in_array( 'curl', get_loaded_extensions() ) ) {
      return 1;
    } else {
      return 0;
    }
  }

  function mo_saml_is_iconv_installed() {
    if ( in_array( 'iconv', get_loaded_extensions() ) ) {
      return 1;
    } else {
      return 0;
    }
  }

  function mo_saml_is_dom_installed() {
    if ( in_array( 'dom', get_loaded_extensions() ) ) {
      return 1;
    } else {
      return 0;
    }
  }

  /*
   * Functions of custom certificate
   * */

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state)
  {
      $form_values = $form_state->getValues();
    $public_cert_value = $form_values['miniorange_saml_publ_certificate'];
    $private_cert_value = $form_values['miniorange_saml_private_certificate'];

    if( empty( $public_cert_value ) || empty( $private_cert_value ) ){
      \Drupal::messenger()->addMessage(t('The <b><u>X.509 Private Certificate</u></b> and <b><u>X.509 Public Certificate</u></b> fields are mandatory.'), 'error');
      return;
    }

    $public_cert = Utilities::sanitize_certificate( $public_cert_value );
    $private_cert = Utilities::sanitize_private_key( $private_cert_value );

    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_publ_certificate', $public_cert)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_private_certificate', $private_cert)->save();
    \Drupal::messenger()->addMessage(t('Custom Certificate Uploaded successfully.'));
  }

  function miniorange_saml_reset_custom_certificate($form, &$form_state) {
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_publ_certificate')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_private_certificate')->save();
    \Drupal::messenger()->addMessage(t('Custom Certificate Removed successfully. Please update your Service Provider configuration with new metadata file.'));
  }

  function miniorange_saml_generate_custom_certificate($form, &$form_state) {
      $form_values = $form_state->getValues();
    $country_code = $form_values['mo_saml_country_code_text'];
    $state_name = $form_values['mo_saml_certificate_state_name'];
    $company_name = $form_values['mo_saml_certificate_company_name'];
    $unit_name = $form_values['miniorange_saml_unit_name'];
    $common_name = $form_values['mo_saml_certificate_common_name'];
    $digest_algo = $form_values['mo_saml_select_digest_algo'];
    $private_key_bits = $form_values['mo_saml_select_private_key_bit'];
    $valid_days = $form_values['mo_saml_select_valid_days'];

    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_country_code_text', $country_code)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_certificate_state_name', $state_name)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_certificate_company_name', $company_name)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_unit_name', $unit_name)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_certificate_common_name', $common_name)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_select_digest_algo', $digest_algo)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_select_private_key_bit', $private_key_bits)->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_select_valid_days', $valid_days)->save();

    $RegisteredEmail = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');

    $dn = array(
      "countryName" => $country_code,
      "stateOrProvinceName" => $state_name,
      "organizationName" => $company_name,
      "organizationalUnitName" => $unit_name,
      "commonName" => $common_name,
      "emailAddress" => $RegisteredEmail,
    );

    $successful = Utilities::generateCertificate( $dn, $digest_algo, $private_key_bits, $valid_days );

    if ($successful)
      $this->messenger()->addMessage($this->t('Successfully genertated new certificates. Please update your Service Provider configuration with new metadata file.'));
  }

  function miniorange_saml_clear_genrate_certificate_data(){

    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_country_code_text')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_certificate_state_name')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_certificate_company_name')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_unit_name')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_certificate_common_name')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_select_digest_algo')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_select_private_key_bit')->save();
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('mo_saml_select_valid_days')->save();
    $this->messenger()->addMessage($this->t('Generate Custom Certificate data cleared successfully.'));
  }
}
