<?php
/**
 * @file
 * Contains Attribute and Role Mapping for miniOrange SAML Login Module.
 */

/**
 * Showing Settings form.
 */
namespace Drupal\miniorange_saml_idp\Form;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Render\Markup;
use Drupal\miniorange_saml_idp\Utilities;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\field\FieldConfigInterface;
use Drupal\miniorange_saml_idp\MiniorangeSAMLSP;

class Mapping extends FormBase {

    public function getFormId() {
        return 'miniorange_saml_mapping';
    }


    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
    {
        global $base_url;
        $form['markup_start'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                    <h3>Attribute Mapping (Optional)</h3><hr><br>',
            '#attached' => array(
                'library' => array(
                    "miniorange_saml_idp/miniorange_saml_idp.admin",
                )
            ),
        );

        $disabled = Utilities::isCustomerRegistered($form, $form_state);

        $id_req = \Drupal::request()->query->get('id');
        if (!isset($id_req))
        {
            $form['mo_saml_idp_splist'] = array(
                '#type' => 'fieldset',
                '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%;' ),
            );


            $header = array(
                'sp_name' => array(
                    'data' => t('SP Name')
                ),

                'edit' => array(
                    'data' => t('Edit')
                ),
            );

            $table_content = array();
            $saml_sp = new MiniorangeSAMLSP();
            $data = $saml_sp->getTable();

            $form['mo_saml_idp_splist']['libarary'] = array(
                '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.test',),
            );


            foreach ($data as $sp_data)
            {
                $table_content[$sp_data->id] = array(
                    'sp_name' => $sp_data->mo_idp_sp_name,
                    'test' => Markup::create('<a href="'. $base_url . '/admin/config/people/miniorange_saml_idp/Mapping/?id=' . $sp_data->id .'" >Edit</a>'),
                );
            }

            $form['mo_saml_idp_splist']['mo_saml_idp_splist_table'] = array(
                '#type' => 'table',
                '#header' => $header,
                '#rows' => $table_content,
                '#empty' => t('<b>You have not configured any SP yet, Please Add SP in "<a href="'. $base_url .'/admin/config/people/miniorange_saml_idp/idp_setup">Identity Provider Setup</a>" Tab</b>'),
                '#disabled' => $disabled,
                '#suffix' => '</div>',
            );

            Utilities::spConfigGuide( $form, $form_state);

            return $form;
        }

        $saml_sp = new MiniorangeSAMLSP($id_req);

        $form['mo_saml_attribute_mapping'] = array(
            '#type' => 'fieldset',
            '#attributes' => array( 'style' => 'padding:2% 2% 4%; margin-bottom:2%' ),
        );

        $form['mo_saml_attribute_mapping']['back_button'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => $this->t('Back'),
            '#submit' => array('::miniorange_saml_back_to_mapping_list'),
            '#attributes' => array('style' => 'float:right'),
            '#suffix' => '<br><br>'
        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_nameid_attr_map'] = array(
            '#type' => 'select',
            '#title' => t('NameID Attribute:'),
            '#options' => array(
                '' => t('Select a NameID attribute value to be sent in the SAML Response'),
                'emailAddress' => t('Drupal Email Address'),
                'username' => t('Drupal Username'),
            ),
            '#default_value' => $saml_sp->getMoIdpNameidAttrMap(),
            '#attributes' => array('style' => 'width:700px'),
            '#disabled' => $disabled,
            '#prefix' => '<br>'
        );

        $form['mo_saml_attribute_mapping']['markup_idp_sp_note'] = array(
            '#markup' => '<div class = "mo_saml_highlight_background_note" span style="color:red">
                <b>Note:</b></divspan> This attribute value is sent in SAML Response. Users in your Service Provider
         will be searched (existing users) or created (new users) based on this attribute.
         Use <b>EmailAddress</b> by default.</b></div></br><h3>Attribute Statement(Optional)</h3>
         <div><div class="mo_idp_attr_map_sp_name">',

        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr1_name'] = array(
            '#type' => 'textfield',
            '#title' => t('Attribute Name 1'),
            '#default_value' => $saml_sp->getMoIdpAttr1Name(),
            '#attributes' => array('placeholder' => t('Enter Attribute Name'),'class' => array('"mo_idp_attr_map_sp_name"'),),
            '#required' => FALSE,
            '#disabled' => $disabled,
            '#suffix' => '</div><div class="mo_idp_attr_map_idp_name">',
        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr1_value'] = array(
            '#type' => 'select',
            '#title' => t('Attribute Value'),
            '#options' => array(
                '' => t('Select Attribute Value'),
                'mail' => t('Email Address'),
                'name' => t('Username'),
                'roles' => t('User Roles'),
            ),
            '#attributes' => array('style' => 'width:700px',),
            '#default_value' => $saml_sp->getMoIdpAttr1Value(),
            '#disabled' => $disabled,
            '#suffix' => '</div></div><div><div class="mo_idp_attr_map_sp_name">',

        );


        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr2_name'] = array(
            '#type' => 'textfield',
            '#title' => t('Attribute Name 2'),
            '#default_value' => $saml_sp->getMoIdpAttr2Name(),
            '#attributes' => array('placeholder' => t('Enter Attribute Name')),
            '#required' => FALSE,
            '#disabled' => $disabled,
            '#suffix' => '</div><div class="mo_idp_attr_map_idp_name">',
        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr2_value'] = array(
            '#type' => 'select',
            '#title' => t('Attribute Value'),
            '#options' => array(
                '' => t('Select Attribute Value'),
                'mail' => t('Email Address'),
                'name' => t('Username'),
                'roles' => t('User Roles'),
            ),
            '#attributes' => array('style' => 'width:700px'),
            '#default_value' => $saml_sp->getMoIdpAttr2Value(),
            '#disabled' => $disabled,
            '#suffix' => '</div></div><div><div class="mo_idp_attr_map_sp_name">',
        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr3_name'] = array(
            '#type' => 'textfield',
            '#title' => t('Attribute Name 3'),
            '#default_value' => $saml_sp->getMoIdpAttr3Name(),
            '#attributes' => array('placeholder' => t('Enter Attribute Name')),
            '#required' => FALSE,
            '#disabled' => $disabled,
            '#suffix' => '</div><div class="mo_idp_attr_map_idp_name">',
        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr3_value'] = array(
            '#type' => 'select',
            '#title' => t('Attribute Value'),
            '#options' => array(
                '' => t('Select Attribute Value'),
                'mail' => t('Email Address'),
                'name' => t('Username'),
                'roles' => t('User Roles'),
            ),
            '#attributes' => array('style' => 'width:700px'),
            '#default_value' => $saml_sp->getMoIdpAttr3Value(),
            '#disabled' => $disabled,
            '#suffix' => '</div></div>',
        );


        $custom_fields = Utilities::customUserFields();

        $str = '';
        foreach ($custom_fields as $key => $value) {
            $str .= $value . '=>';
        }

        $custom_field_disabled = TRUE;
        if(count($custom_fields) > 1){
            $custom_field_disabled = FALSE;
        }

        if ( !($custom_field_disabled || $disabled) ){
        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_additional_user_attrs'] = array(
            '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.attributes',),
            '#markup' => '<br /><br><hr><br><br><h3>Additional User Attributes(Optional) &nbsp;&nbsp;&nbsp;&nbsp;
                            <a id="addUserAttr" class="mobtn mo_btn_primary mo_btn_sm" style="padding:6px 12px;" onclick="add_user_attibute();">+</a>
                            <br></h3>',

        );
        }
        else {
            $form['mo_saml_attribute_mapping']['miniorange_saml_idp_additional_user_attrs'] = array(
            '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.attributes',),
            '#markup' => '<br /><br><hr><br><br><h3>Additional User Attributes(Optional)<br></h3>',
                        );
        }



        $form['mo_saml_attribute_mapping']['markup_idp_user_attr_note'] = array(
            '#markup' => '<div class="mo_saml_highlight_background_note" id="attrNote"><b>User Profile Attribute Name:</b> It is the name which you want to send to your SP. It should be unique.
                            <br /><b>User Profile Attribute Value:</b> It is the user attribute (machine name) whose value you want to send to SP.</div>',
        );

        if ($custom_field_disabled){
            global $base_url;
            $custom_fields_url = $base_url . '/admin/config/people/accounts/fields/';
            $form['mo_saml_attribute_mapping']['miniorange_saml_idp_disable_quote'] = array(
                '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.attributes',),
                '#markup' => '<div class="mo_saml_configure_message">You need to <a href="'.$custom_fields_url.'" >add</a> custom user fields to map additional user attributes.</div><br><br>',
            );
        }



        $counter = 0;
        $userProfileAttributes = $saml_sp->getMoIdpUserAttributes();
        $userProfileAttributes = unserialize($userProfileAttributes);


        $form['mo_saml_attribute_mapping']['attr_map_string'] = array(
            '#markup' => '<p  id="attr_string" hidden>'.$str.'</p>',
        );

        $form['mo_saml_attribute_mapping']['mo_saml_user_attr_top'] = array(
            '#markup' => '<div id = "mo_user_attributes_parent">',
        );

        if(isset($userProfileAttributes) && !empty($userProfileAttributes)){
            foreach($userProfileAttributes as $profileAttribute)
            {


                $form['mo_saml_attribute_mapping']['user_profile_attr_name[' . $counter . ']'] = array(
                        '#type' => 'textfield',
                        '#title' => 'User Profile Attribute Name'.($counter + 1),
                        '#value' => $profileAttribute['attr_name'],
                        '#prefix' => '<div class="mo_idp_attr_map_row" id="mo_idp_user_attr_map_'.$counter.'"><div class="mo_idp_attr_map_sp_name">',
                        '#suffix' => '</div>',
                        '#disabled' => $disabled || $custom_field_disabled,
                );


                $form['mo_saml_attribute_mapping']['user_profile_attr_value[' . $counter . ']'] = array(
                    '#type' => 'select',
                    '#title' => 'User Profile Attribute Value'.($counter + 1),
                    '#options' => $custom_fields,
                    '#default_value' => array_search($profileAttribute['attr_value'],$custom_fields),
                    '#attributes' => array('style' => 'width:80%'),
                    '#prefix' => '<div class="mo_idp_attr_map_idp_name">',
                    '#suffix' => '</div>',
                    '#disabled' => $disabled || $custom_field_disabled,
                );

                $form['mo_saml_attribute_mapping']['mo_idp_user_attr_delete[' . $counter . ']'] = array(
                    '#type' => 'button',
                    '#value' => t('-'),
                    '#attributes' => array('class'=>array('button_class_attr','removeUserAttr'),'onclick' => "remove_user_attibute($counter);"),
                    '#prefix' => '<div class="mo_idp_attr_map_delete">',
                    '#suffix' => '</div></div>',
                    '#disabled' => $disabled || $custom_field_disabled,
                );
                $counter+=1;
            }
        }
        if($counter == 0) {
            $form['mo_saml_attribute_mapping']['user_profile_attr_name[' . $counter . ']'] = array(
                '#type' => 'textfield',
                '#title' => 'User Profile Attribute Name'.($counter + 1),
                '#prefix' => '<div class="mo_idp_attr_map_row" id="mo_idp_user_attr_map_'.$counter.'"><div class="mo_idp_attr_map_sp_name">',
                '#suffix' => '</div>',
                '#disabled' => $disabled || $custom_field_disabled,

            );

            $form['mo_saml_attribute_mapping']['user_profile_attr_value[' . $counter . ']'] = array(
                    '#type' => 'select',
                    '#title' => t('User Profile Attribute Value'.($counter + 1)),
                    '#options' => $custom_fields,
                    '#default_value' => 0,
                    '#attributes' => array('style' => 'width:80%'),
                    '#prefix' => '<div class="mo_idp_attr_map_idp_name">',
                    '#suffix' => '</div>',
                    '#disabled' => $disabled || $custom_field_disabled,
                );

            $form['mo_saml_attribute_mapping']['mo_idp_user_attr_delete[' . $counter . ']'] = array(
                    '#type' => 'button',
                    '#value' => t('-'),
                    '#attributes' => array('class'=>array('button_class_attr','removeUserAttr'),'onclick' => "remove_user_attibute($counter);"),
                    '#prefix' => '<div class="mo_idp_attr_map_delete">',
                    '#suffix' => '</div></div>',
                    '#disabled' => $disabled || $custom_field_disabled,
                );
        }

        $form['mo_saml_attribute_mapping']['mo_saml_user_attr_bottom'] = array(
            '#markup' => '</div><br /><br><hr><br><br>',
        );


        if ( !($disabled || $custom_field_disabled) ) {

          $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attrs'] = array(
            '#markup' => '<h3>Constant  Attributes(Optional) &nbsp;&nbsp;&nbsp;&nbsp;
                            <a id="addConstAttr" class="mobtn mo_btn_primary mo_btn_sm" style="padding:6px 12px;" onclick="add_user_attibute();">+</a>&nbsp;&nbsp;&nbsp;&nbsp;
                            <br></h3>',

          );
        }

      $form['mo_saml_attribute_mapping']['markup_idp_constant_attr_note'] = array(
        '#markup' => '<div class="mo_saml_highlight_background_note" id="ConstAttrNote"><b>Attribute Name:</b> It is the name which you want to send to your SP. It should be unique.
                            <br /><b>Attribute Value:</b> It is the constant value you want to send to your SP in every SSO.</div>',
      );
      $counter = 0;
        $constAttributes = $saml_sp->getMoIdpConstAttributes();
      $constAttributes = unserialize($constAttributes);

      $form['mo_saml_attribute_mapping']['mo_saml_constant_attr_top'] = array(
            '#markup' => '<div id = "mo_constant_attributes_parent">',
        );

      if(isset($constAttributes) && !empty($constAttributes)){
        foreach($constAttributes as $Attribute)
        {

          $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attr_name[' . $counter . ']'] = array(
            '#type' => 'textfield',
            '#title' => 'Constant Attribute Name'.($counter + 1),
            '#value' => $Attribute['attr_name'],
            '#prefix' => '<div class="mo_idp_attr_map_row" id="mo_idp_constant_attr_map_'.$counter.'"><div class="mo_idp_attr_map_sp_name">',
                    '#suffix' => '</div>',
            '#disabled' => $disabled || $custom_field_disabled,
          );
          $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attr_value[' . $counter . ']'] = array(
            '#type' => 'textfield',
            '#title' => 'Constant  Attribute Value'.($counter + 1),
            '#value' =>$Attribute['attr_value'],
            '#prefix' => '<div class="mo_idp_attr_map_idp_name">',
                    '#suffix' => '</div>',
            '#disabled' => $disabled || $custom_field_disabled,
          );

          $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attr_delete[' . $counter . ']'] = array(
                    '#type' => 'button',
                    '#value' => t('-'),
                    '#attributes' => array('class'=>array('button_class_attr','removeConstAttr'),'onclick' => "remove_constant_attibute($counter);"),
                    '#prefix' => '<div class="mo_idp_attr_map_delete">',
                    '#suffix' => '</div></div>',
                    '#disabled' => $disabled || $custom_field_disabled,
                );

          $counter+=1;
        }
      }
      if($counter == 0) {
        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attr_name[' . $counter . ']'] = array(
          '#type' => 'textfield',
          '#title' => 'Constant Attribute Name'.($counter + 1),
          '#prefix' => '<div class="mo_idp_attr_map_row" id="mo_idp_constant_attr_map_'.$counter.'"><div class="mo_idp_attr_map_sp_name">',
                    '#suffix' => '</div>',
        );
        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attr_value[' . $counter . ']'] = array(
          '#type' => 'textfield',
          '#title' => 'Constant Attribute Value'.($counter + 1),
          '#prefix' => '<div class="mo_idp_attr_map_idp_name">',
                    '#suffix' => '</div>',
          '#disabled' => $disabled || $custom_field_disabled,
        );

        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_constant_attr_delete[' . $counter . ']'] = array(
                    '#type' => 'button',
                    '#value' => t('-'),
                    '#attributes' => array('class'=>array('button_class_attr','removeConstAttr'),'onclick' => "remove_constant_attibute($counter);"),
                    '#prefix' => '<div class="mo_idp_attr_map_delete">',
                    '#suffix' => '</div></div>',
                    '#disabled' => $disabled || $custom_field_disabled,
                );

      }

        $form['mo_saml_attribute_mapping']['mo_saml_constant_attr_bottom'] = array(
            '#markup' => '</div>',
        );


        $form['mo_saml_attribute_mapping']['miniorange_saml_idp_attr_map_submit'] = array(
            '#type' => 'submit',
            '#value' => t('Save Configuration'),
            '#disabled' => $disabled || $custom_field_disabled,
            '#attributes' => array( 'class' => array('button button--primary')),
            '#prefix' => '<br /><br />',
            '#suffix' => '<br /><br /></div>',
        );


        Utilities::spConfigGuide( $form, $form_state);

        return $form;
    }


    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state)
    {
        $form_values = $form_state->getValues();
        global $base_url;
        $mo_admin_email = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');

        if( !isset( $mo_admin_email ) )
        {
            $this->messenger()->addMessage($this->t('Please register with miniOrange to enable Drupal as IDP.'));
            $test=  $base_url . '/admin/config/people/miniorange_saml_idp/Mapping';
            $response = new RedirectResponse($test);
            $response->send();
        }

        $attr1_name = $form_values['miniorange_saml_idp_attr1_name'];
        $attr1_value = $form_values['miniorange_saml_idp_attr1_value'];
        $attr2_name = $form_values['miniorange_saml_idp_attr2_name'];
        $attr2_value = $form_values['miniorange_saml_idp_attr2_value'];
        $attr3_name = $form_values['miniorange_saml_idp_attr3_name'];
        $attr3_value = $form_values['miniorange_saml_idp_attr3_value'];

        $nameid_attr = $form_values['miniorange_saml_idp_nameid_attr_map'];

        $nameid_attr_value = $nameid_attr == '' ? 'emailAddress' : $nameid_attr;

       $user_attribute_mapping=Utilities::miniorange_serialize_user_attributes(\Drupal::request()->request->getIterator()->getArrayCopy(),'user_profile_attr_name','user_profile_attr_value');
       $const_attribute_mapping=Utilities::miniorange_serialize_const_attributes(\Drupal::request()->request->getIterator()->getArrayCopy(),'miniorange_saml_idp_constant_attr_name','miniorange_saml_idp_constant_attr_value');

       $id_req = \Drupal::request()->query->get('id');
        $saml_sp = new MiniorangeSAMLSP($id_req);
        $saml_sp->updateMapping($nameid_attr_value,$attr1_name,$attr1_value,$attr2_name,$attr2_value,$attr3_name,$attr3_value,$user_attribute_mapping,$const_attribute_mapping);

        $this->messenger()->addMessage($this->t('Your settings are saved successfully.'));

        $redirect = $base_url . '/admin/config/people/miniorange_saml_idp/Mapping';
        $response = new RedirectResponse($redirect);
        $response->send();
    }

    function miniorange_saml_back_to_mapping_list()
    {
        global $base_url;
        $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/Mapping/');
        $response->send();
    }
}
