<?php

  /**
   * @file
   * Contains \Drupal\miniorange_saml_idp\Form\MiniorangeIDPSetup.
   */
namespace Drupal\miniorange_saml_idp\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpConstants;
use Drupal\miniorange_saml_idp\Utilities;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\miniorange_saml_idp\MiniorangeSAMLSP;
use Drupal\miniorange_saml_idp\MiniorangeSAMLCustomer;

class MiniorangeIDPSetup extends FormBase
{
    public function getFormId()
    {
        return 'miniorange_saml_idp_setup';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;

        $delete_req = \Drupal::request()->query->get('delete');
        if (isset($delete_req) && ($delete_req != ""))
        {
            $saml_sp = new MiniorangeSAMLSP($delete_req);
            $saml_sp->delete();
            \Drupal::messenger()->addMessage($this->t('SP Configuration Successfully Deleted.'));
            $redirect = $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup';
            $response = new RedirectResponse($redirect);
            $response->send();
        }

        $saml_sp = new MiniorangeSAMLSP();
        $sp_count = $saml_sp->spCount();

        if($sp_count != 0)
        {
            $id_req = \Drupal::request()->query->get('id');
            if (!isset($id_req))
            {
                $form['mo_saml_markup_sp_table'] = array(
                    '#markup' => '<div id="sp_config_table" style="display:inline-block;" class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container"><h2>IDP Setup</h2><hr/>',
                    '#attached' => array(
                        'library' => array(
                            "miniorange_saml_idp/miniorange_saml_idp.admin",
                        )
                    ),
                );

                $disabled = Utilities::isCustomerRegistered($form, $form_state);

                $form['mo_saml_idp_splist'] = array(
                    '#type' => 'fieldset',
                    '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%;' ),
                );

                $noOfSP = Utilities::getNumberOfSP();

                $add_button_disabled = FALSE;
                if ($sp_count >= $noOfSP)
                {
                    $add_button_disabled = TRUE;
                }
                $sp_left = $noOfSP - $sp_count;

                $form['mo_saml_idp_splist']['mo_saml_idp_add_sp'] = array(
                    '#type' => 'submit',
                    '#value' => t('+ Add New SP'),
                    '#button_type' => 'primary',
                    '#prefix'=>'<td>',
                    '#suffix'=>'</td>',
                    '#submit' => array('::miniorange_add_new_sp'),
                    '#disabled' => $add_button_disabled || $disabled,
                );


                if ($sp_left >= 0) {
                    $form['mo_saml_idp_splist']['mo_saml_remaining_sp_info'] = array(
                        '#markup' => '  (No of SP left: ' . $sp_left.')',
                    );
                }

                $form['mo_saml_idp_splist']['mo_saml_add_more_sp'] = array(
                  '#markup' => '<br><br><div class="mo_saml_highlight_background_note">Click <u> <a target="_blank" href="'. MiniorangeSAMLIdpConstants::BASE_URL .'/moas/login?redirectUrl='. MiniorangeSAMLIdpConstants::BASE_URL .'/moas/initializepayment&requestOrigin='. Utilities::getSamlRequestPlanName() . '">here</a></u> to add more Service Providers.</div>',
                );

                $header = array(
                        'sp_name' => array(
                            'data' => t('SP Name')
                        ),
                        'issuer' => array(
                            'data' => t('SP Entity-ID/Issuer')
                        ),
                        'test' => array(
                            'data' => t('Test')
                        ),
                        'action' => array(
                          'data' => t('Action')
                        ),
                    );

                $table_content = array();
                $saml_sp = new MiniorangeSAMLSP();
                $data = $saml_sp->getTable();

                $form['mo_saml_idp_splist']['libarary'] = array(
                  '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.test',),
                );

                foreach ($data as $sp_data)
                {
                  $drop_button = array(
                    '#type' => 'dropbutton',
                    '#links' => [
                      'edit' => [
                        'title' => $this->t('Edit'),
                        'url' =>  Url::fromUri($base_url.'/admin/config/people/miniorange_saml_idp/idp_setup/?id='.$sp_data->id),
                      ],
                      'delete' => [
                        'title' => $this->t('Delete'),
                        'url' =>  Url::fromUri($base_url.'/admin/config/people/miniorange_saml_idp/idp_setup/?delete='.$sp_data->id),
                      ],
                    ],
                  );

                    $table_content[$sp_data->id] = array(
                        'sp_name' => $sp_data->mo_idp_sp_name,
                        'issuer' => $sp_data->mo_idp_sp_issuer,

                        'test' => Markup::create('<a href="#" onclick="testconfig(\''. $base_url . '/testConfig/?id=' . $sp_data->id .'\')" >Test</a>'),
                        'action' => array(
                          'data' => $drop_button,
                        ),
                    );
                }

                $form['mo_saml_idp_splist']['mo_saml_idp_splist_table'] = array(
                        '#type' => 'table',
                        '#header' => $header,
                        '#rows' => $table_content,
                        '#empty' => t('<b>You have not configured any SP yet, Please Add SP by clicking above "Add New SP" button</b>'),
                        '#prefix' => '<br>',
                        '#suffix' => '</div>',
                        '#disabled' => $disabled,
                    );

              Utilities::spConfigGuide( $form, $form_state);

                return $form;
            }
        }

        $miniorange_config_status = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_config_status');
        if( $miniorange_config_status == 'CONFIGURATION_SAVED' ){
            \Drupal::messenger()->addMessage(t('Your Service Provider Configuration are Imported successfully.'));
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_config_status')->save();
        }

        $form['mo_saml_markup_sp_form'] = array(
          '#markup' => '<div id="sp_config_form" style="display:none;">',
            '#attached' => array(
                'library' => array(
                    "miniorange_saml_idp/miniorange_saml_idp.admin",
                )
            ),
        );

        $form['metadata_1'] = array(
          '#attached' => array(
              'library' => array(
                  'miniorange_saml_idp/miniorange_saml_idp.test',
              )
          ),
        );

        $form['markup_top'] = array(
          '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                       <h2>IDP Setup</h2><hr/>',
        );

        $disabled = Utilities::isCustomerRegistered($form, $form_state);

        $form['mo_saml_SP_setup'] = array(
            '#type' => 'details',
            '#title' => t('Upload SP Metadata' ),
            '#attributes' => array( 'style' => 'padding:0% 2%; margin-bottom:2%' )
        );

        $form['mo_saml_SP_setup']['metadata_file'] = array(
            '#type' => 'file',
            '#title' => t('Upload Metadata File'),
            '#prefix' => '<hr><br><div class="container-inline">',
        );

        $form['mo_saml_SP_setup']['metadata_upload'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Upload'),
            '#submit' => array('::miniorange_saml_upload_file'),
            '#disabled' => $disabled,
            '#suffix'  => t('</div><br><h2>&emsp;&emsp;&emsp;OR</h2><br>'),
            '#limit_validation_errors'=>array(),
        );

        $form['mo_saml_SP_setup']['metadata_URL'] = array(
            '#type' => 'url',
            '#title' => t('Upload Metadata URL'),
            '#attributes' => array('placeholder' => 'Enter metadata URL of your IdP.', 'style' => 'width:65%'),
            '#prefix' => '<div class="container-inline">',
        );

        $form['mo_saml_SP_setup']['metadata_fetch'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Fetch Metadata'),
            '#submit' => array('::miniorange_saml_fetch_metadata'),
            '#disabled' => $disabled,
            '#limit_validation_errors'=>array(),
            '#suffix' => '</div><br>',
        );


        $form['mo_saml_identity_provider_metadata'] = array(
            '#type' => 'fieldset',
            '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.test',),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['mo_saml_identity_provider_metadata']['back_button'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => $this->t('Back'),
            '#submit' => array('::miniorange_saml_back_to_idp_setup'),
            '#attributes' => array('style' => 'float:right' ),
            '#limit_validation_errors'=>array(),
            '#suffix' => '<br><br>'
        );



        $form['mo_saml_identity_provider_metadata']['markup_idp_note'] = array(
            '#markup' => '<div id="idpdata"><br><div class="mo_saml_highlight_background_note">Please note down the following information from your Service Provider
            and keep it handy to configure your Identity Provider.<br>
            <b><ol><li>SP Entity ID / Issuer</li>
            <li>ACS URL</li>
            <li>NameID Format</li>
            <li>Relay State</li>
            <li>X.509 Certificate for Signing if you are using HTTP-POST Binding. (Optional)</li>
            <li>X.509 Certificate for Encryption. (Optional)</li>
            </ol></b><br /></div>'
        );

        $form['mo_saml_identity_provider_metadata']['markup_idp_list'] = array(
            '#markup' => '',
        );

        if(!isset($mo_admin_email))
        {
            $form['mo_saml_identity_provider_metadata']['markup_saml_idp_disabled'] = array(
            '#markup' => '<div style="position: absolute;margin: 7% 33%;font-weight: bold;z-index: 1;">'
            . ' </div><div style="background-color: rgba(218, 218, 218, 0);padding: 2%;opacity: 0.3;">',
            );
        }

        $id_req = \Drupal::request()->query->get('id');
        if (isset($id_req) && $id_req != 'new') {
          $saml_sp = new MiniorangeSAMLSP($id_req);
        }
        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_name'] = array(
            '#type' => 'textfield',
            '#title' => t('Service Provider Name'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpSpName() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'Service Provider Name'),
            '#required' => TRUE,
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_entity_id'] = array(
            '#type' => 'textfield',
            '#title' => t('SP Entity ID or Issuer'),
            '#description' => t('<b>Note:</b> You can find the EntityID in Your SP-Metadata XML file enclosed in <code>EntityDescriptor</code> tag having attribute<br> as <code>entityID</code>.'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpSpIssuer() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'SP Entity ID or Issuer'),
            '#required' => TRUE,
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_nameid_format'] = array(
            '#type' => 'select',
            '#title' => t('NameID Format:'),
            '#options' => array(
             '1.1:nameid-format:emailAddress' => t('urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress'),
             '1.1:nameid-format:unspecified' => t('urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified'),
             '2.0:nameid-format:transient' => t('urn:oasis:names:tc:SAML:1.1:nameid-format:transient'),
             '2.0:nameid-format:persistent' => t('urn:oasis:names:tc:SAML:1.1:nameid-format:persistent'),
            ),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpNameidFormat() : 0,
            '#attributes' => array('style' => 'width:92%'),
            '#description' => t('<b>Note:</b> urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress is selected by default.'),
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_acs_url'] = array(
            '#type' => 'url',
            '#title' => t('ACS URL'),
            '#description' => t('<b>Note:</b> You can find the SAML Login URL in your SP-Metadata XML file enclosed in <code>AssertionConsumerService </code> tag <br/>having attribute as <code>Location</code>.'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpAcsUrl() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'ACS URL'),
            '#required' => TRUE,
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_http_binding'] = array(
            '#type' => 'radios',
            '#title' => t('Logout request HTTP Binding'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpLogoutBindingType() : 0,
            '#options' => array(
                 t('HTTP-Redirect'),
                 t('HTTP-POST')),
            '#disabled' => $disabled,
            '#prefix' => '<div class="container-inline">',
            '#suffix' => '</div>',
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_logout_url'] = array(
            '#type' => 'url',
            '#title' => t('SAML Logout URL'),
            '#description' => t('<b>Note:</b> You can find the SAML Logut URL in your SP-Metadata XML file enclosed in SingleLogoutService tag.'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpSloUrl() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'SAML Logout URL'),
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_relay_state'] = array(
            '#type' => 'textfield',
            '#title' => t('Relay State'),
            '#description' => t('<b>Note:</b> To identify the specific resource at the resource provider in the IDP initiated SSO. It specifes the landing page</br> at the service provider once SSO completes.'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpDefaultRelayState() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'Relay State (optional)'),
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_certificate_signed'] = array(
            '#type' => 'textarea',
            '#title' => t('x.509 Certificate Value [For Signed Request]'),
            '#cols' => '10',
            '#rows' => '5',
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpCert() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'Copy and Paste the content from the downloaded certificate or copy the content enclosed in X509Certificate tag (has parent tag KeyDescriptor use=signing) in SP-Metadata XML file)'),
            '#disabled' => $disabled,
            '#description' => '<strong>NOTE :</strong> Format of the certificate:<br /><strong>-----BEGIN CERTIFICATE-----<br />'
              . 'XXXXXXXXXXXXXXXXXXXXXXXXXXX<br />-----END CERTIFICATE-----</strong><br /><br />'
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_certificate_encrypted'] = array(
            '#type' => 'textarea',
            '#title' => t('X.509 Certificate [For Encrypted Assertion]'),
            '#cols' => '10',
            '#rows' => '5',
            '#default_value' => (isset($id_req) && $id_req != 'new') ? $saml_sp->getMoIdpCertEncrypt() : '',
            '#attributes' => array('style' => 'width:92%','placeholder' => 'Copy and Paste the content from the downloaded certificate or copy the content enclosed in X509Certificate tag (has parent tag KeyDescriptor use=encryption)'),
            '#description' => '<strong>NOTE :</strong> Format of the certificate:<br /><strong>-----BEGIN CERTIFICATE-----<br />'
              . 'XXXXXXXXXXXXXXXXXXXXXXXXXXX<br />-----END CERTIFICATE-----</strong><br /><br />',
            '#states' => array('disabled' => array(':input[name = "miniorange_saml_idp_encrypted_assertion"]' => array('checked' => FALSE || $disabled ), ),),
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_response_signed'] = array(
            '#type' => 'checkbox',
            '#title' => t('Response Signed (Check If you want to sign SAML Response.)'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? (($saml_sp->getMoIdpResponseSigned() == 1) ? TRUE : FALSE) : FALSE,
            '#disabled' => $disabled,
        );


        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_assertion_signed'] = array(
            '#type' => 'checkbox',
            '#title' => t('Assertion Signed (Check If you want to sign SAML Assertion.)'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? (($saml_sp->getMoIdpAssertionSigned() == 1) ? TRUE : FALSE) : FALSE,
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_encrypted_assertion'] = array(
            '#type' => 'checkbox',
            '#title' => t('Encrypted Assertion (Check If you want to encrypt SAML Assertion.)'),
            '#default_value' => (isset($id_req) && $id_req != 'new') ? (($saml_sp->getMoIdpEncryptedAssertion() == 1) ? TRUE : FALSE) : FALSE,
            '#description' => '<b>Note:</b> If you check this option then you need to enter the <em>X.509 Certificate [For Encrypted Assertion]</em> in the <a href="#edit-miniorange-saml-idp-certificate-encrypted"> above</a> field.',
            '#suffix' => '<br/><br/>',
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_config_submit'] = array(
            '#type' => 'submit',
            '#value' => t('Save Configuration'),
            '#attributes' => array( 'class' => array('button button--primary')),
            '#disabled' => $disabled,
        );

        $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_test_config_button'] = array(
            '#type' => 'button',
            '#value' => $this->t('Test Configuration'),
            '#attributes' => array('type'=> 'button', 'onclick' => 'testconfig(\''. $base_url . '/testConfig/?id=' . $saml_sp->getId() .'\');','href'=>"#"),
            '#suffix' => '<br/><br/></div></div></div>',
            '#disabled' => $disabled,
        );

        Utilities::spConfigGuide( $form, $form_state);

        return $form;
    }



    function submitForm(array &$form, FormStateInterface $form_state)
    {
        $form_values = $form_state->getValues();
        global $base_url;
        $sp_name = $form_values['miniorange_saml_idp_name'];
        $issuer = $form_values['miniorange_saml_idp_entity_id'];
        $acs_url = $form_values['miniorange_saml_idp_acs_url'];

        $relay_state = $form_values['miniorange_saml_idp_relay_state'];
        $nameid_format = $form_values['miniorange_saml_idp_nameid_format'];
        $signed_certificate = '';
        $encrypted_certificate = '';
        if (!empty($form_values['miniorange_saml_idp_certificate_signed'])) {
            $signed_certificate = Utilities::sanitize_certificate($form_values['miniorange_saml_idp_certificate_signed']);
        }
        if (!empty($form_values['miniorange_saml_idp_certificate_encrypted'])) {
            $encrypted_certificate = Utilities::sanitize_certificate($form_values['miniorange_saml_idp_certificate_encrypted']);
        }

        $is_response_signed = $form_values['miniorange_saml_idp_response_signed'];// == 1 ? TRUE : FALSE;
        $is_assertion_signed = $form_values['miniorange_saml_idp_assertion_signed'];// == 1 ? TRUE : FALSE;
        $is_encrypted_assertion = $form_values['miniorange_saml_idp_encrypted_assertion'];// == 1 ? TRUE : FALSE;

        $logout_url = $form_values['miniorange_saml_idp_logout_url'];
        $http_binding_value = $form_values['miniorange_saml_idp_http_binding'];

        if (empty($encrypted_certificate) && $is_encrypted_assertion == 1) {
            $this->messenger()->addError($this->t('Note: If you want to use Encrypted assertion feature then you have to enter the certificate for X.509 Certificate [For Encrypted Assertion].'));
            return;
        }

        $id_req = \Drupal::request()->query->get('id');
        if(isset($id_req)) {
          $saml_sp = new MiniorangeSAMLSP($id_req);
        }
        else{
          $saml_sp = new MiniorangeSAMLSP();
        }
        $db_query_status = $saml_sp->insert($sp_name, $issuer, $acs_url, $signed_certificate, $encrypted_certificate, $logout_url, $nameid_format, $relay_state, $is_assertion_signed, $is_encrypted_assertion, $is_response_signed, $http_binding_value);

        if($db_query_status)
        {
        $this->messenger()->addMessage($this->t('Your Service Provider Configuration are successfully saved. Click on Test button below to test these configurations.'));
        $redirect = $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup';
        }

        else {
          $redirect = $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup/?id='.$id_req;
        }

        $response = new RedirectResponse($redirect);
        $response->send();
    }

    function miniorange_add_new_sp(array &$form, FormStateInterface $form_state){
        global $base_url;
        $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/idp_setup/?id=new');
        $response->send();
    }

    function getTestUrl()
    {
        global $base_url;
        $testUrl = $base_url . '/?q=testConfig';
        return $testUrl;
    }

    function miniorange_saml_upload_file(array &$form, FormStateInterface $form_state) {
        $file_name = $_FILES['files']['tmp_name']['metadata_file'];
        $file = file_get_contents($file_name);
        $saml_sp = new MiniorangeSAMLSP();

        $sp_count = $saml_sp->spCount();

        $noOfSP = Utilities::getNumberOfSP();

        if ($noOfSP > $sp_count) {
            $redirect_id = $saml_sp->update_metadata($file, $type = 'file');
            if ($redirect_id != FALSE) {
                global $base_url;
                $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/idp_setup/?id=' . $redirect_id);
                $response->send();
            }
        }
        else {
            $this->messenger()->addError($this->t('You have exceeded the limit of Service Providers you can configure.'));
            return;
        }
    }

    function miniorange_saml_fetch_metadata(array &$form, FormStateInterface $form_state) {
       $url = filter_var($form['mo_saml_SP_setup']['metadata_URL']['#value'],FILTER_SANITIZE_URL);
       $arrContextOptions=array(
            "ssl"=>array(
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
       );
        $file = file_get_contents($url, false, stream_context_create($arrContextOptions));
        $saml_sp = new MiniorangeSAMLSP();

        $sp_count = $saml_sp->spCount();

        $noOfSP = Utilities::getNumberOfSP();

        if ($noOfSP > $sp_count) {
            $redirect_id = $saml_sp->update_metadata($file, $type = 'file');
            if ($redirect_id != FALSE) {
                global $base_url;
                $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/idp_setup/?id=' . $redirect_id);
                $response->send();
            }
        }
        else {
            $this->messenger()->addError($this->t('You have exceeded the limit of Service Providers you can configure.'));
            return;
        }
    }

    function miniorange_saml_back_to_idp_setup(){
        global $base_url;
        $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/idp_setup/');
        $response->send();
    }
}
