<?php

/**
 * @file
 * Contains \Drupal\miniorange_saml_idp\Form\MiniorangeSPInformation.
 */

namespace Drupal\miniorange_saml_idp\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Url;
use Drupal\miniorange_saml_idp\Utilities;
use Drupal\Core\Render\Markup;

class MiniorangeSPInformation extends FormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormId()
  {
    return 'miniorange_sp_setup';
  }
public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
{
    global $base_url;
    $url = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_idp_base_url');
    $url = isset($url) && !empty($url)? $url : $base_url;
    $acs_url = $url  . '/samlassertion';
    $logout_url =$url  . '/user/logout';

    $login_url = $url . '/initiatelogon';
    $issuer = $base_url . '/?q=admin/config/people/miniorange_saml_idp/';
    $module_path = drupal_get_path('module', 'miniorange_saml_idp');

    $form['markup_style_1'] = array(
        '#attached' => array(
            'library' => array(
                "miniorange_saml_idp/miniorange_saml_idp_copy.icon",
                "miniorange_saml_idp/miniorange_saml_idp.admin",
            )
        ),
    );

    $form['markup_top'] = array(
      '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                         ',
    );

  Utilities::isCustomerRegistered($form, $form_state);

    $form['mo_saml_service_provider_metadata'] = array(
            '#type' => 'fieldset',
            //'#title' => t('Service Provider Metadata'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );


    $form['mo_saml_service_provider_metadata']['miniorange_saml_idp_div_st'] = array(
        '#markup' => '<h2>Identity Provider Metadata</h2><hr>',
    );

    $form['mo_saml_service_provider_metadata']['mo_saml_idp_metadata_customization'] = array(
      '#type' => 'details',
      '#title' => t('Metadata Customization'),
    );

  $form['mo_saml_service_provider_metadata']['mo_saml_idp_metadata_customization']['mo_saml_idp_sign_authentication_request'] = array(
    '#type' => 'radios',
    '#title' => $this->t('Authentication request type:'),
    '#options' => array('signed' => t('Signed'), 'unsigned' => t('Unsigned')),
    '#default_value' => \Drupal::config('miniorange_saml_idp.settings')->get('mo_saml_idp_sign_authentication_request'),
    '#description' => $this->t('Select the authentication request type (Signed or Unsigned) you want to send.'),
  );

  $form['mo_saml_service_provider_metadata']['mo_saml_idp_metadata_customization']['mo_saml_idp_save_metadata_customization'] = array(
    '#type' => 'submit',
    '#value' => $this->t('Save'),
  );


    $form['mo_saml_service_provider_metadata']['markup_idp_sp_2'] = array(
        '#markup' => '<br><div class="mo_saml_idp_font_SP_setup_for_heading">Provide this metadata information to your Service Provider team.<br> You can choose any one of the below options:</div><br>
                    <b>a) You can provide this metadata URL to your Service Provider.</b><br /><div>
                            <div class="mo_saml_highlight_background_url_note" id="idp_metadata_url">
                                <code id="meta_data_url_for_SP"><b>
                                    <span>
                                        <a target="_blank" href="' . $base_url . '/moidp_metadata/?download=0">' . $base_url . '/moidp_metadata</a>
                                    </span></b>
                                </code>
                            </div>
                            <img class ="fa fa-fw fa-lg fa-copy mo_copy mo-margin" src="'. $base_url.'/'.$module_path . '/includes/images/copy-regular.svg">
                        </div>',
    );

    $form['mo_saml_service_provider_metadata']['download_xml_metadata'] = array(
        '#markup' => '<br><br><div id="download_metadata">
            <b>b)Download the Plugin XML metadata and upload it on your Service Provider :</b></b>
                                <span><a href="' . $base_url . '/moidp_metadata/?download=1" class="mobtn mo_btn_primary mo_btn_sm">Download XML Metadata</a></span>
                        </div><br><br><div><b>c) Provide the following information to your Service Provider. Copy it and keep it handy.</b><br/></div><br>', );


    $copy_image = '<img class ="fa fa-fw fa-pull-right fa-lg fa-copy mo_copy" src="'. $base_url.'/'.$module_path . '/includes/images/copy-regular.svg">';

    $SP_Entity = [
      'data' => Markup::create('<span id="issuer_id">' . $issuer . '</span>'. $copy_image )
    ];
    $SAML_login_url = [
      'data' => Markup::create('<span id="login_url">' . $login_url . '</span>'. $copy_image )
    ];
    $SAML_logout_url = [
      'data' => Markup::create('<span id="logout_url">' . $logout_url . '</span>'. $copy_image )
    ];
    $certificate = [
        'data' => Markup::create('<a href="' . $base_url . '/moidp_certificate" >Click here</a> to download X509 certificate.')
    ];
    $response_signed = [
      'data' => Markup::create('You can choose to sign your response in <a href="' . $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup">Identity Provider setup</a> tab.')
    ];
    $assertion_signed = [
      'data' => Markup::create('You can choose to sign your response in <a href="' . $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup">Identity Provider setup</a> tab.')
    ];

    $mo_table_content = array(
      array('IDP-Entity ID / Issuer',$SP_Entity),
      array('SAML Login URL',$SAML_login_url),
      array('SAML Logout URL',$SAML_logout_url),
      array('Certificate (Optional) ',$certificate),
      array('Response Signed',$response_signed),
      array('Assertion Signed',$assertion_signed),
    );

    $form['mo_saml_service_provider_metadata']['mo_saml_attrs_list_idp'] = array(
      '#type' => 'table',
      '#header' => array('ATTRIBUTE','VALUE'),
      '#rows' => $mo_table_content,
      '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a>'),
      '#responsive' => TRUE ,
      '#sticky'=> TRUE,
      '#size'=> 2,
      '#suffix' => '</div>',
    );

    Utilities::spConfigGuide( $form, $form_state);

  return $form;
 }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $sign_authentication_request = $form_state->getValue('mo_saml_idp_sign_authentication_request');
    \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_idp_sign_authentication_request', $sign_authentication_request)->save();
  }
}
