<?php

/**
 * @file
 * Contains \Drupal\miniorange_saml\Form\MiniorangeSamlCustomerSetup.
 */

namespace Drupal\miniorange_saml_idp\Form;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_saml_idp\Utilities;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\miniorange_saml_idp\MiniorangeSAMLCustomer;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpConstants;

class MiniorangeSamlCustomerSetup extends FormBase {

    public function getFormId() {
        return 'miniorange_saml_customer_setup';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {

        if ( \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_idp_server_unreachable') == true ){
          \Drupal::messenger()->addError(t('Could not reach to the server at the moment. Please check your internet connectivity.'));
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_idp_server_unreachable', false)->save();
        }

        $current_status = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_status');

        if ($current_status == 'PLUGIN_CONFIGURATION' && \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_license_key') == NULL)
        {
            $form['header_top_style1'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">',
                '#attached' => array(
                    'library' => array(
                        "miniorange_saml_idp/miniorange_saml_idp.admin",
                    )
                ),

            );

          $form['mo_saml_license_key_form'] = array(
                '#type' => 'fieldset',
                '#title' => t('license key'),
                '#attributes' => array( 'style' => 'padding:2% 2% 40%; margin-bottom:2%' ),
            );


          $email = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
          $license_page_url = MiniorangeSAMLIdpConstants::BASE_URL .'/moas/login?username='. $email .'&redirectUrl=' . MiniorangeSAMLIdpConstants::BASE_URL . '/moas/admin/customer/viewlicensekeys';

          $form['mo_saml_license_key_form']['markup_license'] = array(
            '#markup' => '<br><hr><br><div class="mo_saml_highlight_background_url_note"><strong>Note:</strong> Click <a href="'.$license_page_url.'" target="_blank">here</a> and login to get the license key.</div>'
          );

            $form['mo_saml_license_key_form']['miniorange_saml_license_key'] = array(
                '#type' => 'textfield',
                '#title' => t('License Key<span style="color: red">*</span>'),
                '#attributes' => array(
                    'placeholder' => 'Enter your license key to activate the plugin',
                ),
                '#prefix' => '<br><br>'
            );

            $form['mo_saml_license_key_form']['miniorange_saml_customer_validate_license_button'] = array(
                '#type' => 'submit',
                '#value' => t('Activate License'),
                '#button_type' => 'primary',
                '#submit' => array('::miniorange_saml_validate_license_submit'),
            );

			$form['mo_saml_license_key_form']['miniorange_saml_customer_setup_back'] = array(
                '#type' => 'submit',
                '#value' => t('Back'),
                '#button_type' => 'danger',
                '#submit' => array('::miniorange_saml_idp_back'),
            );

            $form['mo_saml_license_key_form']['div_close'] = array(
                '#markup' => '<br></div>',
            );

            Utilities::spConfigGuide( $form, $form_state);

            return $form;

        }elseif ($current_status == 'PLUGIN_CONFIGURATION') {

            $form['markup_top_welocome_message'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                    <div class="mo_saml_welcome_message">Thank you for registering with miniOrange</div>',
                '#attached' => array(
                    'library' => array(
                        "miniorange_saml_idp/miniorange_saml_idp.admin",
                    )
                ),
            );

            $form['mo_saml_user_profile'] = array(
                    '#type' => 'fieldset',
                    '#title' => t('Your Profile:'),
                    '#attributes' => array( 'style' => 'padding:2% 2% 4%; margin-bottom:2%' ),
                );

            $noOfSP = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_number_of_sp');
            $UserCount = \Drupal::config('miniorange_saml_idp.settings')->get('miniOrange_saml_idp_user_count');
            $license_exp = date("F j, Y, g:i a", \Drupal::config('miniorange_saml_idp.settings')->get('miniOrange_saml_idp_l_exp') );
            $connection = \Drupal::database();
            $query = $connection->query("SELECT count(*) as count FROM {miniorange_saml_idp_user} where UserIn = 1");
            $result = $query->fetchAll();
            $UserInCount = $result[0]->count;

            $header = array( t('ATTRIBUTE NAME'), t('ATTRIBUTE VALUE'));

            $options = [];

            $options[0] = array(
                'Customer Email' => 'Customer Email',
                'email' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email'),
            );

            $options[1] = array(
                'Customer ID' => 'Customer ID',
                'customerid' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id'),
            );

            $options[2] = array(
                'Token Key' => 'Token Key',
                'token' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_token'),
            );

            $options[3] = array(
                'API Key' => 'API Key',
                'apikey' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key'),
            );

            $options[4] = array(
                'SSO Users Allowed' => 'SSO Users Allowed',
                'sso_users_allowed' => $UserCount,
            );

            $options[5] = array(
                'Current SSO Users' => 'Current SSO Users',
                'current_sso_users' => $UserInCount . ' (Remaining: '. ($UserCount - $UserInCount) . ')' ,
            );

            $options[6] = array(
                'No of SP allowed' => 'No of SP allowed',
                'no_of_sp_allowed' => $noOfSP,
            );

            $options[7] = array(
                'License Validity' => 'License Validity',
                'license_validity' => $license_exp,
            );

            $form['mo_saml_user_profile']['fieldset']['customerinfo'] = array(
                '#theme' => 'table',
                '#header' => $header,
                '#rows' => $options,
                '#prefix' => '<br>',
            );

            $form['mo_saml_user_profile']['miniorange_saml_check_license'] = array(
                '#type' => 'submit',
                '#value' => t('Check License'),
                '#button_type' => 'primary',
                '#submit' => array('::miniorange_check_license'),
                '#prefix' => '<br>',
            );

            $form['mo_saml_user_profile']['miniorange_saml_customer_Remove_Account_info'] = array(
                '#markup' => '<br/><br/><br/><br/><br/><h3>Remove Account:</h3><p>This section will help you to remove your current logged in account without losing your current configurations.</p>');

            $form['mo_saml_user_profile']['miniorange_saml_customer_Remove_Account'] = array(
                '#markup' => '<br><br></div>',
                '#type' => 'submit',
                '#value' => t('Remove Account'),
                '#submit' => array('::miniorange_saml_remove_account'),
            );

            Utilities::spConfigGuide( $form, $form_state);
            return $form;
        }

        $form['markup_14'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">',
            '#attached' => array(
                'library' => array(
                    "miniorange_saml_idp/miniorange_saml_idp.admin",
                )
            ),
        );

        $form['mo_saml_login_form'] = array(
                '#type' => 'fieldset',
                '#title' => t('Login with miniOrange account'),
                '#attributes' => array( 'style' => 'padding:2% 2% 30%; margin-bottom:2%' ),
        );

        $form['mo_saml_login_form']['markup_15'] = array(
            '#markup' => '<br><div class="mo_saml_highlight_background_note">Please login with your miniorange account for which you
                    have purchased license. After successful login you need to enter license key. You can find license key 
                    <u><a target="_blank" href="'. MiniorangeSAMLIdpConstants::BASE_URL .'/moas/login?redirectUrl='. MiniorangeSAMLIdpConstants::BASE_URL  .'/moas/admin/customer/viewlicensekeys">here</a></u>. </div><br>'
        );

        $form['mo_saml_login_form']['miniorange_saml_customer_setup_username'] = array(
            '#type' => 'textfield',
            '#title' => t('Email<span style="color: red">*</span>'),
            '#attributes' => array('style' => 'width:500px; margin-bottom: 5px;'),
        );

        $form['mo_saml_login_form']['miniorange_saml_customer_setup_password'] = array(
            '#type' => 'password',
            '#title' => t('Password<span style="color: red">*</span>'),
            '#attributes' => array('style' => 'width:500px; margin-bottom: 10px;'),
        );

        $form['mo_saml_login_form']['miniorange_saml_customer_setup_button'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Login'),
        );

        $form['mo_saml_login_form']['miniorange_saml_password_reset_button'] = array(
          '#type' => 'submit',
          '#button_type' => 'danger',
          '#submit' => array('::miniorange_reset_password'),
          '#limit_validation_errors' => array(),
          '#value' => t('Forgot Password'),
          '#attributes' => array('onclick' => 'this.form.target="_blank";'),
        );

        $form['miniorange_support_div_close'] = array(
            '#markup' => '<br></div>'
        );

        Utilities::spConfigGuide($form, $form_state);

        return $form;
    }


    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $username = $form['mo_saml_login_form']['miniorange_saml_customer_setup_username']['#value'];
        $password = $form['mo_saml_login_form']['miniorange_saml_customer_setup_password']['#value'];

        if( empty( $username ) || empty( $password ) ){
            \Drupal::messenger()->addMessage(t('The <b><u>Email Address</u></b> and <b><u>Password</u></b> fields are mandatory.'), 'error');
            return;
        }
		    if ( !\Drupal::service('email.validator')->isValid( $username ) ) {
            \Drupal::messenger()->addMessage(t('The email address <i>' . $username . '</i> is not valid.'), 'error');
            return;
        }

        $customer_config = new MiniorangeSAMLCustomer($username, '', $password, NULL);
        $check_customer_response = json_decode($customer_config->checkCustomer());

        if ($check_customer_response->status == 'CUSTOMER_NOT_FOUND') {
            \Drupal::messenger()->addMessage(t('user not found with the username - <b><i>' . $username . '</i></b>.  Please register with miniorange before login.'), 'error');
            return;
        }elseif ($check_customer_response->status == 'CURL_ERROR') {
            \Drupal::messenger()->addMessage(t('cURL is not enabled. Please enable cURL'), 'error');
            return;
        }else {
            $customer_keys_response = json_decode($customer_config->getCustomerKeys());
            if (json_last_error() == JSON_ERROR_NONE) {
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_id', $customer_keys_response->id)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_token', $customer_keys_response->token)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_email', $username)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_api_key', $customer_keys_response->apiKey)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_phone', $customer_keys_response->phone)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'PLUGIN_CONFIGURATION')->save();
                \Drupal::messenger()->addMessage(t('Successfully retrieved your account.'));
            }else {
                \Drupal::messenger()->addMessage(t('Invalid credentials'), 'error');
                return;
            }

            if (\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_license_key'))
              \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_license_key')->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'PLUGIN_CONFIGURATION')->save();

        }
    }

    function miniorange_saml_idp_back(&$form, $form_state) {
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'CUSTOMER_SETUP')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_miniorange_saml_customer_admin_email')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_tx_id')->save();
        \Drupal::messenger()->addMessage(t('Login with your miniOrange Account'),'status');
    }

    /**
     * Handles License.
     */
    function miniorange_saml_validate_license_submit(&$form, $form_state) {

        $code = $form['mo_saml_license_key_form']['miniorange_saml_license_key']['#value'];
        $customer = new MiniorangeSAMLCustomer(NULL, NULL, NULL, NULL);
        $content = json_decode( $customer->ccl(), true );

        if(  $content['status'] != 'SUCCESS' ) {
            \Drupal::messenger()->addMessage(t('Invalid license! Please use a valid license to activate the module.'),'error');
            return;
        }

        $response = json_decode( $customer->check_status( $code ), true );

        if(strcasecmp( $response['status'], 'SUCCESS') == 0 ) {

            //Number of SP allowed to configure
            $noOfSP = isset($content['noOfSP']) ? $content['noOfSP'] : 0;

            $UserCount  = isset( $content['noOfUsers'] ) ? $content['noOfUsers'] : '';
            $l_exp      = isset( $content['licenseExpiry'] ) ? strtotime( $content['licenseExpiry'] ) : '';
            $key        = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_token');
            $stored_key = Utilities::encrypt_data( $code, $key );

          $hostName        = MiniorangeSAMLIdpConstants::BASE_URL;
          $licensePlanName = Utilities::getSamlRequestPlanName();
          $licenseType     = Utilities::getSamlRequestPlanType();
          $customerId      = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id');
          $apiKey          = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key');
          $upgradeUrl      = Utilities::createUpdateUrl( $code, $licensePlanName, $licenseType, $apiKey, $customerId, $hostName );
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_upgrade_url',$upgradeUrl)->save();

            /*
             * Saved license key for duplicate license check
             */
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_original_license_key', $code)->save();

            //Saved Number of SP allowed for limiting number of SP that can be configured
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_number_of_sp', $noOfSP)->save();

            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_license_key', $stored_key)->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniOrange_saml_idp_user_count', $UserCount)-> save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniOrange_saml_idp_l_exp', $l_exp)-> save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('tmp_exp', $l_exp)-> save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('te_count', 0)->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('ue_count', 0)->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('dcheck', 0)->save();

            \Drupal::messenger()->addMessage(t('Your license has been verified. You can configure your Service Provider settings now.'));

        }else if( strcasecmp($response['status'], 'FAILED' ) == 0){
            if(strcasecmp($response['message'], 'Code has Expired') == 0){
                \Drupal::messenger()->addMessage(t('License key you have entered has already been used. Please enter a key which has not been used before on any other instance or if you have exausted all your keys thenbuy more license from Licensing'), 'error');
                return;
            }else {
                \Drupal::messenger()->addMessage(t('You have entered an invalid license key. Please enter a valid license key.'), 'error');
                return;
            }

        }else {
            \Drupal::messenger()->addMessage(t('An error occured while processing your request. Please Try again.'), 'error');
            return;
        }
    }


    /*
     * Remove Account
     */
    function miniorange_saml_remove_account(&$form, $form_state) {
        global $base_url;
        $post = \Drupal::request()->request->getIterator()->getArrayCopy();

        $form_id = $post['form_id'];
        $form_token = $post['form_token'];
        $op = $post['op'];
        $build_id = $post['form_build_id'];
        ?>

        <html>
        <head>
            <title>Confirmation</title>
            <link href="https://fonts.googleapis.com/css?family=PT+Serif" rel="stylesheet">
        </head>

        <body style="font-family: 'PT Serif', serif;">
        <div style="margin: 15% auto; height:35%; width: 40%; background-color: #eaebed; text-align: center; box-shadow: 10px 5px 5px darkgray; border-radius: 2%;">
        <div style="color: #a94442; background-color:#f2dede; padding: 15px; margin-bottom: 20px; text-align:center; border:1px solid #E6B3B2; font-size:16pt; border-radius: 2%;">
            <strong>Are you sure you want to remove account..!!</strong>
        </div>
        <p style="font-size:14px; margin-left: 8%; margin-right: 8%"><strong>Warning </strong>: If you remove your account, you will have to enter licence Key Again after login/sign in with the new account.</p>
        <br/>
        <form name="f" method="post" action=" <?php echo $base_url . '/remove_license' ?> " id="mo_remove_account">
        <div>
        <input type="hidden" name="op" value=<?php echo $op;?>>
        <input type="hidden" name="form_build_id" value= <?php echo $build_id;?>>
        <input type="hidden" name="form_token" value=<?php echo $form_token;?>>
        <input type="hidden" name="form_id" value= <?php echo $form_id;?>>
        <input type="hidden" name="value_check" value= 'True'>
        </div>
        <div  style="margin: auto; text-align: center;"   class="mo2f_modal-footer">
        <input type="submit" style=" padding:1%; width:100px; background: #0091CD none repeat scroll 0% 0%; cursor: pointer; font-size:15px; border-width: 1px; border-style: solid; border-radius: 3px; white-space: nowrap; box-sizing: border-box;border-color: #0073AA; box-shadow: 0px 1px 0px rgba(120, 200, 230, 0.6) inset; color: #FFF;" name="miniorange_confirm_submit" class="button button-danger button-large" value="Confirm"/>
        <a href=<?php echo $base_url.'/admin/config/people/miniorange_saml_idp/customer_setup'; ?> >Cancel</a>
        </div>
    </form>
    </div>
    </body>
    </html>
    <?php
    exit;
}

  function miniorange_reset_password( &$form, \Drupal\Core\Form\FormStateInterface $form_state ) {
      $response = new TrustedRedirectResponse(MiniorangeSAMLIdpConstants::BASE_URL . '/moas/idp/resetpassword');
      $form_state->setResponse( $response );
    }

  function miniorange_check_license(){
        $customer   = new MiniorangeSAMLCustomer(NULL, NULL, NULL, NULL);
        $content = json_decode($customer->ccl(), true);
        $license_exp = strtotime($content['licenseExpiry']);

        //Checking and updating expiry date
        if ($license_exp > \Drupal::config('miniorange_saml_idp.settings')->get('miniOrange_saml_idp_l_exp')){
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniOrange_saml_idp_l_exp', $license_exp)->save();
          $connection = \Drupal::database();
          $update  = $connection->update('miniorange_saml_idp_user')
              ->fields(['UserIn' => 0,])
              ->execute();
        }

        $UserCountOld = \Drupal::config('miniorange_saml_idp.settings')->get('miniOrange_saml_idp_user_count');
        $UserCount  = isset( $content['noOfUsers'] ) ? $content['noOfUsers'] : $UserCountOld;

        //Updating number of users
        if ($UserCount != $UserCountOld){
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniOrange_saml_idp_user_count', $UserCount)->save();
        }

        $noOfSPOld = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_number_of_sp');
        $noOfSP = isset($content['noOfSP']) ? $content['noOfSP'] : $noOfSPOld;

        //Updating number of SP
        if ($noOfSP != $noOfSPOld){
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_number_of_sp', $noOfSP)->save();
        }
    }


}
