<?php
/**
 * @file
 * Contains Login Settings for miniOrange SAML Login Module.
 */

 /**
 * Showing Settings form.
 */
namespace Drupal\miniorange_saml_idp\Form;

use Drupal\Core\Form\FormBase;
use Drupal\miniorange_saml_idp\MiniorangeSAMLSP;
use Drupal\miniorange_saml_idp\Utilities;
use Drupal\Core\Render\Markup;

class MiniorangeSignonSettings extends FormBase {

  public function getFormId() {
    return 'miniorange_saml_login_setting';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {

      global $base_url;
      $module_path = drupal_get_path('module', 'miniorange_saml_idp');

      $form['markup_idp_login_header'] = array(
            '#attached' => array(
              'library' => array(
                  "miniorange_saml_idp/miniorange_saml_idp_copy.icon",
                  "miniorange_saml_idp/miniorange_saml_idp.admin",
              )
            ),
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                    <h2>IDP Initiated Login</h2><hr>',
      );

      $disabled = Utilities::isCustomerRegistered($form, $form_state);

      $form['mo_saml_singnin_settings'] = array(
            '#type' => 'fieldset',
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

      $form['mo_saml_singnin_settings']['markup_idp_login_note'] = array(
          '#markup' => '<br><div><b>Add a link to user dashboard for login into your Service Provider.</b></div></br>
            <div class="mo_saml_highlight_background_note"><b>Note: </b>Add the following link such that it is only
           visible to your logged in users.</div><br>',
      );

      $header = array(
          'sp_name' => array(
              'data' => t('SP Name')
          ),

          'url' => array(
              'data' => t('URL')
          ),
      );

      $table_content = array();
      $saml_sp = new MiniorangeSAMLSP();
      $data = $saml_sp->getTable();

      $form['mo_saml_idp_splist']['libarary'] = array(
          '#attached' => array('library' => 'miniorange_saml_idp/miniorange_saml_idp.test',),
      );


      $copy_image = '<img class ="fa fa-fw fa-pull-right fa-lg fa-copy mo_copy" src="'. $base_url.'/'.$module_path . '/includes/images/copy-regular.svg">';

      foreach ($data as $sp_data)
      {
          $idp_initiated_login = '<code><a>Please login before to see the IDP initiated login URL.</a></code>';
          if( $disabled == FALSE ) {
              $idp_initiated_login = '<code><a class="site-url"  >' . $base_url . '/saml_user_login' . '?sp='.$sp_data->mo_idp_sp_name.'</a></code>';
          }
          $table_content[$sp_data->id] = array(
              'sp_name' => $sp_data->mo_idp_sp_name,
              'test' => Markup::create('<b id="copy_idp_initiat_url'. $sp_data->id .'" >' . $idp_initiated_login. '</b>' . $copy_image . ''),
          );
      }

      $form['mo_saml_singnin_settings']['mo_saml_idp_splist_table'] = array(
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $table_content,
          '#empty' => t('<b>You have not configured any SP yet, please add SP in "<a href="'. $base_url .'/admin/config/people/miniorange_saml_idp/idp_setup">Identity Provider Setup</a>" tab.</b>'),
          '#suffix' => '</div>',
      );


      $form['mo_saml_singnin_settings']['markup_idp_dynamic_login_note'] = array(
            '#markup' => '<br><br><div class="mo_saml_highlight_background_note"><b>Note: </b>If you want to redirect users dynamically on specific URL at Service Provider 
                            site, after authentication. Then, just append the following part to Service Provider login URL from above table.</div>',
      );

      $idp_initiated_dynamic_login = '<br><b><code><a href="'. $base_url .'/admin/config/people/miniorange_saml_idp/customer_setup">Please login to see the IDP initiated dynamic login URL.</a></code></b><br><br>';
      if( $disabled == FALSE ) {
          $url = "&RelayState=";
          $url .= htmlspecialchars("<URL to which the user is to be redirected>");
          $idp_initiated_dynamic_login = "<td><b id=\"copy_idp_initiat_dynamic_url\"><code><a class=\"site-url\">$url</a></code></b>" . $copy_image . "</td>";
      }

      $form['mo_saml_singnin_settings']['markup_idp_dynamic_login_link'] = array(
          '#markup' => '<table>
                          <tr>
                            ' . $idp_initiated_dynamic_login. '
                          </tr>
                        </table>',


      );

      $enable_logging = \Drupal::config('miniorange_saml_idp.settings')->get('mo_saml_idp_debugging_log');
      $form['mo_saml_singnin_settings']['mo_saml_idp_debugging_log'] = array(
        '#type' => 'checkbox',
        '#title' => t('Enable logging'),
        '#default_value' => isset($enable_logging) ? $enable_logging : 0,
        '#description' => t('Enabling this checkbox will add loggers under the <a target="_blank" href="'. $base_url .'/admin/reports/dblog"> Reports </a> section '),
        '#suffix' => '<br><br>',
        '#disabled' => $disabled,
      );

      $form['mo_saml_singnin_settings']['mo_saml_idp_logging'] = array(
        '#type' => 'submit',
        '#value' => t('Save Configuration'),
        '#disabled' => $disabled,
        '#attributes' => array( 'class' => array('button button--primary')),
      );

      Utilities::spConfigGuide( $form, $form_state);

      return $form;
 }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state){
      $enable_logging = $form['mo_saml_singnin_settings']['mo_saml_idp_debugging_log']['#value'];
      \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('mo_saml_idp_debugging_log',$enable_logging)->save();
      $this->messenger()->addMessage($this->t('Your settings are saved successfully.'));
    }

}
