<?php


namespace Drupal\miniorange_saml_idp\Form;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_saml_idp\Utilities;

class MiniorangeSupport extends FormBase
{
  public function getFormId()
  {
    return 'miniorange_saml_idp_support';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
  {
    $email = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
    $phone = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_phone');
    $form['markup_1'] = array(
      '#markup' => t('<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'),
        '#attached' => array(
            'library' => array(
                "miniorange_saml_idp/miniorange_saml_idp.admin",
            )
        ),
    );

    $form['mo_saml_vertical_tabs'] = array(
      '#type' => 'vertical_tabs',
      '#default_tab' => 'edit-publication',
    );

    /**
     * Support form
     */
    $form['miniorange_support'] = array (
      '#type' => 'details',
      '#title' => t('Support'),
      '#group' => 'mo_saml_vertical_tabs',
    );

    $form['miniorange_support']['markup_1'] = array(
      '#markup' => t('<h3>Support</h3><hr><p class="mo_saml_highlight_background_note">Need any help? We can help you with configuring miniOrange SAML SP module on your site. Just send us a query and we will get back to you soon.</p>'),
    );

    $form['miniorange_support']['mo_saml_support_email_address'] = array(
      '#type' => 'email',
      '#title' => t('Email <span style="color: red">*</span>'),
      '#default_value' => $email,
      '#attributes' => array('placeholder' => t('Enter your email'), 'style' => 'width:99%;margin-bottom:1%;'),
    );

    $form['miniorange_support']['mo_saml_support_phone_number'] = array(
      '#type' => 'textfield',
      '#title' => t('Phone'),
      '#default_value' => $phone,
      '#attributes' => array('placeholder' => t('Enter number with country code Eg. +00xxxxxxxxxx'), 'style' => 'width:99%;margin-bottom:1%;'),
    );

    $form['miniorange_support']['mo_saml_support_query'] = array(
      '#type' => 'textarea',
      '#title' => t('Query <span style="color: red">*</span>'),
      '#attributes' => array('placeholder' => t('Describe your query here!'), 'style' => 'width:99%'),
      '#suffix' => '<br>',
    );

    $form['miniorange_support']['actions']['submit'] = array(
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => t('Submit query'),
    );

    $form['miniorange_support']['markup_support_note'] = array(
      '#markup' => t('<div><br/><br/>If you want custom features in the module, just drop an email to <a href="mailto:info@xecurify.com">info@xecurify.com</a> or <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a></div><br><hr><br>'),
    );

    /**
     * New feature request form
     */
    $form['miniorange_feature_request'] = array (
      '#type' => 'details',
      '#title' => t('New Feature Request'),
      '#group' => 'mo_saml_vertical_tabs',
    );

    $form['miniorange_feature_request']['markup_1'] = array(
      '#markup' => t('<h3>New Feature Request</h3><hr><p class="mo_saml_highlight_background_note">Need new feature or any customization in the module? Just send us a requirement so we can help you.'),
    );
    $form['miniorange_feature_request']['mo_saml_feature_email_address'] = array(
      '#type' => 'email',
      '#title' => t('Email <span style="color: red">*</span>'),
      '#default_value' => $email,
      '#attributes' => array('placeholder' => t('Enter your email'), 'style' => 'width:99%;margin-bottom:1%;'),
    );
    $form['miniorange_feature_request']['mo_saml_feature_phone_number'] = array(
      '#type' => 'textfield',
      '#title' => t('Phone'),
      '#default_value' => $phone,
      '#attributes' => array('placeholder' => t('Enter number with country code Eg. +00xxxxxxxxxx'), 'style' => 'width:99%;margin-bottom:1%;'),
    );
    $form['miniorange_feature_request']['mo_saml_feature_query'] = array(
      '#type' => 'textarea',
      '#title' => t('Description <span style="color: red">*</span>'),
      '#attributes' => array('placeholder' => t('Describe your requirement here!'), 'style' => 'width:99%'),
      '#suffix' => '<br>',
    );

    $form['miniorange_feature_request']['actions']['submit'] = array(
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => t('Submit'),
    );
    $form['miniorange_feature_request']['markup_feature_request_note'] = array(
      '#markup' => t('<div><br/><br/>For any other queries, reach out to us using support form or just drop an email to <a href="mailto:info@xecurify.com">info@xecurify.com</a> or <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a></div><br><hr><br>'),
    );

    $form['miniorange_support_tab_end'] = array(
      '#markup' => t('</div>'),
    );

    Utilities::spConfigGuide($form, $form_state);

    return $form;

  }

  public function validateForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    if( $form_values['mo_saml_vertical_tabs__active_tab'] === 'edit-miniorange-support' ) {
      if( !\Drupal::service('email.validator')->isValid( $form_values['mo_saml_support_email_address'] ) ) {
        $form_state->setErrorByName('mo_saml_support_email_address', $this->t('The email address is not valid.'));
      }
      if( empty( $form_values['mo_saml_support_query'] ) ) {
        $form_state->setErrorByName('mo_saml_support_query', $this->t('The <b><u>Query</u></b> fields is mandatory'));
      }
    }  elseif( $form_values['mo_saml_vertical_tabs__active_tab'] === 'edit-miniorange-feature-request' ) {
      if( !\Drupal::service('email.validator')->isValid( $form_values['mo_saml_feature_email_address'] ) ) {
        $form_state->setErrorByName('mo_saml_feature_email_address', $this->t('The email address is not valid.'));
      }
      if( empty( $form_values['mo_saml_feature_query'] ) ) {
        $form_state->setErrorByName('mo_saml_feature_query', $this->t('The <b><u>Description</u></b> fields is mandatory'));
      }
    }
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    $email = ''; $phone = ''; $query = ''; $query_type = '';
    if( $form_values['mo_saml_vertical_tabs__active_tab'] === 'edit-miniorange-support' ) {
      $email = $form_values['mo_saml_support_email_address'];
      $phone = $form_values['mo_saml_support_phone_number'];
      $query = $form_values['mo_saml_support_query'];
      $query_type = 'Support';

    } elseif ( $form_values['mo_saml_vertical_tabs__active_tab'] === 'edit-miniorange-feature-request' ) {
      $email = $form_values['mo_saml_feature_email_address'];
      $phone = $form_values['mo_saml_feature_phone_number'];
      $query = $form_values['mo_saml_feature_query'];
      $query_type = 'New Feature Request';
    }
    Utilities::send_support_query( $email, $phone, $query, $query_type );
  }

}
