<?php
/**
 * @file
 * Contains miniOrange Customer class.
 */

/**
 * @file
 * This class represents configuration for customer.
 */
namespace Drupal\miniorange_saml_idp;

use GuzzleHttp\Exception\RequestException;


class MiniorangeSAMLCustomer {

  public $email;

  public $phone;

  public $customerKey;

  public $transactionId;

  public $password;

  public $otpToken;

  private $defaultCustomerId;

  private $defaultCustomerApiKey;

/**
   * Constructor.
   */
  public function __construct($email, $phone, $password, $otp_token) {
    $this->email = $email;
    $this->phone = $phone;
    $this->password = $password;
    $this->otpToken = $otp_token;
    $this->defaultCustomerId = "16555";
    $this->defaultCustomerApiKey = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";

  }


  /**
   * Check if customer exists.
   */
  public function checkCustomer() {
    $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/rest/customer/check-if-exists';
    $email = $this->email;
    $fields = array(
      'email' => $email,
    );
    $fields_string = json_encode($fields);

    try{
      $response = \Drupal::httpClient()
      ->request('POST',$url, [
        'body' => $fields_string,
        'allow_redirects' => TRUE,
        'http_errors' => FALSE,
        'decode_content'  => true,
        'verify' => FALSE,
        'headers' =>array(
          'Content-Type'=> 'application/json',
          'Authorization'=>'Basic',
        ),
      ]);

      return $response->getBody()->getContents();

    }
    catch (RequestException $exception)
    {

      \Drupal::logger('miniorange_saml_idp')->notice('Error at %method of %file: %error',
        array(
          '%method' => 'checkCustomer',
          '%file' => 'customer_setup.php',
          '%error' => $exception->getMessage(),
        ));
    }
  }


  /**
   * Get Customer Keys.
   */
  public function getCustomerKeys() {
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
        "apiKey" => 'CURL_ERROR',
        "token" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }

    $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/rest/customer/key';
    $email = $this->email;
    $password = $this->password;
    $fields = array(
      'email' => $email,
      'password' => $password,
    );
    $fields_string = json_encode($fields);
    try{
      $response = \Drupal::httpClient()
        ->request('POST',$url, [
          'body' => $fields_string,
          'allow_redirects' => TRUE,
          'http_errors' => FALSE,
          'decode_content'  => true,
          'verify' => FALSE,
          'headers' =>array(
            'Content-Type'=> 'application/json',
            'charset'=>'UTF - 8',
            'Authorization'=>'Basic',
          ),
        ]);

      return $response->getBody()->getContents();

    }
    catch (RequestException $exception)
    {
      \Drupal::logger('miniorange_saml_idp')->notice('Error at %method of %file: %error',
        array(
          '%method' => 'getCustomerKeys',
          '%file' => 'customer_setup.php',
          '%error' => $exception->getMessage(),
        ));
    }

  }


  function check_status($code){

    global $base_url;
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
      "status" => 'CURL_ERROR',
      "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }

    $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/api/backupcode/verify';
    $customerKey = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id');
    $apiKey =\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key');
    $currentTimeInMillis = round(microtime(TRUE) * 1000);
    $stringToHash = $customerKey . number_format($currentTimeInMillis, 0, '', '' ) . $apiKey;
    $hashValue = hash("sha512", $stringToHash);


    $fields = array(
      'code' => $code ,
      'customerKey' => $customerKey,
      'additionalFields' => array(
        'field1' => $base_url
      )
    );

    $fields_string = json_encode($fields);
    try{
      $response = \Drupal::httpClient()
        ->request('POST',$url, [
          'body' => $fields_string,
          'allow_redirects' => TRUE,
          'http_errors' => FALSE,
          'decode_content'  => true,
          'verify' => FALSE,
          'headers' =>array("Content-Type"=>"application/json", "Customer-Key"=>$customerKey,
            "Timestamp"=>$currentTimeInMillis, "Authorization"=>$hashValue)
        ]);
      return $response->getBody()->getContents();

    }
    catch (RequestException $exception)
    {
      echo 'Request Error:' . $exception->getMessage();
      exit();
    }


  }



    function update_status(){

        global $base_url;
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "status" => 'CURL_ERROR',
                "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
            ));
        }
        $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/api/backupcode/updatestatus';

        $customerKey = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id');
        $apiKey =\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key');
        $currentTimeInMillis = round(microtime(TRUE) * 1000);
        $stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
        $hashValue = hash("sha512", $stringToHash);
        $key = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_token');// @FIXME
        $code = Utilities::decrypt_data(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_license_key'),$key);
        $fields = array('code' => $code ,'customerKey' => $customerKey);
        $fields_string = json_encode($fields);

        try{
          $response = \Drupal::httpClient()
            ->request('POST',$url, [
              'body' => $fields_string,
              'allow_redirects' => TRUE,
              'http_errors' => FALSE,
              'decode_content'  => true,
              'verify' => FALSE,
              'headers' =>array("Content-Type"=>"application/json", "Customer-Key"=>$customerKey,
                "Timestamp"=>$currentTimeInMillis, "Authorization"=>$hashValue)
            ]);

          return $response->getBody()->getContents();

        }
        catch (RequestException $exception)
        {
          echo 'Request Error:' . $exception->getMessage();
        }


    }


  function ccl() {

    $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/rest/customer/license';

    $customerKey = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id');
    $apiKey =\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key');

    /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
    $currentTimeInMillis = round(microtime(TRUE) * 1000);

    /* Creating the Hash using SHA-512 algorithm */
    $stringToHash = $customerKey . number_format($currentTimeInMillis, 0, '', '' ) . $apiKey;
    $hashValue = hash("sha512", $stringToHash);

    $fields = array(
      'customerId' => $customerKey,
      'applicationName' => Utilities::getSamlRequestPlanName(),
    );

    $fields_string = json_encode($fields);

    try{
      $response = \Drupal::httpClient()
        ->request('POST',$url, [
          'body' => $fields_string,
          'allow_redirects' => TRUE,
          'http_errors' => FALSE,
          'decode_content'  => true,
          'verify' => FALSE,
          'headers' =>array("Content-Type"=>"application/json", "Customer-Key"=>$customerKey,
            "Timestamp"=>$currentTimeInMillis, "Authorization"=>$hashValue)
        ]);


     return $response->getBody()->getContents();
    }
    catch (RequestException $exception)
    {

      echo 'Request Error:' . $exception->getMessage();
    }

  }

  /**
   * Duplicate license check function
   */
  function mo_saml_vl( $code, $mo_email, $mo_days ) {

    if( !$this->check_internet_connection() ) {
      return '{"status":"SUCCESS","moConnection":"FALSE"}';
    }
    global $base_url;
    $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/api/backupcode/check';
    /* The customer Key and API Key */
    $customerKey = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id');
    $apiKey = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key');

    /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
    $currentTimeInMillis = round ( microtime ( TRUE ) * 1000 );

    /* Creating the Hash using SHA-512 algorithm */
    $stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
    $hashValue = hash ( "sha512", $stringToHash );

    $currentTimeInMillis = number_format ( $currentTimeInMillis, 0, '', '' );
    $fields = array (
      'code' => $code ,
      'customerKey' => $customerKey,
      'additionalFields' => array(
        'field1' => $base_url,
        'field2' => $mo_email,
        'field3' => $mo_days,
      )
    );

    $fields_string = json_encode($fields);

    try{
      $response = \Drupal::httpClient()
        ->request('POST',$url, [
          'body' => $fields_string,
          'allow_redirects' => TRUE,
          'http_errors' => FALSE,
          'decode_content'  => true,
          'verify' => FALSE,
          'headers' =>array("Content-Type"=>"application/json", "Customer-Key"=>$customerKey,
            "Timestamp"=>$currentTimeInMillis, "Authorization"=>$hashValue)
        ]);

      return $response->getBody()->getContents();

    }
    catch (RequestException $exception)
    {
      echo 'Request Error:' . $exception->getMessage();
    }

  }

  function check_internet_connection() {
    return ( bool ) @fsockopen('login.xecurify.com', 443, $iErrno, $sErrStr, 5 );
  }
}
