<?php
/**
 * @file
 * Contains miniOrange Support class.
 */

/**
 * @file
 * This class represents support information for customer.
 */
namespace Drupal\miniorange_saml_idp;

use GuzzleHttp\Exception\RequestException;
class MiniorangeSAMLIdpSupport {

	public $email;

	public $phone;

	public $query;

	/**
     * Constructor.
     */
	public function __construct($email, $phone, $query,$query_type) {
      $this->email = $email;
      $this->phone = $phone;
      $this->query = $query;
      $this->query_type = $query_type;
	}

	/**
	 * Send support query.
	 */
	public function sendSupportQuery() {
      $modules_info    = \Drupal::service('extension.list.module')->getExtensionInfo('miniorange_saml_idp');
      $modules_version = $modules_info['version'];
      $moPHPVersion    = phpversion();
      $moQueryType     =$this->query_type === 'New Feature Request' ? '(New Feature Request)' : '';

      $this->query = '[Drupal '. \Drupal::VERSION .' SAML IDP ' . ucfirst ( strtolower(MiniorangeSAMLIdpConstants::PLUGIN_VERSION ) ) . ' ' . $moQueryType . ' | ' . $modules_version . ' | PHP ' . $moPHPVersion . ' ] ' . $this->query;

      $fields = array (
        'company' => $_SERVER ['SERVER_NAME'],
        'email' => $this->email,
        'ccEmail' => 'drupalsupport@xecurify.com',
        'phone' => $this->phone,
        'query' => $this->query,
        'subject' => "Drupal-8 SAML IDP Premium Query",
      );
      $field_string = json_encode ($fields);
      $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/rest/customer/contact-us';

    try{
      $response = \Drupal::httpClient()
        ->request('POST',$url, [
          'body' => $field_string,
          'allow_redirects' => TRUE,
          'http_errors' => FALSE,
          'decode_content'  => true,
          'verify' => FALSE,
          'headers' =>array(
            'Content-Type'=> 'application/json',
            'Authorization'=>'Basic',
          ),
        ]);
    }
    catch (RequestException $exception)
    {

      \Drupal::logger('miniorange_saml_idp')->notice('Error at %method of %file: %error',
        array(
          '%method' => 'sendSupportQuery',
          '%file' => 'MiniorangeSAMLIdpSupport.php',
          '%error' => $exception->getMessage(),
        ));
      return false;
    }
    return true;
	}


	public static function liexpi($email,$query)
  {

    $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/api/notify/send';
    $ch = curl_init($url);

    $customerKey = "16555";
    $apiKey = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";

    $currentTimeInMillis = round(microtime(true) * 1000);
    $stringToHash = $customerKey . number_format($currentTimeInMillis, 0, '', '') . $apiKey;
    $hashValue = hash("sha512", $stringToHash);
    $customerKeyHeader = "Customer-Key: " . $customerKey;
    $timestampHeader = "Timestamp: " . number_format($currentTimeInMillis, 0, '', '');
    $authorizationHeader = "Authorization: " . $hashValue;
    $fromEmail = $email;
    $subject = "License expiry alert for Drupal 8 IDP with Multiple SP";

    $content = $query;

    $fields = array(
      'customerKey' => $customerKey,
      'sendEmail' => true,
      'email' => array(
        'customerKey' => $customerKey,
        'fromEmail' => $fromEmail,

        'fromName' => 'miniOrange',
        'toEmail' => $email,
        'toName' => $email,
        'subject' => $subject,
        'content' => $content
      ),
    );
    $field_string = json_encode($fields);

    try {
      $response = \Drupal::httpClient()
        ->request('POST', $url, [
          'body' => $field_string,
          'allow_redirects' => TRUE,
          'http_errors' => FALSE,
          'decode_content' => true,
          'verify' => FALSE,
          'headers' => array("Content-Type" => "application/json", "Customer-Key" => $customerKey,
            "Timestamp" => $currentTimeInMillis, "Authorization" => $hashValue)
        ]);
        $content = $response->getBody()->getContents();
    }

    catch (RequestException $exception){
      return json_encode(array("status" => 'ERROR', 'statusMessage' => $exception->getMessage()));
    }
    return;
  }
}
