<?php


namespace Drupal\miniorange_saml_idp;


class MiniorangeSAMLSP
{
  private $id;
  private $mo_idp_sp_name;
  private $mo_idp_sp_issuer;
  private $mo_idp_acs_url;
  private $mo_idp_slo_url;
  private $mo_idp_default_relayState;
  private $mo_idp_cert;
  private $mo_idp_cert_encrypt;
  private $mo_idp_nameid_format;
  private $mo_idp_response_signed;
  private $mo_idp_assertion_signed;
  private $mo_idp_logout_binding_type;
  private $mo_idp_encrypted_assertion;

  //Mapping fields
    private $mo_idp_nameid_attr_map;
    private $mo_idp_attr1_name;
    private $mo_idp_attr1_value;
    private $mo_idp_attr2_name;
    private $mo_idp_attr2_value;
    private $mo_idp_attr3_name;
    private $mo_idp_attr3_value;
    private $mo_idp_user_attributes;
    private $mo_idp_const_attributes;

  /**
   * MiniorangeSAMLSP constructor.
   * @param $id
   */
  public function __construct($id='new')
  {
    if ($id != 'new') {
        $connection = \Drupal::database();
        $query = $connection->query("SELECT * FROM {miniorange_saml_idp_spdata} WHERE id='" . $id . "'");
        $result = $query->fetchAll();
        if (sizeof($result) > 0) {
          foreach ($this as $key => $value) {
            $this->$key = $result[0]->$key;
          }
        }
    }

    else{
      $this->id = $id;
    }
  }

  public function insert($sp_name, $entityID_issuer, $acs_url, $saml_x509_certificate, $encrypted_certificate, $logout_url, $nameid_format, $relay_state, $is_assertion_signed, $is_encrypted_assertion, $is_response_signed, $http_binding_value)
  {
      $connection = \Drupal::database();
      $fields_array = array(
          'mo_idp_sp_name' => $sp_name, 'mo_idp_sp_issuer' => $entityID_issuer,
          'mo_idp_acs_url' => $acs_url, 'mo_idp_cert' => $saml_x509_certificate,
          'mo_idp_cert_encrypt' => $encrypted_certificate, 'mo_idp_slo_url' => $logout_url,
          'mo_idp_nameid_format' => $nameid_format, 'mo_idp_default_relayState' => $relay_state,
          'mo_idp_assertion_signed' => $is_assertion_signed, 'mo_idp_encrypted_assertion' => $is_encrypted_assertion,
          'mo_idp_response_signed' => $is_response_signed, 'mo_idp_logout_binding_type' => $http_binding_value,
      );

      if ($this->id == 'new' || is_null($this->id)) {
          $query = $connection->query("SELECT * FROM {miniorange_saml_idp_spdata}");
          $data = $query->fetchAll();
          foreach ($data as $spinfo) {
              if ($spinfo->mo_idp_sp_name == $sp_name) {
                  \Drupal::messenger()->addError(t('The name of the Service Provider already exists, try with a unique name.'));
                  return FALSE;
              }
              if ($spinfo->mo_idp_sp_issuer == $entityID_issuer){
                \Drupal::messenger()->addError(t('SP with same entity id/Issuer already exists.'));
                return FALSE;
              }
          }

          $db_insert = $connection->insert('miniorange_saml_idp_spdata')->fields($fields_array)->execute();
          $res = $this->sqlQueryWithWhereClause('id','mo_idp_sp_name',$sp_name);
          $id = $res[0]->id;
          return $id;
      }
      else
      {
          $db_update = $connection->update('miniorange_saml_idp_spdata')->fields($fields_array)->condition('id', $this->id, '=')->execute();
      }
      return TRUE;
  }

  public function update_metadata($file,$type)
  {
      $document = new \DOMDocument();
      $document->loadXML( $file );
      restore_error_handler();
      $first_child = $document->firstChild;
      if( !empty( $first_child ) ) {
          $metadata = new MetadataReader($document);
          $service_providers = $metadata->getServiceProviders();

          if( empty( $service_providers ) ) {
              \Drupal::messenger()->addMessage(t('Please provide a valid metadata '.$type.'.'),'error');
              return FALSE;
          }

          $connection = \Drupal::database();

          foreach( $service_providers as $key => $sp ) {

              $logout_url = $sp->getLogoutURL( 'HTTP-Redirect' );

              if( empty( $logout_url ) ) {
                  $logout_url = $sp->getLogoutURL( 'HTTP-POST' );
              }

              $entityID_issuer = $sp->getEntityID();
              $acs_url = $sp->getAcsURL();
              $is_assertion_signed = $sp->getAssertionsSigned() == 'true' ? 1 : 0;
              $saml_x509_certificate = is_array( $sp->getSigningCertificate() ) ? '' : $sp->getSigningCertificate();

              $id = $connection->query("SELECT max(id) as id from {miniorange_saml_idp_spdata}")->fetchAll();
              $next_id = $id[0]->id +1;
              $sp_name = 'Service Provider '. $next_id;

              $fields_array = array(
                  'mo_idp_sp_name' => $sp_name, 'mo_idp_sp_issuer' => $entityID_issuer,
                  'mo_idp_acs_url' => $acs_url, 'mo_idp_cert' => $saml_x509_certificate,
                  'mo_idp_slo_url' => $logout_url, 'mo_idp_assertion_signed' => $is_assertion_signed,
                  'mo_idp_response_signed' => 0, 'mo_idp_encrypted_assertion' => 0,
                  'mo_idp_nameid_format' => '1.1:nameid-format:emailAddress', 'mo_idp_logout_binding_type' => 0,
              );

              //Checking if IDP with same entity-id already exists.
              $query = $connection->query("SELECT * FROM {miniorange_saml_idp_spdata}");
              $data = $query->fetchAll();
              foreach ($data as $spinfo) {
                if($spinfo->mo_idp_sp_issuer == $entityID_issuer){
                  \Drupal::messenger()->addError(t('SP with same entity-id/Issuer already exists.'));
                  return FALSE;
                }
              }

              $dbinsert = $connection->insert('miniorange_saml_idp_spdata')->fields($fields_array)->execute();
          }
          \Drupal::messenger()->addMessage(t('Service Provider Configuration successfully saved.'));
          $connection = \Drupal::database();
          $id = $connection->query("SELECT max(id) as id from {miniorange_saml_idp_spdata}")->fetchAll();
          $next_id = $id[0]->id;
          return $next_id;
      }else {
          \Drupal::messenger()->addMessage(t('Please provide a valid metadata '.$type.'.'),'error');
          return FALSE;
      }
  }

    public function updateMapping($nameid_attr_value,$attr1_name,$attr1_value,$attr2_name,$attr2_value,$attr3_name,$attr3_value,$user_attribute_mapping,$const_attribute_mapping)
    {
        $connection = \Drupal::database();
        $fields_array = array(
            'mo_idp_nameid_attr_map' => $nameid_attr_value,
            'mo_idp_attr1_name' => $attr1_name, 'mo_idp_attr1_value' => $attr1_value,
            'mo_idp_attr2_name' => $attr2_name, 'mo_idp_attr2_value' => $attr2_value,
            'mo_idp_attr3_name' => $attr3_name, 'mo_idp_attr3_value' => $attr3_value,
            'mo_idp_user_attributes' => $user_attribute_mapping,
            'mo_idp_const_attributes' => $const_attribute_mapping,
        );
        $dbupdate = $connection->update('miniorange_saml_idp_spdata')->fields($fields_array)->condition('id', $this->id, '=')->execute();
    }

    public function delete()
    {
        \Drupal::database()->delete('miniorange_saml_idp_spdata')->condition('id', $this->id)->execute();
    }

    public function spCount()
    {
        $connection = \Drupal::database();
        $query = $connection->query("SELECT count(*) as count FROM {miniorange_saml_idp_spdata}");
        $result = $query->fetchAll();
        $sp_count = $result[0]->count;
        return $sp_count;
    }

    public function getTable()
    {
        return \Drupal::database()->query("SELECT * FROM {miniorange_saml_idp_spdata}")->fetchall();
    }

    public function sqlQuery($select_field){
        $connection = \Drupal::database();
        $query = $connection->query("SELECT " . $select_field . " FROM {miniorange_saml_idp_spdata}");
        $result = $query->fetchAll();
        return $result;
    }

    public function sqlQueryWithWhereClause($select_field, $where_field, $where_field_value)
    {
        $connection = \Drupal::database();
        $query = $connection->query("SELECT " . $select_field . " FROM {miniorange_saml_idp_spdata} WHERE ". $where_field ." = '" . $where_field_value . "'");
        $result = $query->fetchAll();
        return $result;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return mixed
     */
    public function getMoIdpSpName()
    {
        return $this->mo_idp_sp_name;
    }

    /**
     * @return mixed
     */
    public function getMoIdpSpIssuer()
    {
        return $this->mo_idp_sp_issuer;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAcsUrl()
    {
        return $this->mo_idp_acs_url;
    }

    /**
     * @return mixed
     */
    public function getMoIdpSloUrl()
    {
        return $this->mo_idp_slo_url;
    }

    /**
     * @return mixed
     */
    public function getMoIdpDefaultRelayState()
    {
        return $this->mo_idp_default_relayState;
    }

    /**
     * @return mixed
     */
    public function getMoIdpCert()
    {
        return $this->mo_idp_cert;
    }

    /**
     * @return mixed
     */
    public function getMoIdpCertEncrypt()
    {
        return $this->mo_idp_cert_encrypt;
    }

    /**
     * @return mixed
     */
    public function getMoIdpNameidFormat()
    {
        return $this->mo_idp_nameid_format;
    }

    /**
     * @return mixed
     */
    public function getMoIdpResponseSigned()
    {
        return $this->mo_idp_response_signed;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAssertionSigned()
    {
        return $this->mo_idp_assertion_signed;
    }

    /**
     * @return mixed
     */
    public function getMoIdpLogoutBindingType()
    {
        return $this->mo_idp_logout_binding_type;
    }

    /**
     * @return mixed
     */
    public function getMoIdpEncryptedAssertion()
    {
        return $this->mo_idp_encrypted_assertion;
    }

    /**
     * @return mixed
     */
    public function getMoIdpNameidAttrMap()
    {
        return $this->mo_idp_nameid_attr_map;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAttr1Name()
    {
        return $this->mo_idp_attr1_name;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAttr1Value()
    {
        return $this->mo_idp_attr1_value;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAttr2Name()
    {
        return $this->mo_idp_attr2_name;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAttr2Value()
    {
        return $this->mo_idp_attr2_value;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAttr3Name()
    {
        return $this->mo_idp_attr3_name;
    }

    /**
     * @return mixed
     */
    public function getMoIdpAttr3Value()
    {
        return $this->mo_idp_attr3_value;
    }

    /**
     * @return mixed
     */
    public function getMoIdpUserAttributes()
    {
        return $this->mo_idp_user_attributes;
    }

    /**
     * @return mixed
     */
    public function getMoIdpConstAttributes()
    {
        return $this->mo_idp_const_attributes;
    }
}
