<?php
/**
 * @package    miniOrange
 * @subpackage Plugins
 * @license    GNU/GPLv3
 * @copyright  Copyright 2015 miniOrange. All Rights Reserved.
 *
 *
 * This file is part of miniOrange Joomla SAML IDP plugin.
 *
 * miniOrange Joomla SAML IDP plugin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * miniOrange Joomla IDP plugin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with miniOrange SAML plugin.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Drupal\miniorange_saml_idp;
use DOMElement;
use DOMDocument;
use DOMNode;
use DOMXPath;
use Drupal\miniorange_saml_idp\XMLSecurityKey;
use Drupal\miniorange_saml_idp\XMLSecurityDSig;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Render\Markup;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpSupport;
use Drupal\field\FieldConfigInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use \Drupal\user\Entity\User;


class Utilities {

    public static function isCustomerRegistered(&$form, $form_state)
    {
        global $base_url;
        $register_url = $base_url . '/admin/config/people/miniorange_saml_idp/customer_setup/';
        if(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email') == NULL
            || \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id') == NULL
            || \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_token') == NULL
            || \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key') == NULL)
        {
            $form['markup_reg_msg'] = array(
                '#markup' => '<div class="mo_saml_configure_message">You need to <a href="'.$register_url.'" >login</a> using miniOrange account before using this module.</div>',
            );

            return TRUE;
        }
        else if((\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_license_key') == NULL) &&
            (\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email') != NULL))
        {
            $form['markup_msg'] = array(
                '#markup' => '<div class="mo_saml_configure_message">You need to <a href="'.$register_url.'" >verify your license</a> key before using this module.</div>'
            );
            return TRUE;
        }else{
            return FALSE;
        }
    }

    public static function spConfigGuide(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $form['miniorange_idp_guide_link1'] = array(
            '#markup' => '<div class="mo_saml_table_guide_layout_1 mo_saml_container_2">
                    <div style="font-size: 15px;">To see detailed documentation of how to configure
                    Drupal SAML IdP with any Service Provider</div></br>',
        );

        $mo_Tableau = Markup::create('<strong><a href="https://plugins.miniorange.com/configure-tableau-sp-as-sp-and-drupal-8-as-idp" class="mo_guide_text-color" target="_blank">Tableau</a></strong>');
        $mo_Zendesk = Markup::create('<strong><a href="https://plugins.miniorange.com/zendesk-single-sign-sso-for-drupal-8-idp" class="mo_guide_text-color" target="_blank">Zendesk</a></strong>');
        $mo_Workplace_by_facebook = Markup::create('<strong><a href="https://plugins.miniorange.com/guide-drupal-idp-workplace-sp" class="mo_guide_text-color" target="_blank">Workplace by Facebook</a></strong>');
        $mo_Canvas_LMS = Markup::create('<strong><a href="https://plugins.miniorange.com/guide-to-configure-canvas-lms-as-sp-and-drupal-as-idp" class="mo_guide_text-color" target="_blank">Canvas LMS</a></strong>');
        $mo_Owncloud = Markup::create('<strong><a class="mo_guide_text-color" href="https://plugins.miniorange.com/configure-owncloud-sp-and-drupal-as-idp" target="_blank">Owncloud</a></strong>');
        $mo_Inkling = Markup::create('<strong><a class="mo_guide_text-color" href="https://plugins.miniorange.com/configure-inkling-as-sp-for-drupal-8-idp" target="_blank">Inkling</a></strong>');
        $mo_appstream2 = Markup::create('<strong><a class="mo_guide_text-color" href="https://plugins.miniorange.com/guide-to-setup-drupal-as-idp-and-aws-appstream2-as-sp" target="_blank">AppStream2</a></strong>');
        $mo_for_any_other_sp = Markup::create('<strong><a href="https://plugins.miniorange.com/guide-enable-miniorange-drupal-saml-idp" class="mo_guide_text-color" target="_blank">For any Other SP</a></strong>');

        $mo_table_content = array(
          array($mo_Zendesk,$mo_Tableau),
          array($mo_Canvas_LMS,$mo_Workplace_by_facebook),
          array($mo_Inkling,$mo_Owncloud),
          array($mo_appstream2,$mo_for_any_other_sp),
        );

        $header = array( array(
            'data' => Markup::create('<div style="text-align: center">Service Provider Setup Guides</div>') ,
          'colspan' => 2,
            ),
        );

        $form['mo_saml_idp_sp_guide_links'] = array(
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $mo_table_content,
          '#responsive' => TRUE ,
        );

        self::faq($form, $form_state);
        $form['miniorange_idp_guide_link_end'] = array(
            '#markup' => '</div>',
        );
    }

    public static function faq(&$form, &$form_state){

        $form['miniorange_faq'] = array(
            '#markup' => '<div class="mo_saml_text_center"><b></b>
                          <a class="mobtn mo_btn_primary-faq btn-large mo_faq_button_left" href="https://faq.miniorange.com/kb/drupal/saml-drupal/" target="_blank">FAQs</a>
                          <b></b><a class="mobtn mo_btn_primary-faq btn-large mo_faq_button_right" href="https://forum.miniorange.com/" target="_blank">Ask questions on forum</a></div>',
        );
    }


    /**
     * SEND SUPPORT QUERY | NEW FEATURE REQUEST | DEMO REQUEST
     * @param $email
     * @param $phone
     * @param $query
     * @param $query_type = Support | Demo Request | New Feature Request
     */
    public static function send_support_query( $email, $phone, $query, $query_type )   {
      $support = new MiniorangeSamlIdpSupport( $email, $phone, $query, $query_type );
      $support_response = $support->sendSupportQuery();
      if ( $support_response ) {
        \Drupal::messenger()->addMessage(t('Thanks for getting in touch! We will get back to you shortly.'));
      } else {
        \Drupal::messenger()->addMessage(t('Error submitting the support query. Please send us your query at <a href="mailto:info@xecurify.com">info@xecurify.com</a>.'), 'error');
      }
    }


    Public static function getClassNameForImport_Export(){
        $tab_class_name = array(
            "Identity_Provider" => 'mo_options_enum_identity_provider',
            "Service_Provider"  => 'mo_options_enum_service_provider',
            "Attribute_Mapping"  => 'mo_options_enum_attribute_mapping',
            "Custom_Certificate"  => 'mo_options_enum_custom_certificate',
        );
        return $tab_class_name;
    }

    Public static function getVariableArrayForImport_Export($class_name){
        if($class_name == "mo_options_enum_identity_provider") {
            $class_object = array(
                'IdP_Entity_ID' => 'miniorange_saml_idp_entity_id_issuer',
                'IdP_Login_Url' => 'miniorange_saml_idp_login_url',
            );
        }
        else if($class_name == "mo_options_enum_service_provider") {
            $class_object = array(
                'Sevice_Provider_name' => 'mo_idp_sp_name',
                'SP_Entity_ID_or_Issuer' => 'mo_idp_sp_issuer',
                'ACS_URL' => 'mo_idp_acs_url',
                'Certificate_Signed' => 'mo_idp_cert',
                'Certificate_Encrypted' => 'mo_idp_cert_encrypt',
                'Logout_URL' => 'mo_idp_slo_url',
                'Name_ID_format' => 'mo_idp_nameid_format',
                'Relay_State' => 'mo_idp_default_relayState',
                'Assertion_Signed' => 'mo_idp_assertion_signed',
                'Encrypted_Assertion' => 'mo_idp_encrypted_assertion',
                'Response_Signed' => 'mo_idp_response_signed',
                'HTTP_Binding' => 'mo_idp_logout_binding_type',
            );
        }
        else if($class_name == "mo_options_enum_attribute_mapping") {
            $class_object = array(
                'NameID_Attribute' => 'mo_idp_nameid_attr_map',
                'Attribute1_Name' => 'mo_idp_attr1_name',
                'Attribute1_Value' => 'mo_idp_attr1_value',
                'Attribute2_Name' => 'mo_idp_attr2_name',
                'Attribute2_Value' => 'mo_idp_attr2_value',
                'Attribute3_Name' => 'mo_idp_attr3_name',
                'Attribute3_Value' => 'mo_idp_attr3_value',
                'User_Profile_Attributes' => 'mo_idp_user_attributes',
                'Constant_Attributes' => 'mo_idp_const_attributes',
            );
        }
        else if($class_name == "mo_options_enum_custom_certificate") {
            $class_object = array(
                'Private_Certificate' => 'miniorange_saml_private_certificate',
                'Public_Certificate' => 'miniorange_saml_publ_certificate',
            );
        }
        return $class_object;
    }


    public static function isCurlInstalled() {
      if (in_array('curl', get_loaded_extensions())) {
        return 1;
      }
      else {
        return 0;
      }
    }

    public static function generateID() {
        return '_' . self::stringToHex(self::generateRandomBytes(21));
    }

    public static function stringToHex($bytes) {
        $ret = '';
        for($i = 0; $i < strlen($bytes); $i++) {
            $ret .= sprintf('%02x', ord($bytes[$i]));
        }
        return $ret;
    }

    public static function generateRandomBytes($length, $fallback = TRUE) {
        return openssl_random_pseudo_bytes($length);
    }

    public static function createAuthnRequest($acsUrl, $issuer, $force_authn = 'false') {
        $requestXmlStr = '<?xml version="1.0" encoding="UTF-8"?>' .
                        '<samlp:AuthnRequest xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" ID="' . self::generateID() .
                        '" Version="2.0" IssueInstant="' . self::generateTimestamp() . '"';
        if( $force_authn == 'true') {
            $requestXmlStr .= ' ForceAuthn="true"';
        }
        $requestXmlStr .= ' ProtocolBinding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" AssertionConsumerServiceURL="' . $acsUrl .
                        '" ><saml:Issuer xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion">' . $issuer . '</saml:Issuer></samlp:AuthnRequest>';
        $deflatedStr = gzdeflate($requestXmlStr);
        $base64EncodedStr = base64_encode($deflatedStr);
        $urlEncoded = urlencode($base64EncodedStr);
        return $urlEncoded;
    }

	public static function createLogoutRequest($nameId, $sessionIndex = '', $issuer, $destination,$slo_binding_type){

		$requestXmlStr='<?xml version="1.0" encoding="UTF-8"?>' .
				'<samlp:LogoutRequest xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" ID="'. self::generateID() .
				'" IssueInstant="' . self::generateTimestamp() .'" Version="2.0" Destination="'. $destination . '">
						<saml:Issuer xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion">' . $issuer . '</saml:Issuer>
						<saml:NameID xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion">'. $nameId . '</saml:NameID>';
		if(!empty($sessionIndex)) {
			$requestXmlStr .= '<samlp:SessionIndex>' . $sessionIndex . '</samlp:SessionIndex>';
		}
		$requestXmlStr .= '</samlp:LogoutRequest>';

		if(empty($slo_binding_type) || $slo_binding_type == 'HTTP-Redirect') {

            $deflatedStr = gzdeflate($requestXmlStr);
            $base64EncodedStr = base64_encode($deflatedStr);
            $urlEncoded = urlencode($base64EncodedStr);
            $requestXmlStr = $urlEncoded;
        }

		return $requestXmlStr;
	}


    public static function generateTimestamp($instant = NULL) {
        if($instant === NULL) {
            $instant = time();
        }
        return gmdate('Y-m-d\TH:i:s\Z', $instant);
    }

    public static function xpQuery(DOMNode $node, $query){
        static $xpCache = NULL;

        if ($node instanceof DOMDocument) {
            $doc = $node;
        } else {
            $doc = $node->ownerDocument;
        }

        if ($xpCache === NULL || !$xpCache->document->isSameNode($doc)) {
            $xpCache = new DOMXPath($doc);
            $xpCache->registerNamespace('soap-env', 'http://schemas.xmlsoap.org/soap/envelope/');
            $xpCache->registerNamespace('saml_protocol', 'urn:oasis:names:tc:SAML:2.0:protocol');
            $xpCache->registerNamespace('saml_assertion', 'urn:oasis:names:tc:SAML:2.0:assertion');
            $xpCache->registerNamespace('saml_metadata', 'urn:oasis:names:tc:SAML:2.0:metadata');
            $xpCache->registerNamespace('ds', 'http://www.w3.org/2000/09/xmldsig#');
            $xpCache->registerNamespace('xenc', 'http://www.w3.org/2001/04/xmlenc#');
        }

        $results = $xpCache->query($query, $node);
        $ret = array();
        for ($i = 0; $i < $results->length; $i++) {
            $ret[$i] = $results->item($i);
        }

        return $ret;
    }

    public static function parseNameId(DOMElement $xml)
    {
        $ret = array('Value' => trim($xml->textContent));

        foreach (array('NameQualifier', 'SPNameQualifier', 'Format') as $attr) {
            if ($xml->hasAttribute($attr)) {
                $ret[$attr] = $xml->getAttribute($attr);
            }
        }

        return $ret;
    }

    public static function xsDateTimeToTimestamp($time)
    {
        $matches = array();

        // We use a very strict regex to parse the timestamp.
        $regex = '/^(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)T(\\d\\d):(\\d\\d):(\\d\\d)(?:\\.\\d+)?Z$/D';
        if (preg_match($regex, $time, $matches) == 0) {
            echo sprintf("invalid SAML2 timestamp passed to xsDateTimeToTimestamp: ". xss::filter($time));
            exit;
        }

        // Extract the different components of the time from the  matches in the regex.
        // intval will ignore leading zeroes in the string.
        $year   = intval($matches[1]);
        $month  = intval($matches[2]);
        $day    = intval($matches[3]);
        $hour   = intval($matches[4]);
        $minute = intval($matches[5]);
        $second = intval($matches[6]);

        // We use gmmktime because the timestamp will always be given
        //in UTC.
        $ts = gmmktime($hour, $minute, $second, $month, $day, $year);

        return $ts;
    }

    public static function dayleft($days) {

        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content="Hello,<br /><br />This email is to notify you that your 1 year license for Drupal SAML IDP Single Sign-On Module will expire
        in <b>". $days ."days</b>. The module will stop working after the licensed period expires.<br /><br />You will need to renew your license to continue using the module on your website: <b>". $site_name ."</b>.
        <br /><br />Click <a href='". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/login?username=". $username ."&redirectUrl=" . MiniorangeSAMLIdpConstants::BASE_URL . "/moas/initializepayment&requestOrigin=renew_user_license>here'</a> to renew the license.<br /><br />Thanks,<br />miniOrange Team";
        $subject="License Expiry - Drupal IDP SAML Single Sign-On Module |". $site_name;
        self::send_mail($subject,$content);
    }

   public static function send_mail($subject,$content) {

        $hostname = MiniorangeSAMLIdpConstants::BASE_URL;
        $url = MiniorangeSAMLIdpConstants::BASE_URL . '/moas/api/notify/send';
        $ch = curl_init($url);
        $customerKey = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id');
        $apiKey =\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key');

        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $currentTimeInMillis= round(microtime(true) * 1000);
        $stringToHash       = $customerKey .  number_format($currentTimeInMillis, 0, '', '') . $apiKey;
        $hashValue          = hash("sha512", $stringToHash);
        $customerKeyHeader  = "Customer-Key: " . $customerKey;
        $timestampHeader    = "Timestamp: " .  number_format($currentTimeInMillis, 0, '', '');
        $authorizationHeader= "Authorization: " . $hashValue;
        $toEmail            = $username;

        $fromEmail="info@xecurify.com";
        $fields = array(
            'customerKey'   => $customerKey,
            'sendEmail'     => true,
            'email'         => array(
                'customerKey'   => $customerKey,
                'fromEmail'     => $fromEmail,
                'bccEmail'      => "drupalsupport@xecurify.com",
                'fromName'      => 'miniOrange',
                'toEmail'       => $toEmail,
                'toName'        => $toEmail,
                'subject'       => $subject,
                'content'       => $content
            ),
        );

        $field_string = json_encode($fields);

        curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
        curl_setopt( $ch, CURLOPT_ENCODING, "" );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );    # required for https urls
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);

        curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", $customerKeyHeader,
            $timestampHeader, $authorizationHeader));
        curl_setopt( $ch, CURLOPT_POST, true);
        curl_setopt( $ch, CURLOPT_POSTFIELDS, $field_string);
        $content = curl_exec($ch);

        if(curl_errno($ch)){
            return json_encode(array("status"=>'ERROR','statusMessage'=>curl_error($ch)));
        }
        curl_close($ch);
        return json_encode(array("status"=>'SUCCESS','statusMessage'=>'SUCCESS'));


    }
    public static function extractStrings(DOMElement $parent, $namespaceURI, $localName) {
        $ret = array();
        for ( $node = $parent->firstChild; $node !== NULL; $node = $node->nextSibling ) {
            if ( $node->namespaceURI !== $namespaceURI || $node->localName !== $localName ) {
                continue;
            }
            $ret[] = trim($node->textContent);
        }

        return $ret;
    }

    public static function limit($exp_date) {

        global $base_url;
        $site_name = $base_url;
        $content="Hello,<br /><br />Your 1 year license for Drupal IDP SAML Single Sign-On Module has expired on <b>".$exp_date."</b>. You have not renewed your license within the 30 days grace period given to you. The module has been deactivated on your website <b>". $site_name ."</b>.
            <br />Thanks,<br />miniOrange Team" ;
        $subject="License Expired - Drupal IDP SAML Single Sign-On Premium Module |". $site_name;
        self::send_mail( $subject, $content );
    }


    public static function limitmid( $users ) {

        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content="Hello,<br /><br />You have purchased license for Drupal SAML IDP Single Sign-On Module for <b>". $users ." users</b>. The number of users on your site has exceeded the license limit 15 days ago. You should upgrade your license for the module on your website: <b> ". $site_name ."</b>.<br/>If you fail to upgrade the license within next 15 days, the new users beyond the license limit would not be able to Single Sign-On.<br /> <br /> Click <a href='" . MiniorangeSAMLIdpConstants::BASE_URL . "/moas/login?username=". $username ."&redirectUrl=". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/initializepayment&requestOrigin=2fa_add_user_plan'>here</a> to upgrade the license.<br /><br />Thanks,<br />miniOrange Team";

        $subject="Exceeded License Limit For No Of Users - Drupal IDP SAML Single Sign-On Premium |" . $site_name;
        self::send_mail($subject,$content);
    }

    public static function limitend($users) {

        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content   = "Hello,<br /><br />You have purchased license for Drupal SAML IDP Single Sign-On Module for <b>". $users ."users</b>. The number of users on your site has exceeded the license limit 30 days ago. You have not upgraded your license in the 30 days grace period provided to you.The new users beyond the license limit would not be able to Single Sign-On now.<br /> <br /> Click <a href='". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/login?username=". $username ."&redirectUrl=". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/initializepayment&requestOrigin=2fa_add_user_plan'>here</a> to upgrade the license.<br /><br />Thanks,<br />miniOrange Team";

        $subject = "Exceeded License Limit For No Of Users - Drupal IDP SAML Single Sign-On Premium |" . $site_name;
        self::send_mail($subject,$content);
    }

    public static function peruser ( $percent, $users ) {
        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content = "Hello,<br /><br />You have purchased license for Drupal SAML IDP Single Sign-On Module for <b> ". $users ." users</b>. As number of users on your site have grown to more than ". $percent ." percent of ". $users ." users now, you should upgrade your license for miniOrange module on your website: <b>". $site_name ."</b>.<br /> <br /> Click <a href='". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/login?username=". $username ."&redirectUrl=". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/initializepayment&requestOrigin=2fa_add_user_plan'>here</a> to upgrade the license.<br /><br />Thanks,<br />miniOrange Team";
        $subject="License Limit Reached ". $percent ." percent Of Allowed No Of Users - Drupal IDP SAML Single Sign-On Premium |" . $site_name ;

        self::send_mail($subject,$content);
    }

    public static function tenuser( $users, $userin ) {

        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content = "Hello,<br /><br />You have purchased license for Drupal SAML IDP Single Sign-On Module for <b> ". $users ."users</b>.
            As number of users on your site have grown to ". $userin ." users now, you should upgrade your license for miniOrange module on your website: <b>". $site_name ."</b>.
            <br/><br /> Click <a href='". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/login?username=". $username ."&redirectUrl=". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/initializepayment&requestOrigin=2fa_add_user_plan'>here</a> to upgrade the license.<br />
            <br />Thanks,<br />miniOrange Team";

        $subject = "Ten users left to reach Limit Of Allowed No Of Users - Drupal-8 IDP SAML Single Sign-On Premium Module | " . $site_name;
        self::send_mail($subject, $content);
    }

    public static function checkupdate( $user_count ) {
            $customer  = new MiniorangeSAMLCustomer(NULL, NULL, NULL, NULL );
            $content   = json_decode( $customer->ccl(), true );
            $noOfUsers = $content['noOfUsers'];
            if( $noOfUsers != $user_count ) {
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniOrange_saml_idp_user_count', $noOfUsers)->save();
                return TRUE;

            }else {
                return FALSE;
            }
    }

  public static function licensevalidity($expire) {
      $customer   = new MiniorangeSAMLCustomer(NULL, NULL, NULL, NULL);
      $content = json_decode($customer->ccl(), true);
      $license_exp = strtotime($content['licenseExpiry']);

      if($license_exp > $expire) {
          \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniOrange_saml_idp_l_exp', $license_exp)->save();
          $connection = \Drupal::database();
          $update  = $connection->update('miniorange_saml_idp_user')
            ->fields(['UserIn' => 0,])
            ->execute();
          return TRUE;
      }else {
          return FALSE;
      }
  }

    /*
     * Remove Account
     */
    public static function dellkey()
    {
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_license_key')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_customer_admin_email')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_customer_admin_token')->save();
        $current_status = "MOIDP_VERIFY_LICENSE";
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', $current_status)->save();
        $val = 0;
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exli_thirtyabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exli_fifteenabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exli_fiveabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_exliabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_grc_fiveabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_grc_tenabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_grc_fifteenabc', $val)->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('ue_count', $val)->save();
    }

    public static function limitreach($users,$userin) {

        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content="Hello,<br /><br />You have purchased license for Drupal SAML IDP Single Sign-On Module for <b> ". $users ." users</b>. As number of users on your site has reached". $userin ." users now, you should upgrade your license for miniOrange module on your website: <b>". $site_name ."</b>.<br/>If you fail to upgrade the license within next 30 days, the new users beyond the license limit would not be able to Single Sign-On.<br /> <br /> Click <a href='". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/login?username=". $username ."&redirectUrl=". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/initializepayment&requestOrigin=2fa_add_user_plan'>here</a> to upgrade the license.<br /><br />Thanks,<br />miniOrange Team";

        $subject="Reached License Limit For No Of Users - Drupal IDP SAML Single Sign-On Premium |" . $site_name;
        self::send_mail($subject,$content);
    }
    public static function validateElement(DOMElement $root)
    {
        /* Create an XML security object. */
        $objXMLSecDSig = new \Drupal\miniorange_saml_idp\XMLSecurityDSig();
        /* Both SAML messages and SAML assertions use the 'ID' attribute. */
        $objXMLSecDSig->idKeys[] = 'ID';

        /* Locate the XMLDSig Signature element to be used. */
        $signatureElement = self::xpQuery($root, './ds:Signature');

        if (count($signatureElement) === 0) {
            //echo "here count";exit;
            /* We don't have a signature element to validate. */
            return FALSE;
        } elseif (count($signatureElement) > 1) {
            echo sprintf("XMLSec: more than one signature element in root.");
            exit;
        }

        $signatureElement = $signatureElement[0];
        $objXMLSecDSig->sigNode = $signatureElement;

        /* Canonicalize the XMLDSig SignedInfo element in the message. */
        $objXMLSecDSig->canonicalizeSignedInfo();

       /* Validate referenced xml nodes. */
        if (!$objXMLSecDSig->validateReference()) {
            echo sprintf("XMLsec: digest validation failed");
            exit;
        }

        /* Check that $root is one of the signed nodes. */
        $rootSigned = FALSE;
        /** @var DOMNode $signedNode */
        foreach ($objXMLSecDSig->getValidatedNodes() as $signedNode) {
            if ($signedNode->isSameNode($root)) {
                $rootSigned = TRUE;
                break;
            } elseif ($root->parentNode instanceof DOMDocument && $signedNode->isSameNode($root->ownerDocument)) {
                /* $root is the root element of a signed document. */
                $rootSigned = TRUE;
                break;
            }
        }

        if (!$rootSigned) {
            echo sprintf("XMLSec: The root element is not signed.");
            exit;
        }

        /* Now we extract all available X509 certificates in the signature element. */
        $certificates = array();
        foreach (self::xpQuery($signatureElement, './ds:KeyInfo/ds:X509Data/ds:X509Certificate') as $certNode) {
            $certData = trim($certNode->textContent);
            $certData = str_replace(array("\r", "\n", "\t", ' '), '', $certData);
            $certificates[] = $certData;
        }

        $ret = array(
            'Signature' => $objXMLSecDSig,
            'Certificates' => $certificates,
            );


        return $ret;
    }

	public static function createLogoutResponse( $inResponseTo, $issuer, $destination, $slo_binding_type = 'HTTP-Redirect'){

        $requestXmlStr='<?xml version="1.0" encoding="UTF-8"?>
        <samlp:LogoutResponse xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" ID="' . self::generateID() . '" Version="2.0" IssueInstant="' . self::generateTimestamp() . '" Destination="' . htmlspecialchars($destination) . '" InResponseTo="' . $inResponseTo . '">
        <saml:Issuer xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion">' . htmlspecialchars($issuer) . '</saml:Issuer><samlp:Status><samlp:StatusCode Value="urn:oasis:names:tc:SAML:2.0:status:Success"/></samlp:Status></samlp:LogoutResponse>';

        if(empty($slo_binding_type) || $slo_binding_type == 'HTTP-Redirect') {
            $deflatedStr = gzdeflate($requestXmlStr);
            $base64EncodedStr = base64_encode($deflatedStr);
            $urlEncoded = urlencode($base64EncodedStr);
            $requestXmlStr = $urlEncoded;
        }
        return $requestXmlStr;
    }

    public static function validateSignature(array $info, XMLSecurityKey $key)
    {

        /** @var XMLSecurityDSig $objXMLSecDSig */
        $objXMLSecDSig = $info['Signature'];

        $sigMethod = self::xpQuery($objXMLSecDSig->sigNode, './ds:SignedInfo/ds:SignatureMethod');
        if (empty($sigMethod)) {
            echo sprintf('Missing SignatureMethod element');
            exit();
        }
        $sigMethod = $sigMethod[0];
        if (!$sigMethod->hasAttribute('Algorithm')) {
            echo sprintf('Missing Algorithm-attribute on SignatureMethod element.');
            exit;
        }
        $algo = $sigMethod->getAttribute('Algorithm');

        if ($key->type === XMLSecurityKey::RSA_SHA1 && $algo !== $key->type) {
            $key = self::castKey($key, $algo);
        }

        /* Check the signature. */
        if (! $objXMLSecDSig->verify($key)) {
            echo sprintf('Unable to validate Sgnature');
            exit;
        }
    }

    public static function castKey(XMLSecurityKey $key, $algorithm, $type = 'public')
    {
        // do nothing if algorithm is already the type of the key
        if ($key->type === $algorithm) {
            return $key;
        }

        $keyInfo = openssl_pkey_get_details($key->key);
        if ($keyInfo === FALSE) {
            echo sprintf('Unable to get key details from XMLSecurityKey.');
            exit;
        }
        if (!isset($keyInfo['key'])) {
            echo sprintf('Missing key in public key details.');
            exit;
        }

        $newKey = new XMLSecurityKey($algorithm, array('type'=>$type));
        $newKey->loadKey($keyInfo['key']);

        return $newKey;
    }

    public static function processResponse($currentURL, $certFingerprint, $signatureData,
        SAML2_Response $response) {

        /* Validate Response-element destination. */
        $msgDestination = $response->getDestination();
        if ($msgDestination !== NULL && $msgDestination !== $currentURL) {
            echo sprintf('Destination in response doesn\'t match the current URL. Destination is "' .
              xss::filter($msgDestination) . '", current URL is "' . xss::filter($currentURL) . '".');
            exit;
        }

        $responseSigned = self::checkSign($certFingerprint, $signatureData);

        /* Returning boolean $responseSigned */
        return $responseSigned;
    }

	public static function signXML($binding_type, $xml, $publicCert, $privateKey,$signed_slo_response=FALSE) {
        if ($binding_type == "HTTP-Redirect")
        {
            $xml = urldecode($xml);
            $xml = base64_decode($xml);
            $xml = gzinflate($xml);
        }
		$document = new DOMDocument();
		$xml = str_replace("&","&amp;",$xml);
		$document->loadXML($xml);
		$element = $document->firstChild;

    if( $signed_slo_response == TRUE && $privateKey != "")
    {
      $param =array( 'type' => 'private');
      $key = new XMLSecurityKey(XMLSecurityKey::RSA_SHA256, $param);
      $key->loadKey($privateKey);

      if( !empty($insertBeforeTagName) ) {
        $domNode = $document->getElementsByTagName( $insertBeforeTagName )->item(0);

        self::insertSignature($key, array ( $publicCert ), $element, $domNode);
      } else {
        self::insertSignature($key, array ( $publicCert ), $element);
      }
    }

		$requestXML = $element->ownerDocument->saveXML($element);
		$base64EncodedXML = base64_encode($requestXML);
		return $base64EncodedXML;
	}

	public static function insertSignature(XMLSecurityKey $key,array $certificates,DOMElement $root = NULL,DOMNode $insertBefore = NULL) {
        $objXMLSecDSig = new XMLSecurityDSig();
        $objXMLSecDSig->setCanonicalMethod(XMLSecurityDSig::EXC_C14N);

        switch ($key->type) {
            case XMLSecurityKey::RSA_SHA256:
                $type = XMLSecurityDSig::SHA256;
                break;
            case XMLSecurityKey::RSA_SHA384:
                $type = XMLSecurityDSig::SHA384;
                break;
            case XMLSecurityKey::RSA_SHA512:
                $type = XMLSecurityDSig::SHA512;
                break;
            default:
                $type = XMLSecurityDSig::SHA1;
        }

        $objXMLSecDSig->addReferenceList(
            array($root),
            $type,
            array('http://www.w3.org/2000/09/xmldsig#enveloped-signature', XMLSecurityDSig::EXC_C14N),
            array('id_name' => 'ID', 'overwrite' => FALSE)
        );

        $objXMLSecDSig->sign($key);

        foreach ($certificates as $certificate) {
            $objXMLSecDSig->add509Cert($certificate, TRUE);
        }

        $objXMLSecDSig->insertSignature($root, $insertBefore);
    }

    public static function processRequest($certFingerprint, $signatureData) {

        $responseSigned = self::checkSign($certFingerprint, $signatureData);

        /* Returning boolean $responseSigned */
        return $responseSigned;
    }

    public static function checkSign($certFingerprint, $signatureData) {
        $certificates = $signatureData['Certificates'];

        if (count($certificates) === 0) {
            return FALSE;
        }

        $fpArray = array();
        $fpArray[] = $certFingerprint;
        $pemCert = self::findCertificate($fpArray, $certificates);

        $lastException = NULL;

        $key = new XMLSecurityKey(XMLSecurityKey::RSA_SHA1, array('type'=>'public'));
        $key->loadKey($pemCert);

        try {
            /*
             * Make sure that we have a valid signature
             */
            self::validateSignature($signatureData, $key);
            return TRUE;
        } catch (Exception $e) {
            $lastException = $e;
        }


        /* We were unable to validate the signature with any of our keys. */
        if ($lastException !== NULL) {
            throw $lastException;
        } else {
            return FALSE;
        }

    }

    public static function validateIssuerAndAudience($samlResponse, $spEntityId, $issuerToValidateAgainst) {
        $issuer = current($samlResponse->getAssertions())->getIssuer();
        $audience = current(current($samlResponse->getAssertions())->getValidAudiences());
        if(strcmp($issuerToValidateAgainst, $issuer) === 0) {
            if(strcmp($audience, $spEntityId) === 0) {
                return TRUE;
            } else {
                echo sprintf('Invalid audience');
                exit;
            }
        } else {
            echo sprintf('Issuer cannot be verified.');
            exit;
        }
    }

    private static function findCertificate(array $certFingerprints, array $certificates) {

        $candidates = array();

        foreach ($certificates as $cert) {
            $fp = strtolower(sha1(base64_decode($cert)));
            if (!in_array($fp, $certFingerprints, TRUE)) {
                $candidates[] = $fp;
                continue;
            }

            /* We have found a matching fingerprint. */
            $pem = "-----BEGIN CERTIFICATE-----\n" .
                chunk_split($cert, 64) .
                "-----END CERTIFICATE-----\n";

            return $pem;
        }
        $correct_cert = self::sanitize_certificate(\Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_publ_certificate'));
        self::miniornage_show_SAML_msg('Unable to find a certificate matching the configured fingerprint.',$correct_cert,'Certificate mismatch, please use above certificate for configuration.');
    }

        /**
     * Decrypt an encrypted element.
     *
     * This is an internal helper function.
     *
     * @param  DOMElement     $encryptedData The encrypted data.
     * @param  XMLSecurityKey $inputKey      The decryption key.
     * @param  array          &$blacklist    Blacklisted decryption algorithms.
     * @return DOMElement     The decrypted element.
     * @throws Exception
     */
    private static function doDecryptElement(DOMElement $encryptedData, XMLSecurityKey $inputKey, array &$blacklist)
    {
        $enc = new XMLSecEnc();
        $enc->setNode($encryptedData);

        $enc->type = $encryptedData->getAttribute("Type");
        $symmetricKey = $enc->locateKey($encryptedData);
        if (!$symmetricKey) {
            echo sprintf('Could not locate key algorithm in encrypted data.');
            exit;
        }

        $symmetricKeyInfo = $enc->locateKeyInfo($symmetricKey);
        if (!$symmetricKeyInfo) {
            echo sprintf('Could not locate <dsig:KeyInfo> for the encrypted key.');
            exit;
        }
        $inputKeyAlgo = $inputKey->getAlgorithm();
        if ($symmetricKeyInfo->isEncrypted) {
            $symKeyInfoAlgo = $symmetricKeyInfo->getAlgorithm();
            if (in_array($symKeyInfoAlgo, $blacklist, TRUE)) {
                echo sprintf('Algorithm disabled: ' . xss::filter(var_export($symKeyInfoAlgo, TRUE)));
                exit;
            }
            if ($symKeyInfoAlgo === XMLSecurityKey::RSA_OAEP_MGF1P && $inputKeyAlgo === XMLSecurityKey::RSA_1_5) {
                /*
                 * The RSA key formats are equal, so loading an RSA_1_5 key
                 * into an RSA_OAEP_MGF1P key can be done without problems.
                 * We therefore pretend that the input key is an
                 * RSA_OAEP_MGF1P key.
                 */
                $inputKeyAlgo = XMLSecurityKey::RSA_OAEP_MGF1P;
            }
            /* Make sure that the input key format is the same as the one used to encrypt the key. */
            if ($inputKeyAlgo !== $symKeyInfoAlgo) {
                echo sprintf( 'Algorithm mismatch between input key and key used to encrypt ' .
                    ' the symmetric key for the message. Key was: ' .
                  xss::filter(var_export($inputKeyAlgo, TRUE)) . '; message was: ' .
                  xss::filter(var_export($symKeyInfoAlgo, TRUE)));
                exit;
            }
            /** @var XMLSecEnc $encKey */
            $encKey = $symmetricKeyInfo->encryptedCtx;
            $symmetricKeyInfo->key = $inputKey->key;
            $keySize = $symmetricKey->getSymmetricKeySize();
            if ($keySize === NULL) {
                /* To protect against "key oracle" attacks, we need to be able to create a
                 * symmetric key, and for that we need to know the key size.
                 */
                echo sprintf('Unknown key size for encryption algorithm: ' . xss::filter(var_export($symmetricKey->type, TRUE)));
                exit;
            }
            try {
                $key = $encKey->decryptKey($symmetricKeyInfo);
                if (strlen($key) != $keySize) {
                    echo sprintf('Unexpected key size (' . xss::filter(strlen($key) * 8) . 'bits) for encryption algorithm: ' .
                      xss::filter(var_export($symmetricKey->type, TRUE)));
                    exit;
                }
            } catch (Exception $e) {
                /* We failed to decrypt this key. Log it, and substitute a "random" key. */

                /* Create a replacement key, so that it looks like we fail in the same way as if the key was correctly padded. */
                /* We base the symmetric key on the encrypted key and private key, so that we always behave the
                 * same way for a given input key.
                 */
                $encryptedKey = $encKey->getCipherValue();
                $pkey = openssl_pkey_get_details($symmetricKeyInfo->key);
                $pkey = sha1(serialize($pkey), TRUE);
                $key = sha1($encryptedKey . $pkey, TRUE);
                /* Make sure that the key has the correct length. */
                if (strlen($key) > $keySize) {
                    $key = substr($key, 0, $keySize);
                } elseif (strlen($key) < $keySize) {
                    $key = str_pad($key, $keySize);
                }
            }
            $symmetricKey->loadkey($key);
        } else {
            $symKeyAlgo = $symmetricKey->getAlgorithm();
            /* Make sure that the input key has the correct format. */
            if ($inputKeyAlgo !== $symKeyAlgo) {
                echo sprintf( 'Algorithm mismatch between input key and key in message. ' .
                    'Key was: ' . xss::filter(var_export($inputKeyAlgo, TRUE)) . '; message was: ' .
                  xss::filter(var_export($symKeyAlgo, TRUE)));
                exit;
            }
            $symmetricKey = $inputKey;
        }
        $algorithm = $symmetricKey->getAlgorithm();
        if (in_array($algorithm, $blacklist, TRUE)) {
            echo sprintf('Algorithm disabled: ' . xss::filter(var_export($algorithm, TRUE)));
            exit;
        }
        /** @var string $decrypted */
        $decrypted = $enc->decryptNode($symmetricKey, FALSE);
        /*
         * This is a workaround for the case where only a subset of the XML
         * tree was serialized for encryption. In that case, we may miss the
         * namespaces needed to parse the XML.
         */
        $xml = '<root xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" '.
                     'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">' .
            $decrypted .
            '</root>';
        $newDoc = new DOMDocument();
        if (!@$newDoc->loadXML($xml)) {
            echo sprintf('Failed to parse decrypted XML. Maybe the wrong sharedkey was used?');
            throw new Exception('Failed to parse decrypted XML. Maybe the wrong sharedkey was used?');
        }
        $decryptedElement = $newDoc->firstChild->firstChild;
        if ($decryptedElement === NULL) {
            echo sprintf('Missing encrypted element.');
            throw new Exception('Missing encrypted element.');
        }

        if (!($decryptedElement instanceof DOMElement)) {
            echo sprintf('Decrypted element was not actually a DOMElement.');
        }

        return $decryptedElement;
    }
/**
    * @param string $data - the key=value pairs separated with &
    * @return string
    */

public static function encrypt_data($data, $key) {
        $key    = openssl_digest($key, 'sha256');
        $method = 'AES-128-CBC';
        $ivSize = openssl_cipher_iv_length($method);
        $iv     = openssl_random_pseudo_bytes($ivSize);
        $strCrypt = openssl_encrypt ($data, $method, $key,OPENSSL_RAW_DATA||OPENSSL_ZERO_PADDING, $iv);
        return base64_encode($iv.$strCrypt);
    }
/**
    * @param string $data - crypt response from Sagepay
    * @return string
    */
    public static function decrypt_data($data, $key) {
        $strIn = base64_decode($data);
        $key    = openssl_digest($key, 'sha256');
        $method = 'AES-128-CBC';
        $ivSize = openssl_cipher_iv_length($method);
        $iv     = substr($strIn,0,$ivSize);
        $data   = substr($strIn,$ivSize);
        $clear  = openssl_decrypt ($data, $method, $key, OPENSSL_RAW_DATA||OPENSSL_ZERO_PADDING, $iv);

        return $clear;
    }

    /**
     * Decrypt an encrypted element.
     *
     * @param  DOMElement     $encryptedData The encrypted data.
     * @param  XMLSecurityKey $inputKey      The decryption key.
     * @param  array          $blacklist     Blacklisted decryption algorithms.
     * @return DOMElement     The decrypted element.
     * @throws Exception
     */
    public static function decryptElement(DOMElement $encryptedData, XMLSecurityKey $inputKey, array $blacklist = array(), XMLSecurityKey $alternateKey = NULL)
    {
        try {
            echo "trying primary";
            return self::doDecryptElement($encryptedData, $inputKey, $blacklist);
        } catch (Exception $e) {
            //Try with alternate key
            try {
                echo "trying secondary";
                return self::doDecryptElement($encryptedData, $alternateKey, $blacklist);
            } catch(Exception $t) {

            }
            /*
             * Something went wrong during decryption, but for security
             * reasons we cannot tell the user what failed.
             */
            echo sprintf('Failed to decrypt XML element.');
            exit;
        }
    }

    /**
     * Parse a boolean attribute.
     *
     * @param  \DOMElement $node          The element we should fetch the attribute from.
     * @param  string     $attributeName The name of the attribute.
     * @param  mixed      $default       The value that should be returned if the attribute doesn't exist.
     * @return bool|mixed The value of the attribute, or $default if the attribute doesn't exist.
     * @throws \Exception
     */
    public static function parseBoolean(DOMElement $node, $attributeName, $default = null)
    {
        if (!$node->hasAttribute($attributeName)) {
            return $default;
        }
        $value = $node->getAttribute($attributeName);
        switch (strtolower($value)) {
            case '0':
            case 'false':
                return FALSE;
            case '1':
            case 'true':
                return TRUE;
            default:
                throw new Exception('Invalid value of boolean attribute ' . var_export($attributeName, TRUE) . ': ' . var_export($value, TRUE));
        }
    }

     /**
     * Generates the metadata of the SP based on the settings
     *
     * @param string    $sp            The SP data
     * @param string    $authnsign     authnRequestsSigned attribute
     * @param string    $wsign         wantAssertionsSigned attribute
     * @param DateTime  $validUntil    Metadata's valid time
     * @param Timestamp $cacheDuration Duration of the cache in seconds
     * @param array     $contacts      Contacts info
     * @param array     $organization  Organization ingo
     *
     * @return string SAML Metadata XML
     */
    public static function metadata_builder($siteUrl)
    {
        $xml = new DOMDocument();
        $url = plugins_url().'/miniorange-saml-20-single-sign-on/sp-metadata.xml';

        $xml->load($url);

        $xpath = new DOMXPath($xml);
        $elements = $xpath->query('//md:EntityDescriptor[@entityID="http://{path-to-your-site}/wp-content/plugins/miniorange-saml-20-single-sign-on/"]');

         if ($elements->length >= 1) {
            $element = $elements->item(0);
            $element->setAttribute('entityID', $siteUrl.'/wp-content/plugins/miniorange-saml-20-single-sign-on/');
        }

        $elements = $xpath->query('//md:AssertionConsumerService[@Location="http://{path-to-your-site}"]');
        if ($elements->length >= 1) {
            $element = $elements->item(0);
            $element->setAttribute('Location', $siteUrl.'/');
        }

        //re-save
        $xml->save(plugins_url()."/miniorange-saml-20-single-sign-on/sp-metadata.xml");
    }


    public static function get_mapped_groups($saml_params, $saml_groups)
    {
            $groups = array();

        if (!empty($saml_groups)) {
            $saml_mapped_groups = array();
            $i=1;
            while ($i < 10) {
                $saml_mapped_groups_value = $saml_params->get('group'.$i.'_map');

                $saml_mapped_groups[$i] = explode(';', $saml_mapped_groups_value);
                $i++;
            }
        }

        foreach ($saml_groups as $saml_group) {
            if (!empty($saml_group)) {
                $i = 0;
                $found = false;

                while ($i < 9 && !$found) {
                    if (!empty($saml_mapped_groups[$i]) && in_array($saml_group, $saml_mapped_groups[$i])) {
                        $groups[] = $saml_params->get('group'.$i);
                        $found = TRUE;
                    }
                    $i++;
                }
            }
        }

        return array_unique($groups);
    }


    public static function getEncryptionAlgorithm($method){
        switch($method){
            case 'http://www.w3.org/2001/04/xmlenc#tripledes-cbc':
                return XMLSecurityKey::TRIPLEDES_CBC;
                break;

            case 'http://www.w3.org/2001/04/xmlenc#aes128-cbc':
                return XMLSecurityKey::AES128_CBC;

            case 'http://www.w3.org/2001/04/xmlenc#aes192-cbc':
                return XMLSecurityKey::AES192_CBC;
                break;

            case 'http://www.w3.org/2001/04/xmlenc#aes256-cbc':
                return XMLSecurityKey::AES256_CBC;
                break;

            case 'http://www.w3.org/2001/04/xmlenc#rsa-1_5':
                return XMLSecurityKey::RSA_1_5;
                break;

            case 'http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p':
                return XMLSecurityKey::RSA_OAEP_MGF1P;
                break;

            case 'http://www.w3.org/2000/09/xmldsig#dsa-sha1':
                return XMLSecurityKey::DSA_SHA1;
                break;

            case 'http://www.w3.org/2000/09/xmldsig#rsa-sha1':
                return XMLSecurityKey::RSA_SHA1;
                break;

            case 'http://www.w3.org/2001/04/xmldsig-more#rsa-sha256':
                return XMLSecurityKey::RSA_SHA256;
                break;

            case 'http://www.w3.org/2001/04/xmldsig-more#rsa-sha384':
                return XMLSecurityKey::RSA_SHA384;
                break;

            case 'http://www.w3.org/2001/04/xmldsig-more#rsa-sha512':
                return XMLSecurityKey::RSA_SHA512;
                break;

            default:
                echo sprintf('Invalid Encryption Method: '. xss::filter($method));
                exit;
                break;
        }
    }

    public static function sanitize_certificate( $certificate ) {

        $certificate = preg_replace("/[\r\n]+/", "", $certificate);
        $certificate = str_replace( "-", "", $certificate );
        $certificate = str_replace( "BEGIN CERTIFICATE", "", $certificate );
        $certificate = str_replace( "END CERTIFICATE", "", $certificate );
        $certificate = str_replace( " ", "", $certificate );
        $certificate = chunk_split($certificate, 64, "\r\n");
        $certificate = "-----BEGIN CERTIFICATE-----\r\n" . $certificate . "-----END CERTIFICATE-----";
        return $certificate;
    }

    public static function sanitize_private_key( $private_key ) {
        $private_key = preg_replace("/[\r\n]+/", "", $private_key);
        $private_key = str_replace( "-", "", $private_key );
        $private_key = str_replace( "BEGIN PRIVATE KEY", "", $private_key );
        $private_key = str_replace( "END PRIVATE KEY", "", $private_key );
        $private_key = str_replace( " ", "", $private_key );
        $private_key = chunk_split($private_key, 64, "\r\n");
        $private_key = "-----BEGIN PRIVATE KEY-----\r\n" . $private_key . "-----END PRIVATE KEY-----";

        return $private_key;
    }

    public static function desanitize_certificate( $certificate ) {
        $certificate = preg_replace("/[\r\n]+/", "", $certificate);
        $certificate = str_replace( "-----BEGIN CERTIFICATE-----", "", $certificate );
        $certificate = str_replace( "-----END CERTIFICATE-----", "", $certificate );
        $certificate = str_replace( " ", "", $certificate );
        return $certificate;
    }

    public static function miniornage_show_SAML_msg( $msg1, $msg2, $msg3 = '-' ) {
      echo '<div style="font-family:Calibri;padding:0 3%;">';
      echo '<div style="color: #a94442;background-color: #f2dede;padding: 15px;margin-bottom: 20px;text-align:center;border:1px solid #E6B3B2;font-size:18pt;"> ERROR</div>
                              <div style="color: #a94442;font-size:14pt; margin-bottom:20px;"><p><strong>Error: </strong>'.xss::filter($msg1).'</p>
                                  <p>'.xss::filter($msg2).'</p>
                                  <p><strong>Possible Cause: </strong>'.xss::filter($msg3).'</p>
                              </div>
                              <div style="margin:3%;display:block;text-align:center;"></div>
                              <div style="margin:3%;display:block;text-align:center;">
                                  <input style="padding:1%;width:100px;background: #0091CD none repeat scroll 0% 0%;cursor: pointer;font-size:15px;border-width: 1px;border-style: solid;border-radius: 3px;white-space: nowrap;box-sizing: border-box;border-color: #0073AA;box-shadow: 0px 1px 0px rgba(120, 200, 230, 0.6) inset;color: #FFF;"type="button" value="Done" onClick="self.close();">
                              </div>';
      exit;
    }

  public static function miniorange_serialize_user_attributes($post_arr,$name,$value){
    /* Updated User Profile Attributes*/

    $user_profile_attr_names = array_key_exists($name, $post_arr) ? $post_arr[$name] : array();
    $user_profile_attr_values = array_key_exists($value, $post_arr) ? $post_arr[$value] : array();
    $custom_fields = self::customUserFields();

    $attribute_mapping = array();
    foreach( $user_profile_attr_names as $key => $value ) {
      if( !empty( trim( $value ) )  ){
        if( !empty( trim( $user_profile_attr_values[$key] ) ) ){
          $anArray = array();
          if ( isset($custom_fields[$user_profile_attr_values[$key]]) ) {
            $anArray['attr_name'] = trim($value);
            $anArray['attr_value'] = trim($custom_fields[$user_profile_attr_values[$key]]);
            array_push($attribute_mapping, $anArray);
          }
        }
      }
    }
    $attribute_mapping = serialize($attribute_mapping);

    return $attribute_mapping;
  }

    public static function miniorange_serialize_const_attributes($post_arr,$name,$value){
        /* Updated User Profile Attributes*/

        $user_profile_attr_names = array_key_exists($name, $post_arr) ? $post_arr[$name] : array();
        $user_profile_attr_values = array_key_exists($value, $post_arr) ? $post_arr[$value] : array();

        $attribute_mapping = array();
        foreach( $user_profile_attr_names as $key => $value ) {
            if( !empty( trim( $value ) ) ){
                if( !empty( trim( $user_profile_attr_values[$key] ) ) ){
                    $anArray = array();
                    $anArray['attr_name'] = trim( $value );
                    $anArray['attr_value'] = trim( $user_profile_attr_values[$key] );
                    array_push($attribute_mapping, $anArray );
                }
            }
        }
        $attribute_mapping = serialize($attribute_mapping);
        return $attribute_mapping;
    }


  public static function generateCertificate($dn, $digest_algo, $private_key_bits, $valid_days)
    {
        $opensslConfigPath = dirname(dirname(__FILE__)) . DIRECTORY_SEPARATOR . 'resources' . DIRECTORY_SEPARATOR . 'openssl.cnf';

        $config = array(
            "config" => $opensslConfigPath,
            "digest_alg" => $digest_algo,
            "private_key_bits" => $private_key_bits,
            "private_key_type" => OPENSSL_KEYTYPE_RSA,
        );

      $private_key = openssl_pkey_new($config);
      if( strlen($dn['countryName'])<=2)
        $csr = openssl_csr_new($dn, $private_key, $config);
      if(empty($csr) || $csr == NULL){
        drupal_set_message(t('<b>Please provide a valid country code.</b>'),'error');
        return false;
      }
      $csr_signed = openssl_csr_sign($csr, null, $private_key, $valid_days, $config, time());

      openssl_x509_export($csr_signed, $public_key_out);
      openssl_pkey_export($private_key, $private_key_out,null,$config);
      openssl_csr_export($csr, $csrout);


      while (($e = openssl_error_string()) !== false) {
            error_log($e);
        }

        $certificates = array(
            'public_key' => $public_key_out,
            'private_key' => $private_key_out,
        );

        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_publ_certificate', $certificates['public_key'])->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_private_certificate', $certificates['private_key'])->save();

        return true;
    }

    public static function lexpireinfo($datediff)
    {
      global $base_url;
      $site_name = $base_url;
      $info = "Hello,<br /><br />This email is to notify you that your 1 year license for Drupal SAML IDP Single Sign-On Module will expire in <b>". $datediff ."days</b>.
            The module will stop working after the licensed period expires.<br /><br />You will need to renew your license to continue using the module on your website: <b>". $site_name ."</b>.
            <br /><br />
            <br />Thanks,<br />miniOrange Team";

      return $info;
    }

    public static function gperiod($exp_date)
    {
        global $base_url;
        $site_name = $base_url;
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $content = "Hello,<br /><br />Your 1 year license for Drupal IDP SAML Single Sign-On Module for your website <b>" . $site_name . "</b> expired on<b> " . $exp_date . "</b>.
            If your license is not renewed soon, the module will be deactivated.<br /><br />Click <a href='". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/login?username=". $username ."&redirectUrl=". MiniorangeSAMLIdpConstants::BASE_URL ."/moas/initializepayment&requestOrigin=renew_user_license'>here</a> to renew the license.<br />
            <br />Thanks,<br />miniOrange Team";
        return $content;
    }

    public static function grcperiodend($exp_date)
    {
      global $base_url;
      $site_name = $base_url;
      $content="Hello,<br /><br />Your 1 year license for Drupal IDP SAML Single Sign-On Module has expired on <b>".$exp_date."</b>.
            You have not renewed your license within the 15 days grace period given to you. The module has been deactivated on your website <b>". $site_name ."</b>.
            <br /><br /><br />Thanks,<br />miniOrange Team" ;
      return $content;
    }

  public static function createUpdateUrl($code,$licensePlanName,$licenseType,$apiKey,$customerId,$hostName){
    $plaintext = "1111111111111111".$code;
    $url = $hostName.'/moas/api/plugin/drupalJoomlaUpdate/'.$customerId.'/'.$licensePlanName.'/'.$licenseType.'/';
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext = openssl_encrypt($plaintext, $cipher, $apiKey, $options=OPENSSL_RAW_DATA, $iv);
    return $url.str_replace(['+','/','='], ['-','_',''], base64_encode($ciphertext));
  }

  public  static function getSamlRequestPlanName(){
      return 'drupal_saml_idp_premium_plan_v8';
  }

  public static function getSamlRequestPlanType(){
      return 'DRUPAL8_SAML_IDP_MODULE';
  }

  public static function customUserFields(){
    $custom_fields = array(''=>'Select Attribute Value');
    $usr = User::load(\Drupal::currentUser()->id());
    $usrVal =  $usr->toArray();
    foreach ($usrVal as $key => $value){
      $custom_fields[$key] = $key;
    }
    return $custom_fields;
  }

  public static function getNumberOfSP(){
    $saml_sp = new MiniorangeSAMLSP();
    $sp_count = $saml_sp->spCount();

    $noOfSPAllowed = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_number_of_sp');

    //Fetching and updating the number of SP allowed if the limit is reached.
    if ($sp_count == $noOfSPAllowed){
      global $base_url;
      $customer = new MiniorangeSAMLCustomer(NULL, NULL, NULL, NULL);
      $content = json_decode( $customer->ccl(), true );

      if ( is_null($content) || $content['status'] != 'SUCCESS' ){
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_idp_server_unreachable', true)->save();
        $response = new RedirectResponse($base_url . '/admin/config/people/miniorange_saml_idp/customer_setup');
        $response->send();
      }

      elseif (isset($content['noOfSP'])){
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_number_of_sp', $content['noOfSP'])->save();
      }
    }
    return \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_number_of_sp');
  }

  public static function addLogger($file,$function,$line,$message){
    if (\Drupal::config('miniorange_saml_idp.settings')->get('mo_saml_idp_debugging_log')) {
      \Drupal::logger('miniorange_saml_idp')->debug('' . date('d F Y H:i:s', time()) . ' ' . $file . ', ' . $function . ' , ' . $line . ': ' . $message);
    }
  }

}
